from spyne.model.primitive import Integer, String, Double, DateTime, Boolean, Time, Date
from spyne.model.enum import Enum
from spyne.model.complex import Iterable, Array, ComplexModel
from spyne.const.xml_ns import DEFAULT_NS

class BaseAif(ComplexModel):
    __namespace__ = DEFAULT_NS

class DictConvertResult(BaseAif):
    result = Boolean
    info = String(encoding='utf8')

class AgerestrictKey(BaseAif):
    alcoholpercent = Double(min_occurs=1, nillable=False)

class Agerestrict(AgerestrictKey):
    age = Integer(min_occurs=1, nillable=False)

class BarcodeKey(BaseAif):
    barcode = String(min_occurs=1, nillable=False, encoding='utf8')

class Barcode(BarcodeKey):
#    __type_name__ = 'AddPriceClass'
    code = String(min_occurs=1, nillable=False, encoding='utf8')
    price = Double(min_occurs=1, nillable=False)
    name = String(encoding='utf8')
    cquant = Double
    measure = Integer
    quantdefault = Double
    aspectvaluesetcode = Integer
    packingmeasure = Integer
    packingprice = Double
    minprice = Double

class ButtonKey(BaseAif):
    code = Integer(min_occurs=1, nillable=False)

class Button(ButtonKey):
    name = String(encoding='utf8')
    tag = String(encoding='utf8')
    rateobject = Integer
    ratetype = Integer
    ratevalue = Double
    maxratevalue = Double
    minratevalue = Double

class CardKey(BaseAif):
    idcard = String(min_occurs=1, nillable=False, encoding='utf8')

class Card(CardKey):
    idcardgroup = Integer
    number = String(min_occurs=1, nillable=False, encoding='utf8')
    blocked = Integer
    validitydatebeg = DateTime
    validitydateend = DateTime
    idclient = String(encoding='utf8')
    cardsum = Double
    lastknownbonusbalance = Double
    lastknownbonusbalancedate = DateTime
    multiplicator = Double
    multiplicatorbeg = DateTime
    multiplicatorend = DateTime
    multiplicatorcurrent = Double
    discountpercent = Double
    pincode = String(encoding='utf8')
    purchases = Integer
    cardstatus = Integer

class CardGroupKey(BaseAif):
    idcardgroup = Integer(min_occurs=1, nillable=False)

class CardGroup(CardGroupKey):
    name = String(encoding='utf8')
    pattern = String(encoding='utf8')
    regexrule = String(encoding='utf8')
    cardmode = Integer
    discounttype = Integer
    discountvalue = Integer
    text = String(encoding='utf8')
    prefixbeg = String(encoding='utf8')
    prefixend = String(encoding='utf8')
    lenbeg = Integer
    lenend = Integer
    inputmask = Integer
    currencycode = Integer
    bonususecase = Integer
    showinfo = Integer
    notaddemptycard = Integer
    printinfo = Integer

class CashUserKey(BaseAif):
    usercode = String(min_occurs=1, nillable=False, encoding='utf8')

class CashUser(CashUserKey):
    login = String(encoding='utf8')
    username = String(encoding='utf8')
    password = String(encoding='utf8')
    blocked = Integer
    userroleids = Array(Integer) # ????????

class CatalogKey(BaseAif):
    catalogid = Integer(min_occurs=1, nillable=False)

class Catalog(CatalogKey):
    catalogname = String(encoding='utf8')

class CatalogAttributeKey(BaseAif):
    catalogid = Integer(min_occurs=1, nillable=False)

class CatalogAttribute(CatalogAttributeKey):
    attribute = Integer
    alcoholpercent = Double
    age = Integer

class ChangeAccountBalanceKey(BaseAif):
    sessionid = String(min_occurs=1, nillable=False)

class ChangeAccountBalance(ChangeAccountBalanceKey):
    accountnumber = String(encoding='utf8')
    cardnumber = String(encoding='utf8')
    changetype = String(encoding='utf8', nillable=False)
    value = Double(nillable=False)
    groupid = String(encoding='utf8')
    begin = DateTime
    end = DateTime
    weight = Integer

class ClientKey(BaseAif):
    idclient = String(min_occurs=1, nillable=False)

class Client(ClientKey):
    name = String(encoding='utf8')
    text = String(encoding='utf8')
    sex = Integer
    birthday = DateTime
    specialdate1 = DateTime
    specialdate2 = DateTime
    specialdate3 = DateTime
    specialdate1name = String(encoding='utf8')
    specialdate2name = String(encoding='utf8')
    specialdate3name = String(encoding='utf8')
    zipcode = String
    address = String(encoding='utf8')
    email = String(encoding='utf8')
    webpage = String(encoding='utf8')
    phonenumber = String(encoding='utf8')
    inn = String
    document = String(encoding='utf8')
    okpo = String
    okpd = String
    occupation = String(encoding='utf8')
    childrencount = Integer
    extendedoptions = String(encoding='utf8')

class CommandKey(BaseAif):
    commandcode = Integer(min_occurs=1, nillable=False)

class Command(CommandKey):
    commandname = String(encoding='utf8')
    description = String(encoding='utf8')
    logmode = Integer

class DeptKey(BaseAif):
    code = Integer(min_occurs=1, nillable=False)

class Dept(DeptKey):
    name = String(encoding='utf8')
    idclient = String(encoding='utf8')
    documentprefix = String(encoding='utf8')

class DiscountActionKey(BaseAif):
    iddiscountaction = Integer(min_occurs=1, nillable=False)

class DiscountAction(DiscountActionKey):
    name = String(encoding='utf8')

class HotKeyKey(BaseAif):
    hotkeycode = Integer(min_occurs=1, nillable=False)

class HotKey(HotKeyKey):
    hotkeyname = String(encoding='utf8')
    inventcodes = Array(String(encoding='utf8'))
    bybarcode = Boolean


################## Invent #####################
class InventKey(BaseAif):
    inventcode = String(min_occurs=1, nillable=False, encoding='utf8')

class RemainsOtions(BaseAif):
    traceremaindepartment = Integer
    modifyremaindepartment = Integer
    traceremainback = Integer
    manualremaintrace = Integer
    traceremainbycode = Integer

class QuantityOptions(BaseAif):
    enabledefaultquantity = Integer
    enablequantitymanual = Integer
    enablequantitybarcode = Integer
    requirequantityscales = Integer
    enablequantitylimit = Integer
    requirequantitymanual = Integer
    requirequantityscales = Integer
    requirequantitybarcode = Integer
    enabledocumentquantitylimit = Integer
    quantitylimit = Integer

class InventItemOptions(BaseAif):
    enablebarcodemanual = Integer
    enablebarcodescanner = Integer
    disableinventsale = Integer
    disableinventback = Integer
    disableinventshow = Integer
    disablebackinsale = Integer
    visualverify = Integer
    ageverify = Integer
    requiresalerestrict = Integer
    enabledepartmentmanual = Integer
    requiredepartmentmanual = Integer
    egaisverify = Integer
    prepackaged = Integer
    nopdfegaisverify = Integer

class PriceOptions(BaseAif):
    enablepricemanual = Integer
    requirepricemanual = Integer
    requireselectprice = Integer
    requiredeferredprice = Integer

class Options(BaseAif):
    remainsoptions = RemainsOtions
    quantityoptions = QuantityOptions
    inventitemoptions = InventItemOptions
    priceoptions = PriceOptions

class InventOptions(InventKey):
    options = Options

class AdditionalPriceKey(BaseAif):
    barcode = String(min_occurs=1, nillable=False, encoding='utf8')
    pricecode = Integer(min_occurs=1, nillable=False)

class AdditionalPrice(AdditionalPriceKey):
    price = Double(min_occurs=1, nillable=False)
    name = String(encoding='utf8')
    documentid = String
    effectivedate = DateTime

class SellRestrictPeriod(BaseAif):
    dateend = Date
    datestart = Date
    dayend = Integer
    daystart = Integer
    timeend = Time
    timestart = Time

class Invent(InventKey):
    barcode = String(min_occurs=1, nillable=False, encoding='utf8')
    name = String(encoding='utf8')
    price = Double
    deptcode = Integer
    cquant = Double
    measurecode = Integer
    inventgroup = String
    barcodes = Array(Barcode)
    minprice = Double
    taxgroupcode = Integer
    extendedoptions = String(encoding='utf8')
    articul = String(encoding='utf8')
    remain = Double
    discautoscheme = Integer
    taracapacity = Double
    lvatcode = Array(Integer)
    defaultquantity = Double
    taramode = Integer
    aspectschemecode = Integer
    aspectvaluesetcode = Integer
    aspectusecase = Integer
    aspectselectionrule = Integer
    remaindate = Date
    age = Integer
    alcoholpercent = Double
    additionalprices = Array(AdditionalPrice)
    sellrestrictperiods = Array(SellRestrictPeriod)
    options = Options
    alctypecode = Integer

class InventCatalog(CatalogKey):
    tmccode = String(min_occurs=1, nillable=False, encoding='utf8')

class InventGroupKey(BaseAif):
    groupcode = String(min_occurs=1, nillable=False, encoding='utf8')

class InventGroup(InventGroupKey):
    parentgroupcode = String(encoding='utf8')
    groupname = String(encoding='utf8')

class KbKeyKey(BaseAif):
    keycode = Integer(min_occurs=1, nillable=False)
    context = String(encoding='utf8')

class KbKey(KbKeyKey):
    cmactioncode = Integer
    keyvalue = String(encoding='utf8')

class LDiscAccumulateKey(BaseAif):
    discscheme = Integer

class LDiscount(BaseAif):
    disccode = Integer(min_occurs=1, nillable=False)
    name = String(encoding='utf8')
    rtext = String(encoding='utf8')
    discperc = Double
    isbegsum = Integer
    begsum = Double
    isendsum = Integer
    endsum = Double
    isbegrcount = Integer
    begrcount = Integer
    isendrcount = Integer
    endrcount = Integer

class LDiscAccumulate(LDiscAccumulateKey):
    discschemename = String(encoding='utf8')
    discount = Array(LDiscount)

class LDiscCardKey(BaseAif):
    cardnumber = String(min_occurs=1, nillable=False, encoding='utf8')

class LDiscCard(LDiscCardKey):
    name = String(encoding='utf8')
    rtext = String(encoding='utf8')
    denycard = Integer
    acdiscscheme = Integer
    cardmode = Integer
    cardsum = Double
    discasheme = Integer
    notaccumulate = Integer
    positionmode = Integer
    ispaycard = Integer
    allpositions = Integer
    msgsheme = Integer
    rcount = Integer
    bonussum = Double
    bonuscode = Integer
    groupcode = Integer
    birthday = DateTime

class LDiscFixedKey(BaseAif):
    disccode = Integer(min_occurs=1, nillable=False)

class LDiscFixed(LDiscFixedKey):
    name = String(encoding='utf8')
    disctype = Integer
    discmode = Integer
    discsize = Double
    rtext = String(encoding='utf8')
    cardpref = String(encoding='utf8')
    receiptmode = Integer
    positionmode = Integer
    cardonly = Integer
    cardsumbeg = Double
    cardsumend = Double

class LDiscSchemeKey(BaseAif):
    schemecode = Integer(min_occurs=1, nillable=False)

class LDiscScheme(LDiscSchemeKey):
    name = String(encoding='utf8')
    iscardscheme = Integer

class LVatKey(BaseAif):
    code = Integer(min_occurs=1, nillable=False)

class LVat(LVatKey):
    name = String(encoding='utf8')
    rate = Double
    mode = Integer

class MCashUserKey(BaseAif):
    code = String(min_occurs=1, nillable=False, encoding='utf8')

class Roleuser(BaseAif):
    rolecode = Integer(min_occurs=1, nillable=False)
    rule = Integer

class MCashUser(MCashUserKey):
    login = String(encoding='utf8')
    name = String(encoding='utf8')
    password = String(encoding='utf8')
    locked = Integer
    roleusers = Array(Roleuser)
    keyposition = Integer

class MeasureKey(BaseAif):
    idmeasure = Integer(min_occurs=1, nillable=False)

class Measure(MeasureKey):
    name = String(encoding='utf8')
    isinteger = Boolean

class PriceKey(BaseAif):
    barcode = String(min_occurs=1, nillable=False, encoding='utf8')
    documentid = String
    doctype = Integer

class Price(PriceKey):
    price = Double(min_occurs=1, nillable=False)
    minprice = Double
    effectivedate = DateTime
    pricetype = Integer

class ShopKey(BaseAif):
    code = String(min_occurs=1, nillable=False, encoding='utf8')

class Shop(ShopKey):
    extendedoptions = String(encoding='utf8')
    title = String(encoding='utf8')

class TaxGroupKey(BaseAif):
    idtaxgroup = Integer

class Tax(BaseAif):
    name = String(encoding='utf8')
    rate = Integer
    changebase = Boolean

class TaxGroup(TaxGroupKey):
    taxes = Array(Tax)

class TimerestrictKey(BaseAif):
    timerestrictid = Integer(min_occurs=1, nillable=False)

class Timerestrict(TimerestrictKey):
    alcoholpercent = Double
    datebeg = Date
    dateend = Date
    daysbeg = Integer
    daysend = Integer
    timebeg = Time
    timeend = Time

class UnitKey(BaseAif):
    unitcode = Integer

class Unit(UnitKey):
    name = String(encoding='utf8')
    fractional = Boolean

class CurrencyKey(BaseAif):
    currencycode = Integer(min_occurs=1, nillable=False)

class CurrencyStatus(BaseAif):
    basic = Boolean
    national = Boolean
    additional = Boolean

class CurrencyMode(BaseAif):
    denypartialsum = Boolean
    denyextrasum = Boolean
    denymakechange = Boolean
    denypartialout = Boolean
    denyinput = Boolean
    opendrawer = Boolean
    dropchange = Boolean
    denyprice = Boolean
    denypaymentforback = Boolean
    denypaymentforsale = Boolean
    printcopyrequired = Boolean

class Currency(CurrencyKey):
    shortname = String(min_occurs=1, nillable=False, encoding='utf8')
    fullname = String(encoding='utf8')
    rate = Double
    mode = CurrencyStatus # ????????????????
    status = CurrencyStatus # ?????????????????
    type = Integer
    ratecb = Integer
    decpoint = Integer
    round = Enum('roundToUp', 'roundMathematical', 'roundMathematicalWithLoss', 'roundToDown', 'roundToDownWithLoss', type_name='CurrencyRound')
    drawer = Boolean
    hidden = Boolean
    operation = Enum('none', 'bonus', 'bank', 'certificate', 'processing', type_name='CurrencyOperation')
    payprocmodule = String(encoding='utf8')

class SoftCheckPosition(BaseAif):
    barcode = String(encoding='utf8')
    quantity = Double
    name = String(encoding='utf8')
    code = String(encoding='utf8')
    articul = String(encoding='utf8')
    measure = Integer
    cquant = Double
    quantitydefault = Double
    quantitylimit = Double
    group = String(encoding='utf8')
    documentdepartament = Integer
    deptnumber = Integer
    deptnumberall = Integer
    price = Double
    priceindexall = Integer
    pricetype = Integer
    pricevalcode = Integer
    minimumprice = Double
    priceselectmode = Integer
    sum = Double
    backmode = Integer
    isnotcalculate = Integer
    withoutcollection = Integer
    useaspects = Integer
    packingmode = Integer
    taraweight = Double
    vatcode1 = Integer
    vatrate1 = Double
    vatsum1 = Double
    vatcode2 = Integer
    vatrate2 = Double
    vatsum2 = Double
    vatcode3 = Integer
    vatrate3 = Double
    vatsum3 = Double
    vatcode4 = Integer
    vatrate4 = Double
    vatsum4 = Double
    vatcode5 = Integer
    vatrate5 = Double
    vatsum5 = Double

class SoftCheckKey(BaseAif):
    softcheckid = String(encoding='utf8')

class SoftCheck(SoftCheckKey):
    options = Integer
    creationdate = DateTime
    positions = Array(SoftCheckPosition)

############# extended objects#################

class DiscountsAutoKey(BaseAif):
    disccode = Integer(min_occurs=1, nillable=False)
    shemecode = Integer(min_occurs=1, nillable=False)

class DiscountsAuto(DiscountsAutoKey):
    name = String(encoding='utf8')
    disctype = Integer
    discmode = Integer
    discsize = Double
    rtext = String(encoding='utf8')
    datebeg = Date
    dateend = Date
    timebeg = Time
    timeend = Time
    daysbeg = Integer
    daysend = Integer
    quantbeg = Double
    quantend = Double
    sumbeg = Double
    sumend = Double
    pricecode = Integer
    rsummode = Integer
    cardgroupcode = Integer
    birthdaymode = Integer
    catalogid = Integer
    catalogquantbeg = Double
    catalogquantend = Double
    catalogsumbeg = Double
    catalogsumend = Double

class HotKeyInvent(InventKey):
    hotkeycode = Integer(min_occurs=1, nillable=False)

class KitKey(BaseAif):
    idkit = Integer(min_occurs=1, nillable=False)

class Kit(KitKey):
    iddiscountaction = Integer(min_occurs=1, nillable=False)
    name = String(min_occurs=1, nillable=False, encoding='utf8')
    kittype = Integer
    maxtimes = Integer

class KitDiscKey(BaseAif):
    idkitdisc = Integer(min_occurs=1, nillable=False)

class KitDisc(KitDiscKey):
    idkitgoods = Integer(min_occurs=1, nillable=False)
    type = Enum('percent', 'summ', type_name='KitDiscType')
    value = Double

class KitInventKey(BaseAif):
    idkitgoods = Integer(min_occurs=1, nillable=False)

class KitInvent(KitInventKey):
    idkit = Integer(min_occurs=1, nillable=False)
    inventcode = String(min_occurs=1, nillable=False, encoding='utf8')
    quantity = Double

class TimePeriodKey(BaseAif):
    idtimeperiod = Integer(min_occurs=1, nillable=False)

class TimePeriod(TimePeriodKey):
    iddiscountaction = Integer(min_occurs=1, nillable=False)
    datestart = Date
    dateend = Date
    timestart = Time
    timeend = Time
    daystart = Integer
    dayend = Integer

class SaleRestrictKey(BaseAif):
    salerestrictcode = Integer(min_occurs=1, nillable=False)

class SaleRestrict(SaleRestrictKey):
    tmccode = String(min_occurs=1, encoding='utf8')
    dateend = Date
    datestart = Date
    dayend = Integer
    daystart = Integer
    timeend = Time
    timestart = Time

class ConsultantKey(BaseAif):
    code = String(min_occurs=1, nillable=False, encoding='utf8')

class Consultant(ConsultantKey):
    password = String(nillable=False, encoding='utf8')
    name = String(encoding='utf8')

class AlcoCode(BaseAif):
    alcocode = String(nillable=False, encoding='utf8')
