# -*- coding: UTF-8 -*-
'''
Created on 12.04.2012

@author: kov & chvv & sdv
'''

import json

'''
|==========================================================================================================|
|                                                ВАЖНО!!!                                                  |
|    Товарищ разработчик!!!!!!                                                                             |
|    После добавления новой команды или изменения существующей добавь изменения в модуль aifSoapClasses.py |
|    Если не понятно как добавлять, то обратись к Вове.                                                    |
|==========================================================================================================|
'''

class BasicValidator(object):
    """
    Базовый валидатор
    проверяет на NULL
    """
    def __init__(self, notNull=False):
        self.notNull = notNull
    def isValid(self, value):
        if value is None and self.notNull:
            return (False, u'не может быть NULL')
        return (True, '')

class StrValidator(BasicValidator):
    """
    Валидатор строки
    умеет проверять строку на пустоту
    """
    def __init__(self, notEmpty=False):
        self.notEmpty = notEmpty
        super(self.__class__, self).__init__(True)
    def isValid(self, value):
        baseValid = super(self.__class__, self).isValid(value)
        if not baseValid[0]:
            return baseValid
        if not isinstance(value, basestring):
            return (False, u'должно быть строкой')
        if self.notEmpty and not value.strip():
            return (False, u'не может быть пустым')
        return (True, '')

class DictValidator(BasicValidator):
    """
    Валидатор словаря
    Проверяет наличие полей и проверяет на валидность вложенные валидаторы
    """
    def __init__(self, items={}):
        super(self.__class__, self).__init__(True)
        self.items = items

    def isValid(self, value):
        baseValid = super(self.__class__, self).isValid(value)
        if not baseValid[0]:
            return baseValid
        if not isinstance(value, dict):
            return (False, u'должно быть словарем')
        for item in self.items:
            childValidator = self.items[item]
            if item not in value:
                return (False, u'отсутствует обязательное поле %s' % item)
            itemValid = childValidator.isValid(value[item])
            if not itemValid[0]:
                delimiter = '.' if isinstance(childValidator, DictValidator) else ' - '
                return (False, u'%s%s%s' % (item, delimiter, itemValid[1]))
        return (True, '')

class AifObjectNotValid(Exception):
    pass

"""
Поддерживаемые объекты (для валидации объектов)
Содержит в качестве ключа команду, в качестве значения параметр, список обязательных полей и флаг пропуска объекта
Вместо списка можно добавить валидатор. Если флаг пропуска True, то объект с ошибкой удаляется из списка
"""
checkObjectsDict = {
    'addTmc':('invent', ['inventcode']),
    'clearTmc':(),
    'deleteTmc':('inventcode', []),
    
    'replaceDiscountSystem':(),
    
    'addCmaction':('cmaction', ['cmactioncode', 'commandcode']),
    'clearCmaction':(),
    'deleteCmaction':('cmactioncode', []),
    
    'addMenu':('menu', ['menucode']),
    'clearMenu':(),
    'deleteMenu':('menucode', []),
    
    'addRole':('role', ['rolecode']),
    'clearRole':(),
    'deleteRole':('rolecode', []),
    
    'addAspectScheme':('aspectscheme', ['aspectschemecode']),
    'clearAspectScheme':(),
    'deleteAspectScheme':('aspectschemecode', []),
    
    'addAspectValueSet':('aspectvalueset', ['aspectschemecode', 'aspectvaluesetcode']),
    'clearAspectValueSet':(),
    'deleteAspectValueSet':('aspectvaluesetcode', []),
    
    'addTmcrequiredquantity':('tmcrequiredquantity', ['tmccode']),
    'clearTmcrequiredquantity':(),
    'deleteTmcrequiredquantity':('tmccode', []),
    
    'addTmcvisualverify':('tmcvisualverify', ['tmccode']),
    'clearTmcvisualverify':(),
    'deleteTmcvisualverify':('tmccode', []),
    
    'addMacro':('macro', ['macrocode']),
    'clearMacro':(),
    'deleteMacro':('macrocode', []),

    'addShopLabels':('group', ['code']),
    'clearShopLabels':(),
    'deleteShopLabels':('groupcode', [])
}

"""
Преобразует Json объект в строку без \u8734 символов
"""
def jsonToStr(jsonObj):
    return json.dumps(jsonObj, ensure_ascii=False)
    
"""
Проверка объектов
Если установлен флаг в checkObjectsDict и валидация не прошла, объект удаляется, для остальных кидается
исключение.
Возвращает количество незагруженных объектов
"""
def checkObjects(listObjects, log):
    invalidObjects = 0
    
    countObjects = len(listObjects)
    currentIndex = 0
    while currentIndex < countObjects:
        aifObj = listObjects[currentIndex]
        if not 'command' in aifObj:
            raise AifObjectNotValid(u"Объект AIF не содержит обязательное поле 'command': %s" % jsonToStr(aifObj))

        command = aifObj['command']
        if not command in checkObjectsDict:
            raise AifObjectNotValid(u"Неизвестная команда '%s' в объекте AIF: %s" % (command, jsonToStr(aifObj)))

        params = checkObjectsDict[command]
        if params:
            try:
                if not params[0] in aifObj:
                    raise AifObjectNotValid(u"Объект AIF не содержит обязательное поле '%s': %s" % (params[0], jsonToStr(aifObj)))
                if isinstance(params[1], BasicValidator):
                    validRes = params[1].isValid(aifObj[params[0]])
                    if not validRes[0]:
                        raise AifObjectNotValid(u"Ошибка валидации %s '%s': %s" % (params[0], validRes[1], jsonToStr(aifObj)))
                else:
                    for field in params[1]:
                        if not field in aifObj[params[0]]:
                            raise AifObjectNotValid(u"Объект AIF не содержит обязательное поле '%s.%s': %s" % (params[0], field, jsonToStr(aifObj)))

            except AifObjectNotValid, e:
                if len(params) > 2 and params[2]:
                    log.error(u"%s" % e)
                    countObjects -= 1
                    invalidObjects += 1
                    del listObjects[currentIndex]
                    continue
                else:
                    raise
        currentIndex += 1
    return invalidObjects

class AIFBase(object):
    def __init__(self, objName=None):
        self.addCommand = None
        self.updateCommand = None
        self.delCommand = None
        self.objName = objName if objName else self.__class__.__name__

    def addCmd(self, **kwargs):
        self.addCommand = {'command': 'add'+self.objName,
                            self.objName.lower(): kwargs }
        return self.addCommand

    def updateCmd(self, **kwargs):
        self.updateCommand = { 'command': 'update'+self.objName,
                                self.objName.lower(): kwargs }
        return self.updateCommand

    def deleteCmd(self, **kwargs):
        return self.deleteCmdByCode(**kwargs)

    def deleteCmdByCode(self, **kwargs):
        self.delCommand = {'command':'delete'+self.objName}
        self.delCommand.update(kwargs)
        return self.delCommand

    def deleteCmdByObj(self, **kwargs):
        self.delCommand = {'command':'delete'+self.objName,
                                self.objName.lower(): kwargs }
        return self.delCommand

    def clearCmd(self):
        return {'command': 'clear'+self.objName}


clearCmaction = {"command": "clearCmaction"}
clearMenu = {"command": "clearMenu"}
clearRole = {"command": "clearRole"}
clearAspectScheme = {"command": "clearAspectScheme"}
clearAspectValueSet = {"command": "clearAspectValueSet"}

class Agerestrict(AIFBase):
    def addCmd(self, alcoholpercent, age):
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def deleteCmd(self, alcoholpercent):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addAgerestrict':('agerestrict', ['alcoholpercent']),
                         'clearAgerestrict':(),
                         'deleteAgerestrict':('alcoholpercent', [])})

#AspectScheme

#AspectValueSet

class Barcode(AIFBase):
    def addCmd(self, code, barcode, price=None, name=None, cquant=None, measure=None, quantdefault=None, aspectvaluesetcode=None,
               packingmeasure=None, packingprice=None, minprice=None):
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def deleteCmd(self, barcode):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addBarcode':('barcode', DictValidator(
                             {'code': StrValidator(True), 
                              'barcode': StrValidator(True)}), True),
                         'clearBarcode':(),
                         'deleteBarcode':('barcode', [])})

class Button(AIFBase):
    def addCmd(self, code, name = None, tag = None, rateobject = None, ratetype = None, ratevalue = None, maxratevalue = None, minratevalue = None):
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def deleteCmd(self, code):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addButton':('button', ['code']),
                         'clearButton':(),
                         'deleteButton':('code', [])})

class Card(AIFBase):
    def addCmd(self, idcard, idcardgroup, number, blocked, idclient=None, validitydatebeg=None, validitydateend=None, cardsum=None, 
               lastknownbonusbalance = None, lastknownbonusbalancedate = None, multiplicator = None, multiplicatorbeg = None, 
               multiplicatorend = None, multiplicatorcurrent = None, discountpercent=None, pincode=None, purchases=None, cardstatus=None):
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def updateCmd(self, idcard = None, number = None, idcardgroup = None, blocked = None, idclient=None, validitydatebeg=None, validitydateend=None, 
                  cardsum=None, lastknownbonusbalance = None, lastknownbonusbalancedate = None, multiplicator = None, multiplicatorbeg = None, 
                  multiplicatorend = None, multiplicatorcurrent = None, discountpercent=None, pincode=None, purchases=None, cardstatus=None):
        if not idcard and not number:
            return
        return AIFBase.updateCmd(self, **getVars(vars()))
    
    def deleteCmd(self, idcard):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addCard':('card', ['idcard']),
                         'clearCard':(),
                         'deleteCard':('idcard', []),
                         'updateCard':('card', ['idcard'])})

class CardGroup(AIFBase):
    def addCmd(self, idcardgroup, name = None, pattern = None, regexrule = None, cardmode = None, discounttype = None, discountvalue = None,
               text = None, prefixbeg = None, prefixend = None, lenbeg = None, lenend = None, inputmask = None, currencycode = None,
               bonususecase = None, showinfo=None, notaddemptycard=None, printinfo=None):
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def deleteCmd(self, idcardgroup):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addCardGroup':('cardgroup', ['idcardgroup']),
                         'clearCardGroup':(),
                         'deleteCardGroup':('idcardgroup', [])})

class CashUser(AIFBase):
    def addCmd(self, usercode, login, username, password = None, blocked = None, userroleids = []):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, usercode):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addCashUser':('cashuser', ['usercode']),
                         'clearCashUser':(),
                         'deleteCashUser':('usercode', [])})

class Catalog(AIFBase):
    def addCmd(self, catalogid, catalogname):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, catalogid):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addCatalog':('catalog', ['catalogid']),
                         'clearCatalog':(),
                         'deleteCatalog':('catalogid', [])})

class CatalogAttribute(AIFBase):
    def addCmd(self, catalogid, attribute, alcoholpercent = None, age = None):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, catalogid):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addCatalogAttribute':('catalogattribute', ['catalogid', 'attribute']),
                         'clearCatalogAttribute':(),
                         'deleteCatalogAttribute':('catalogid', [])})

class ChangeAccountBalance(AIFBase):
    def addCmd(self, sessionid, changetype, value, accountnumber=None, cardnumber=None, groupid=None, begin=None, end=None, weight=None):
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def deleteCmd(self, sessionid):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addChangeAccountBalance':('changeaccountbalance', ['sessionid']),
                         'clearChangeAccountBalance':(),
                         'deleteChangeAccountBalance':('sessionid', [])})

class Client(AIFBase):
    def addCmd(self, idclient=None, name=None,text=None,sex=None,birthday=None,specialdate1=None,specialdate2=None,specialdate3=None,
                 specialdate1name=None,specialdate2name=None,specialdate3name=None,zipcode=None,address=None,email=None,webpage=None,
                 phonenumber=None,inn=None,document=None,okpo=None,okpd=None,occupation=None,childrencount=None,extendedoptions=None):
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def deleteCmd(self, idclient):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addClient':('client', ['idclient']),
                         'clearClient':(),
                         'deleteClient':('idclient', [])})

#Cmaction


class Command(AIFBase):
    def addCmd(self, commandcode, commandname, description = None, logmode = None):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, commandcode):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addCommand':('commandobject', ['commandcode']),
                         'clearCommand':(),
                         'deleteCommand':('commandcode', [])})

class Dept(AIFBase):
    def addCmd(self, code=0, name=None, idclient = None, documentprefix = None):
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def deleteCmd(self, code):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addDept':('dept', ['code']),
                         'clearDept':(),
                         'deleteDept':('code', [])})

# зависимые kit, kitdisc, kitgoods, timeperiod
class DiscountAction(AIFBase):
    def addCmd(self, iddiscountaction, name):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, iddiscountaction): # нет команды удаления
        return #{"command":"deleteDiscountAction", "iddiscountaction": iddiscountaction }
checkObjectsDict.update({'addDiscountAction':('discountaction', ['iddiscountaction']),
                         'clearDiscountAction':()})

class HotKey(AIFBase):
    def addCmd(self, hotkeycode, hotkeyname, inventcodes=[], bybarcode=False):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, hotkeycode):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addHotKey':('hotkey', ['hotkeycode']),
                         'clearHotKey':(),
                         'deleteHotKey':('hotkeycode', [])})

# хорошо бы упростить
class Invent(object):
    def addCmd(self, inventcode, barcode, name, price, deptcode = None, cquant=None, measurecode=None, inventgroup=None, minprice=None, 
                 taxgroupcode=None, extendedoptions=None, articul=None, remain=None, discautoscheme=None, taracapacity=None, lvatcode = None, 
                 defaultquantity = None, taramode = None, aspectschemecode = None, aspectvaluesetcode = None, aspectusecase = None,
                 aspectselectionrule = None, remaindate = None, age = None, alcoholpercent = None, inn = None, kpp = None, barcodes=[], additionalprices = [], 
                 sellrestrictperiods = [], options={}, alctypecode = None):
        return {"command":"addInventItem",
                     "invent": getVars(vars()) }
    
    def deleteCmd(self, inventcode):
        return {"command": "deleteInventItem",
                "inventcode": inventcode}
    
    def clearCmd(self):
        return {"command": "clearInventory"}
checkObjectsDict.update({'addInventItem':('invent', ['inventcode']),
                         'clearInventory':(),
                         'deleteInventItem':('inventcode', []),
                         'updateInventItem':('invent', ['inventcode'])})

class InventCatalog(AIFBase):
    def addCmd(self, tmccode, catalogid):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, tmccode, catalogid):
        return AIFBase.deleteCmdByObj(self, **getVars(vars()))
checkObjectsDict.update({'addInventCatalog':('inventcatalog', ['catalogid', 'tmccode']),
                         'clearInventCatalog':(),
                         'deleteInventCatalog':('inventcatalog', ['catalogid', 'tmccode'])})

class InventGroup(AIFBase):
    def addCmd(self, groupcode, parentgroupcode, groupname):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, groupcode):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addInventGroup':('inventgroup', ['groupcode']),
                         'clearInventGroup':(),
                         'deleteInventGroup':('groupcode', [])})

class KbKey(AIFBase):
    def addCmd(self, keycode, cmactioncode, keyvalue, context = None):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, keycode, context = None):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addKbKey':('kbkey', ['keycode', 'cmactioncode']),
                         'clearKbKey':(),
                         'deleteKbKey':('keycode', [])})

# хорошо бы упростить
class LDiscAccumulate(AIFBase):
    def addCmd(self, discscheme, discschemename, discount = []):
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def addDiscount(self, disccode, name, rtext=None, discperc=None, isbegsum=None, begsum=None, isendsum=None, endsum=None,
                    isbegrcount=None, begrcount=None, isendrcount=None, endrcount=None):
        self.addCommand.get("ldiscaccumulate").get("discount").append(getVars(vars()))
        return self.addCommand
                   
    def deleteCmd(self, discscheme):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addLDiscAccumulate':('ldiscaccumulate', ['discscheme']),
                         'clearLDiscAccumulate':(),
                         'deleteLDiscAccumulate':('discscheme', [])})

class LDiscCard(AIFBase):
    def addCmd(self, cardnumber, name=None, rtext=None, denycard=None, acdiscasheme=None, cardmode=None, cardsum=None, discasheme=None,
                 notaccumulate=None, positionmode=None, ispaycard=None, allpositions=None, msgsheme=None, rcount=None, bonussum=None,
                 bonuscode=None, groupcode=None, birthday=None):
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def deleteCmd(self, cardnumber):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addLDiscCard':('ldisccard', ['cardnumber']),
                         'clearLDiscCard':(),
                         'deleteLDiscCard':('cardnumber', [])})

class LDiscFixed(AIFBase):
    def addCmd(self, disccode, name = None, disctype = None, discmode = None, discsize = None, rtext = None, cardpref = None, receiptmode = None,
               positionmode = None, cardonly = None, cardsumbeg=None, cardsumend=None):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, disccode):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addLDiscFixed':('ldiscfixed', ['disccode']),
                         'clearLDiscFixed':(),
                         'deleteLDiscFixed':('disccode', [])})

# связано с discountsauto
class LDiscScheme(AIFBase):
    def addCmd(self, schemecode, name=None, iscardscheme=None): 
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, schemecode):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addLDiscScheme':('ldiscscheme', ['schemecode']),
                         'clearLDiscScheme':(),
                         'deleteLDiscScheme':('schemecode', [])})

class LVat(AIFBase):
    def addCmd(self, code, name = None, rate = None, mode = None): 
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, code):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addLVat':('lvat', ['code']),
                         'clearLVat':(),
                         'deleteLVat':('code', [])})

# хорошо бы упростить
class MCashUser(AIFBase):
    def addCmd(self, code, login, name, password, locked=0, roleusers = None, keyposition=None):
        if not roleusers:
            roleusers = []
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def addRoleuser(self, rolecode, rule=None):
        self.addCommand["mcashuser"]["roleusers"].append(getVars(vars()))
        return self.addCommand
                   
    def deleteCmd(self, code):
        return {"command": "deleteMCashUser",
                "usercode": code }
checkObjectsDict.update({'addMCashUser':('mcashuser', ['code']),
                         'clearMCashUser':(),
                         'deleteMCashUser':('usercode', [])})

class Measure(AIFBase):
    def addCmd(self, idmeasure, name, isinteger = None):
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def deleteCmd(self, idmeasure):
        return
    
    def clearCmd(self):
        return
checkObjectsDict.update({'addMeasure':('measure', ['idmeasure'])})

#Menu


class Price(AIFBase):
    def addCmd(self, barcode, price, documentid, effectivedate, doctype, minprice=None, pricetype=None):
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def deleteCmd(self, barcode, documentid, doctype):
        return AIFBase.deleteCmd(self, **getVars(vars()))

    def clearCmd(self):
        return
checkObjectsDict.update({'addPrice':('price', ['barcode']),
                         'clearPrice':(),
                         'deletePrice':('barcode', []),
                         'deletePriceByDate':('barcode', [])})

#Role

class Shop(AIFBase):
    def addCmd(self, code, extendedoptions = None, title = None):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, code):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addShop':('shop', ['code']),
                         'clearShop':(),
                         'deleteShop':('code', [])})

# упростить
class TaxGroup(AIFBase):
    def addCmd(self, idtaxgroup, taxes=[]):
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def addTax(self, name, rate = None, changebase = None):
        self.addCommand['taxgroup']['taxes'].append(getVars(vars()))
        return self.addCommand
                   
    def deleteCmd(self):
        return
    
    def clearCmd(self):
        return
checkObjectsDict.update({'addTaxGroup':('taxgroup', ['idtaxgroup'])})

class Timerestrict(AIFBase):
    def addCmd(self, timerestrictid, alcoholpercent, datebeg = None, dateend = None, daysbeg = None, daysend = None, timebeg = None, timeend = None):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, timerestrictid):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addTimerestrict':('timerestrict', ['timerestrictid']),
                         'clearTimerestrict':(),
                         'deleteTimerestrict':('timerestrictid', [])})

class Unit(AIFBase):
    def addCmd(self, unitcode, name = None, fractional = None):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, unitcode):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addUnit':('unit', ['unitcode']),
                         'clearUnit':(),
                         'deleteUnit':('unitcode', [])})

######Currency (Valut) #######
class Currency(AIFBase):
    def addCmd(self, currencycode, shortname, fullname=None, rate = None, mode=None, status={}, type=None, ratecb=None, decpoint=None, round=None,
               drawer=None, hidden=None, operation=None, payprocmodule=None):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, currencycode):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addCurrency':('currency', ['currencycode']),
                         'clearCurrency':(),
                         'deleteCurrency':('currencycode', [])})

############# extended objects#################

class AdditionalPrice(AIFBase):
    def addCmd(self, barcode, pricecode, price, name=None, documentid=None, effectivedate='0000-00-00 00:00:00'):
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def deleteCmd(self, barcode, pricecode):
        return AIFBase.deleteCmdByObj(self, **getVars(vars()))
checkObjectsDict.update({'addAdditionalPrice':('additionalprice', ['barcode']),
                         'clearAdditionalPrice':(),
                         'deleteAdditionalPrice':('additionalprice', ['barcode'])})

class DiscountsAuto(AIFBase):
    def addCmd(self, schemecode, disccode, name='', disctype=None, discmode=None, discsize=None, rtext=None, datebeg=None,
                dateend=None, timebeg=None, timeend=None, daysbeg=None, daysend=None, quantbeg=None, quantend=None, sumbeg=None,
                sumend=None, pricecode=None, rsummode=None, cardgroupcode=None, birthdaymode=None, catalogid=None, 
                catalogquantbeg=None, catalogquantend=None, catalogsumbeg=None, catalogsumend=None):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, disccode, shemecode):
        return AIFBase.deleteCmdByObj(self, **getVars(vars()))
checkObjectsDict.update({'addDiscountsAuto':('discountsauto', ['disccode']),
                         'clearDiscountsAuto':(),
                         'deleteDiscountsAuto':('discountsauto', ['disccode'])})

class HotKeyInvent(AIFBase):
    def addCmd(self, hotkeycode, inventcode):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, hotkeycode, inventcode):
        return AIFBase.deleteCmdByObj(self, **getVars(vars()))
checkObjectsDict.update({'addHotKeyInvent':('hotkeyinvent', ['hotkeycode', 'inventcode']),
                         'clearHotKeyInvent':(),
                         'deleteHotKeyInvent':('hotkeyinvent', ['hotkeycode'])})

class Kit(AIFBase):
    def addCmd(self, iddiscountaction, idkit, name, kittype=None, maxtimes=None):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, idkit):
        return AIFBase.deleteCmdByObj(self, **getVars(vars()))
checkObjectsDict.update({'addKit':('kit', ['idkit', 'iddiscountaction']),
                         'clearKit':(),
                         'deleteKit':('kit', ['idkit'])})

class KitDisc(AIFBase):
    def addCmd(self, idkitdisc, idkitgoods, type, value):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, idkitdisc):
        return AIFBase.deleteCmdByObj(self, **getVars(vars()))
checkObjectsDict.update({'addKitDisc':('kitdisc', ['idkitdisc', 'idkitgoods']),
                         'clearKitDisc':(),
                         'deleteKitDisc':('kitdisc', ['idkitdisc'])})

class KitInvent(AIFBase):
    def addCmd(self, idkitgoods, idkit, inventcode, quantity):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, idkitgoods):
        return AIFBase.deleteCmdByObj(self, **getVars(vars()))
checkObjectsDict.update({'addKitInvent':('kitinvent', ['idkitgoods']),
                         'clearKitInvent':(),
                         'deleteKitInvent':('kitinvent', ['idkitgoods'])})

class TimePeriod(AIFBase):
    def addCmd(self, idtimeperiod, iddiscountaction, datestart=None, dateend=None, timestart=None, timeend=None, daystart=None, dayend=None):
        return AIFBase.addCmd(self, **getVars(vars()))
                   
    def deleteCmd(self, idtimeperiod):
        return AIFBase.deleteCmdByObj(self, **getVars(vars()))
checkObjectsDict.update({'addTimePeriod':('timeperiod', ['idtimeperiod']),
                         'clearTimePeriod':(),
                         'deleteTimePeriod':('timeperiod', ['idtimeperiod'])})

class Certificate(AIFBase):
    def addCmd(self, code, rangefrom, rangeto, inputmask, name=None, sum=None, fullyspend=None, skiplastsymbol=None):
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def deleteCmd(self, code):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addCertificate':('certificate', ['code']), 
                         'clearCertificate':(), 
                         'deleteCertificate':('code', [])})


class SaleRestrict(AIFBase):
    def addCmd(self, salerestrictcode, tmccode, datestart=None, dateend=None, daystart=None, dayend=None, timestart=None, timeend=None):
        return AIFBase.addCmd(self, **getVars(vars()))

    def deleteCmd(self, salerestrictcode):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addSaleRestrict': ('salerestrict', ['salerestrictcode']),
                         'clearSaleRestrict': (),
                         'deleteSaleRestrict': ('salerestrictcode', [])})


class Consultant(AIFBase):
    def addCmd(self, code=None, password=None, name=None):
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def deleteCmd(self, code):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addConsultant':('consultant', ['code','password']),
                         'clearConsultant':(),
                         'deleteConsultant':('code', [])})

class SoftCheck(AIFBase):
    def addCmd(self, softcheckid=None, options=None, positions=None, creationdate=None):
        return AIFBase.addCmd(self, **getVars(vars()))
    
    def deleteCmd(self, softcheckid):
        return AIFBase.deleteCmd(self, **getVars(vars()))

checkObjectsDict.update({'addSoftCheck':('softcheck', ['softcheckid', 'options', 'positions']),
                         'clearSoftCheck':(),
                         'deleteSoftCheck':('softcheckid', [])})

class AlcoCode(AIFBase):
    def addCmd(self, alcocode):
        return AIFBase.addCmd(self, **getVars(vars()))

    def deleteCmd(self, alcocode):
        return AIFBase.deleteCmd(self, **getVars(vars()))
checkObjectsDict.update({'addAlcoCode':('alcocode', ['alcocode']),
                         'clearAlcoCode':(),
                         'deleteAlcoCode':('alcocode', [])})

# проверить существование таких комманды
class AddSellRestrictPeriods(object):
    def __init__(self, inventcode, sellrestrictperiods):
        self.dict = {"command": "updateInventItem",
                     "invent": getVars(vars()) }

class AddInventOptions(object):
    def __init__(self, inventcode, options):
        self.dict = {"command": "updateInventItem",
                     "invent": getVars(vars()) }
        
class AddRemain(object):
    def __init__(self, inventcode, barcode, date, remain, deptcode):
        self.dict = {"command": "updateInventItem",
                     "invent": getVars(vars()) }

class UpdateInventAttr(object):
    def __init__(self, inventcode):
        self.dict = {"command": "updateInventItem",
                     "invent": getVars(vars()) }    

def getVars(args):
    del(args['self'])
    dict = args.copy()
    for key in args:
        if dict[key] == None:
            del(dict[key])
    return dict
