# -*- coding: UTF-8 -*-

import httplib2
import json
from common import logger
import os

STATUS_ERROR = 'STATUS_ERROR'                       # Ошибка загрузки/выгрузки справочника
STATUS_READY = 'STATUS_READY'                       # Справочник успешно загружен/выгружен
STATUS_READY_WITH_ERROR = 'STATUS_READY_WITH_ERROR' # Справочник загружен с ошибками
STATUS_DURING = 'STATUS_DURING'                     # Справочник в данный момент загружается/выгружается

REST_URL = 'http://localhost:8080/CSrest/rest'
SALES_BACKOFFICE_URL = REST_URL + '/sales/backOfficeState'
DISTS_BACKOFFICE_URL = REST_URL + '/dicts/backOfficeState'
SALES_CASHUNLOAD_URL = '/CSrest/rest/sales/cashUnloadState'

http = httplib2.Http(timeout=3)
jsonHeader = {'Content-Type':'application/json', "Accept": "application/json"}
formats = {'aif':'ARTIXINTERCHANGE', 'shtrih':'SHTRIHM', 'smag-ukm':'SUPERMAG', 'smag-ukm4':'SUPERMAG4', 'oracle':'ORACLE', 'atol':'ATOL', 'soap':'ARTIXINTERCHANGE_SOAP', 'crystal':'CRYSTAL', 'rest':'ARTIXINTERCHANGE_REST', 'ns2000':'NS2000','pilot':'PILOT', 'simple_file': 'SIMPLE-FILE'}

def convertFileName(fileName):
    if isinstance(fileName, str):
        return os.path.basename(fileName)
    if isinstance(fileName, list):
        result = []
        for f in fileName:
            result.append(os.path.basename(f))
        return str(result)
    return None

def sendDictState(node, messageEn, messageRu, status, fileName=None):
    body = {'messageEn': messageEn, 'messageRu': messageRu, 'status': status}
    log = logger.Logger().getLoggerUpload(node.id)
    url = DISTS_BACKOFFICE_URL + '/%s' % node.id
    if hasattr(node, 'statusServer'):
        url = url.replace('localhost:8080', node.statusServer)
    if node.currentConverter.dict_state_id:
        log.debug('UPDATE dictState')
        method = 'PUT'
        url += '/%s' % node.currentConverter.dict_state_id
        if node.version >= '3':
            url += '?useDictGenerateId=true'
    else:
        method = 'POST'
        body['format'] = formats.get(node.currentConverter.name, 'UNKNOWN')
        body['fileName'] = convertFileName(fileName)
    log.debug('send dictState: %s' % body)
    body = json.dumps(body)
    try:
        response, content = http.request(url, method, headers=jsonHeader, body=body)
        if response.status == 200:
            return json.loads(content)
        else:
            log.warning(u'Ошибка отправки статуса конвертирования справочника на КС. %s' % content)
    except Exception, e:
        log.warning(u'Ошибка отправки статуса конвертирования справочника на КС. %s' % str(e))
    return None

def getDictState(node, fileName):
    if logger.Logger().location == 'cs':
        url = DISTS_BACKOFFICE_URL + '/%s?fileName=%s' % (node.id, convertFileName(fileName))
        response, content = http.request(url, 'GET', headers=jsonHeader)
        if response.status == 200:
            dictStates = json.loads(content)
            if dictStates and isinstance(dictStates, list):
                dictStates = [value for value in dictStates if value != None]
                if len(dictStates) > 0:
                    return sorted(dictStates, key=lambda k: k['time'], reverse=True)[0]['id']
    return None

def sendSaleState(node, cashCode, shiftNums, messageEn, messageRu, status, format, resultFiles=None, cashUnloadStateId=None, statusHost=None):
    log = logger.Logger().getLoggerUnload(node.id)
    if logger.Logger().location == 'cs':
        body = {'messageEn': messageEn, 'messageRu': messageRu, 'status': status, 'format': format}
        if resultFiles:
            body['resultFiles'] = resultFiles
        results = []
        body = json.dumps(body)
        log.info('send saleState - body: %s' % body)
        for shiftNum in shiftNums:
            url = SALES_BACKOFFICE_URL + '/%s/%s/%s' % (node.id, cashCode, shiftNum)
            result = sendState(url, 'POST', body, log)
            if result:
                results.append(result)
        return results
    elif logger.Logger().location == 'cash' and cashUnloadStateId and statusHost:
        return updateCashUnloadStatus(cashUnloadStateId, messageEn, messageRu, status, log, statusHost=statusHost)
    return None
    
    
def updateSaleState(node, saleBackOfficeIds, state, cashUnloadStateId=None, statusHost=None):
    log = logger.Logger().getLoggerUnload(node.id)
    if logger.Logger().location == 'cs':
        results = []
        body = json.dumps(state)
        log.debug('update saleState - ids: %s body: %s' % (saleBackOfficeIds, body))
        for saleBackOfficeId in saleBackOfficeIds:
            url = SALES_BACKOFFICE_URL + '/%s' % saleBackOfficeId
            result = sendState(url, 'PUT', body, log)
            if result:
                results.append(result)
        return results
    elif logger.Logger().location == 'cash' and cashUnloadStateId and statusHost:
        return updateCashUnloadStatus(cashUnloadStateId, state['messageEn'], state['messageRu'], state['status'], log, statusHost=statusHost)
    return None

def updateCashUnloadStatus(cashUnloadStateId, messageEn, messageRu, status, log, statusHost=None):
    if status == STATUS_READY:
        status = STATUS_DURING
    body = json.dumps({'messageEn': messageEn, 'messageRu': messageRu, 'status': status})
    url = 'http://' + statusHost + SALES_CASHUNLOAD_URL  + '/%s' % cashUnloadStateId
    return sendState(url, 'PUT', body, log)
    
def sendState(url, method, body, log):
    try:
        response, content = http.request(url, method, headers=jsonHeader, body=body)
        if response.status == 200:
            return json.loads(content)
        else:
            log.warning(u'Получен некоректный http-код %s во время отправки статуса на КС: ' % (response.status, content))
    except Exception, e:
        log.warning(u'Во время отправки статуса на КС произошла ошибка: %s' % str(e))
    return None

def startLoadDictCash(cashCode, log):
    try:
        response, content = http.request(REST_URL+'/cashes/%s/dicts/startLoad' % (cashCode), 'POST', headers=jsonHeader)
        if response.status == 200:
            log.info(u'Oтправлен сигнал на кассу %s для запуска загрузки справочников.' % cashCode)
        else:
            log.warning(u'Получен некоректный http-код %s во время отправки сигнала на кассу %s для запуска загрузки справочников: ' % (response.status, cashCode, content))
    except Exception, e:
        log.warning(u'Во время оотправки сигнала на кассу %s для запуска загрузки справочников произошла ошибка: %s' % (cashCode, str(e)))

def rereadQueuesInNES(log):
    try:
        response, content = http.request('http://localhost:28081/nes/queue_reread', 'PUT', headers=jsonHeader)
        if response.status == 200:
            log.info(u'Oтправлен сигнал в NES, чтобы он перечитал очереди.')
        else:
            log.warning(u'Ошибка отправки сигнала в NES, чтобы он перечитал очереди. Он перечитает их позже.')
    except Exception, e:
        log.warning(u'Ошибка отправки сигнала в NES, чтобы он перечитал очереди. Он перечитает их позже.')
