# -*- coding: UTF-8 -*-
'''
Created on 10.05.2012

@author: kov
'''

import os
from common import logger, filesmodify, macros
import datetime
import codecs
import copy
import string
import re

logInfo = "SHTRIH converter: "
id = 0
currentConverter = None

class SecurityLogReports(object):
    def __init__(self, securityLogs = [], cursor = 0):
        self.securityLogs = securityLogs
        self.cursor = cursor
        self.current = securityLogs[cursor] if len(securityLogs) > cursor else None

    def next(self):
        self.cursor += 1
        if len(self.securityLogs) > self.cursor:
            self.current = self.securityLogs[self.cursor]
        else:
            self.current = None
        return self.current

def parseFlag(flag):
    f = open(flag, 'r')
    shifts = None
    dates = None
    if f.readline().strip() == '###':
        str = f.readline().strip()
        if len(str.split('-')) == 2:
            if str.find('.') == -1:
                shifts = str.replace('-', '@')
            else:
                fromDate = str.split('-')[0].split('.')
                toDate = str.split('-')[1].split('.')
                dates = '%s-%s-%sT00:00:00.0@%s-%s-%sT23:59:59.99' % (fromDate[2], fromDate[1], fromDate[0], toDate[2], toDate[1], toDate[0])
    f.close()
    return shifts, dates

def openFile(log, cash, node, workDirectory, shiftNum, previousFile=None):
    output = None
    firstRound = True
    currentPath = macros.applyMacros(currentConverter.path, log, shopcode = node.id, cashcode = cash.code, shiftnum = shiftNum)

    salePath = workDirectory + '/' + os.path.basename(currentPath)
    os.umask(0)
    if os.path.isfile(salePath):
        log.info(logInfo + "file exists " + salePath)

        f = codecs.open(salePath, 'r', currentConverter.charset)
        if f.readline(1) == '#':
            if not currentConverter.rewriteDestFile or previousFile == salePath:
                output = codecs.open(salePath, 'a', currentConverter.charset)
        f.next()
        try:
            f.next()
        except StopIteration:
            if output:
                output.close()
        try:
            f.next()
            firstRound = False
        except StopIteration:
            firstRound = True
        f.close()
        firstRound = False

    if not output:
        output = codecs.open(salePath, 'w', currentConverter.charset)
        output.write("#\r\n")
        output.write(unicode(str(cash.code), 'utf-8') + "\r\n")
        firstRound = True

        log.info(logInfo + "file is created " + salePath)
    return output, firstRound

def run(salesFunc, securityLogsFunc, shiftsFunc, cash, node, workDirectory, currentSales):
    global currentConverter
    log = logger.Logger().getLoggerUnload(node.id)
    log.info(logInfo + "START. Cash №" + str(cash.code))
    currentConverter = node.currentConverter
    output = None
    firstRound = True       # Флаг. Указавет на то, чтобы писать или не писать заголовок отчета в файл.

    shifts = {}
    shiftsNotOpen = []
    for sh in shiftsFunc():
        if sh['timeBeg'] is None:
            shiftsNotOpen.append(sh['shift'])
        else:
            shifts[sh['shift']] = sh

    shift = None
    global id

    securityLogs = {}
    for secLog in securityLogsFunc():
        if not securityLogs.has_key(str(secLog['shiftnum'])):
            securityLogs[str(secLog['shiftnum'])] = []
        if secLog['opcode'] in [8, 11]:
            securityLogs[str(secLog['shiftnum'])].append(secLog)

    securityLogReports = SecurityLogReports()
    sumDrawer = 0
    sumGain = 0

    for check in salesFunc():
        if check['shift'] in shiftsNotOpen:
            continue
        if firstRound:
            output, firstRound = openFile(log, cash, node, workDirectory, str(check['shift']))
            if firstRound:
                output.write(str(check['shift']) + "\r\n")
                firstRound = False

        if shift != check['shift']:
            if shift:
                reports(output, None, shifts[shift], securityLogReports, sumDrawer, sumGain)
            if shifts.has_key(shift):
                workShiftClosing(output, shifts[shift], currentConverter.unloadShiftsFromKkm)
                previousFile = output.name
                output.close()
                output, firstRound = openFile(log, cash, node, workDirectory, str(check['shift']), previousFile)
                if firstRound:
                    output.write(str(check['shift']) + "\r\n")
                    firstRound = False
            shift = check['shift']
            id = 100000 * check['shift'];
            sumDrawer = 0
            sumGain = 0
            securityLogReports = SecurityLogReports(securityLogs[str(shift)] if securityLogs.has_key(str(shift)) else [])

        reports(output, check, shifts[shift], securityLogReports, sumDrawer, sumGain)

            #continue

        if check['docType'] == 16 or check['docType'] == 18:
            inventory(output, check)
            continue

            # pricei -> finalPrice
            # sumi -> posSum
            # tindex -> priceIndex

        listDiscForCheck = []           #Все скидки
        listBonusDiscForCheck = []      #Бонусные скидки
        listBonusEarn = []              #начисления бонусов
        bonusDiscPerc = fullBonusDiscountPerc(check) #процент бунусной скидки на весь чек
        
        positions = check['inventPositions'] + check['stornoPositions']
        for pos in sorted(positions, key=lambda pos: pos['posNum']):
            positionsAdd(output, pos, check["cashCode"], check["docNum"])
            #Начисление бонусов
            if pos.has_key('bonusPositions'):
                listBonusEarn += bonusEarnForPosition(output, pos['bonusPositions'], check['docType'], check['docNum'], check['cashCode'], check["userCode"])

            if pos.has_key('timeStorno'):
                stornoPositionsRemove(output, pos, check["cashCode"], check["docNum"])
            else:
                listDiscForCheck += discounts(output, pos, currentConverter.discountsDetail, check['docType'], check['cashCode'], check['docNum'], check['baseSum'])
                #Списание бонусов
                res = bonusSpend(output, pos, currentConverter.discountsDetail, check['docType'], check['cashCode'], check['docNum'], check["userCode"], bonusDiscPerc)
                listDiscForCheck += res
                listBonusDiscForCheck += res

        discountCheck(output, listDiscForCheck, check['docNum'], check['docType'], check['baseSum'])
        bonusCheck(output, listBonusDiscForCheck, check['docType'], check['cashCode'], check['docNum'], check['userCode'], bonusDiscPerc)

        #Начисление бонусов на чек
        bonusEarnForCheck(output, listBonusEarn, check['docType'], check['docNum'], check['cashCode'], check["userCode"], listBonusDiscForCheck)

        documentPayment(output, check)

        if check['docType'] in [1, 2, 25]:  # Чек продажи/возврата/возврат по чеку
            if check['closed'] != 0:
                documentClosing(output, check)
        elif check['docType'] in [3, 4]:  # Внесения/изъятие денег
            moneyOperations(output, check)

        for moneyPos in check['moneyPositions']:
            if moneyPos['opCode'] in [44, 63,70,76,80]:
                sumDrawer += moneyPos['sumB']
            elif moneyPos['opCode'] in [64, 72,74,82]:
                sumDrawer -= moneyPos['sumB']
            if moneyPos['opCode'] in [63, 70,73,75,76]:
                sumGain += moneyPos['sumB']
            elif moneyPos['opCode'] in [64, 71,72,74,77]:
                sumGain -= moneyPos['sumB']

        # Если анонимная карта, выгружаем анкетные данные клиента
        for card in check['cardPositions']:
            if card['cardstatus'] != 0:
                continue
            for client in check['clientPositions']:
                if card['clientitemid'] == client['clientitemid']:
                    typeTrans = 165
                    if client['phonenumber']:
                        match = re.match("^\+7([0-9]{1})([0-9]{2})([0-9]{4})([0-9]{3})", client['phonenumber'])
                        if match:
                            tab10 = float("%s.%s" % (match.group(1), match.group(2)))
                            tab11 = float("%s.%s" % (match.group(3), match.group(4)))
                            insertData(output, check['timeEnd'], typeTrans, check['cashCode'], check['docNum'], check['userCode'], card['number'], 0, tab10, tab11, 0)
                    if client['birthday']:
                        dateStr = client['birthday'].strftime("%Y%m%d")
                        tab11 = float(dateStr[:5] + '.' + dateStr[5:])
                        insertData(output, check['timeEnd'], typeTrans, check['cashCode'], check['docNum'], check['userCode'], card['number'], 1, 0, tab11, 0)
                    if client['sex'] is not None:
                        tab11 = client['sex'] + 1
                        insertData(output, check['timeEnd'], typeTrans, check['cashCode'], check['docNum'], check['userCode'], card['number'], 2, 0, tab11, 0)
                    break;

    if shifts.has_key(shift):
        workShiftClosing(output, shifts[shift], currentConverter.unloadShiftsFromKkm)

    if output:
        output.close()
        converterSalesReadyFile = open('%s/ready' % (workDirectory), 'w')
        converterSalesReadyFile.close()
        salesReadyFile = open('%s/ready' % (os.path.dirname(os.path.dirname(workDirectory))), 'w')
        salesReadyFile.close()

    if shift:
        shifts[shift] =''
    log.info(logInfo + "Unloaded the following number of shifts: %s" % shifts.keys())
    log.info(logInfo + "FINISH")

def inventory(file, check):
    mapPosition = {}
    for pos in check['inventPositions']:

        if not pos['deptCode'] in mapPosition:
            mapPosition[pos['deptCode']] = [pos]
        else:
            mapPosition[pos['deptCode']].append(pos)

    for key in mapPosition:
        if len(mapPosition[key]):
            docType = 0
            typeTrans = 300
            if currentConverter.useTransaction300ForDocuments:
                docType = 4 # инвентаризация
                if check['docType'] == 18:
                    docType = 7 # возврат поставщику
            else:
                typeTrans = 302 # инвентаризация
                if check['docType'] == 18:
                    typeTrans = 308 # возврат поставщику

            insertData(file, check['timeBeg'], typeTrans, check['cashCode'], check['docNum'], check['userCode'], '', key, check['shift']*1000+check['docNum'], docType, 0)
            doumentStore = '<Document store="%s">\r\n' % (key)
            file.write(unicode(doumentStore, 'utf-8'))
            for pos in mapPosition[key]:
                strPosition = '%s;%s;\r\n' % (pos['inventCode'], pos['quant'])
                file.write(unicode(strPosition, 'utf-8'))
            file.write('</Document>\r\n')

def bonusCheck(file, listBonusDiscForCheck, docType, cashCode, docNum, userCode, discSize):
    
    if not len(listBonusDiscForCheck):
        return

    posTime = ''
    cardNumber = ''
    discSum = 0
    
    for bonus in listBonusDiscForCheck:
        posTime = bonus['posTime']
        cardNumber = bonus['cardNumber']
        discSum += bonus['discSum']

    if docType != 1:
        discSum = -discSum
    insertData(file, posTime, 35, cashCode, docNum, userCode, 0, 0, 0, discSize, discSum)
    insertData(file, posTime, 71, cashCode, docNum, userCode, cardNumber, 4, 0, discSize, discSum)

def discountCheck(file, listCheckDisc, docNum, docType, docBaseSum):
#percent = 1,
#absolute,
#index,
#price,
#bonus

    totalDiscount = {}
    for disc in listCheckDisc:
        if disc['discType'] == 'bonus':
            continue
        if not totalDiscount.has_key(disc['discNumber']):
            totalDiscount[disc['discNumber']] = disc
            totalDiscount[disc['discNumber']]['DISCOUNTSUM'] = disc['discSum']
            totalDiscount[disc['discNumber']]['MAXDISCSIZE'] = disc['discSize']
        else:
            totalDiscount[disc['discNumber']]['DISCOUNTSUM'] += disc['discSum']
            totalDiscount[disc['discNumber']]['MAXDISCSIZE'] = max(totalDiscount[disc['discNumber']]['MAXDISCSIZE'], disc['discSize'])

    for key in totalDiscount:
        prm11 = prm12 = abs(totalDiscount[key]['DISCOUNTSUM'])
        if totalDiscount[key]['discType'] != 'index':     # 3
            prm11 = totalDiscount[key]['MAXDISCSIZE']

        if docType != 1:
            prm12 = -abs(totalDiscount[key]['DISCOUNTSUM'])

        opCode = 37 if totalDiscount[key]['discType'] == 'percent' else 35  # 35 - суммовая, 37 - процентная скидка
        opCode += 1 if totalDiscount[key]['DISCOUNTSUM'] < 0 else 0         # 36 - суммовая, 38 - процентная надбавка

        insertData(file, totalDiscount[key]['posTime'], opCode, totalDiscount[key]['cashcode'], docNum, totalDiscount[key]['userCode'], 0, 0, 0, prm11, prm12)

        prm8 = prm9 = prm10 = prm11 = 0

        if totalDiscount[key]['discMode'] == 'automatic':     # 1
            prm8 = totalDiscount[key]['discScheme']
            prm9 = 2
            prm10 = totalDiscount[key]['discCode']
        elif totalDiscount[key]['discMode'] == 'byCard':      # 2
            prm8 = totalDiscount[key]['cardNumber']
            prm9 = 3
            prm10 = docBaseSum
        elif totalDiscount[key]['discMode'] == 'manual':      # 3
            prm9 = 0
        elif totalDiscount[key]['discMode'] == 'fixed':       # 3
            prm9 = 1
        elif totalDiscount[key]['discMode'] == 'bydisckit':   # 5
            prm8 = totalDiscount[key]['discScheme']
            prm9 = 2
            prm10 = totalDiscount[key]['discCode']


        if totalDiscount[key]['discMode'] == 'automatic':     # 1
            prm11 = totalDiscount[key]['MAXDISCSIZE']
        elif docBaseSum != 0:
            prm11 = abs(totalDiscount[key]['DISCOUNTSUM'] * 100 / docBaseSum )

        insertData(file, totalDiscount[key]['posTime'], 71, totalDiscount[key]['cashcode'], docNum, totalDiscount[key]['userCode'], prm8, prm9, prm10, prm11, prm12)

def discountPosition(file, disc, cashCode, docNum, docType, inventCode, deptCode, docBaseSum):
    opcode = 15
    col8 = inventCode
    col9 = deptCode
    col11 = disc['discSize']
    col12 = -abs(disc['discSum'])

    if disc['discType'] == 'percent':     # 1
        opcode = 17
    if disc['discSum'] < 0:
        opcode+=1

    if disc['discType'] in ['absolute', 'index']:     # 2, 3
        col11 = disc['discSum']

    if docType == 1:
        col12 = abs(disc['discSum'])
    insertData(file, disc['posTime'], opcode, cashCode, docNum, disc['userCode'], col8, col9, 0, col11, col12)

    prm8 = prm9 = prm10 = prm11 = 0

    if disc['discMode'] == 'automatic':     # 1
        prm8 = disc['discScheme']
        prm9 = 2
        prm10 = disc['discCode']
    elif disc['discMode'] == 'byCard':      # 2
        prm8 = disc['cardNumber']
        prm9 = 3
        prm10 = docBaseSum
    elif disc['discMode'] == 'manual':      # 3
        prm9 = 1
    elif disc['discMode'] == 'bydisckit':   # 5
        prm8 = disc['discScheme']
        prm9 = 2
        prm10 = disc['discCode']


    if disc['discMode'] == 'automatic':     # 1
        prm11 = abs(disc['discSize'])
    elif docBaseSum != 0:
        prm11 = abs(disc['discSum'] / docBaseSum * 100)

    insertData(file, disc['posTime'], 70, cashCode, docNum, disc['userCode'], prm8, prm9, prm10, prm11, col12)

def discounts(file, position, discountsDetail, docType, cashCode, docNum, docBaseSum):
    listDiscForCheck = []
    for disc in position['discountPositions']:
        if disc['discType'] == 'bonus':
            continue
        if disc['isPositionDiscount']:
            discountPosition(file, disc, cashCode, docNum, docType, position['inventCode'], position['deptCode'], docBaseSum)
        else:
            if discountsDetail:
                discountPosition(file, disc, cashCode, docNum, docType, position['inventCode'], position['deptCode'], docBaseSum)
            else:
                listDiscForCheck.append(disc)
    return listDiscForCheck

# Процент скидки списания бонусов для всего чека
def fullBonusDiscountPerc(check):
    discAmount = 0
    for pos in check['inventPositions']:
        for disc in pos['discountPositions']:
            if disc['discType'] != 'bonus':
                continue
            discAmount = discAmount + disc['discSum']

    checkSum = check['docSum'] + discAmount
    result = 0 if checkSum == 0 else 100 * discAmount / checkSum
    return result

# Списанные бонусы
def bonusSpend(file, position, discountsDetail, docType, cashCode, docNum, userCode, bonusDiscPerc):
    listBonusForCheck = []
    bonusAmount = 0
    bonusPositions = []
    for disc in position['discountPositions']:
        if disc['discType'] != 'bonus':
            continue
        if disc['isPositionDiscount']:
            bonusAmount = addBonusPosition(bonusPositions, disc, docType, bonusAmount)
        else:
            if discountsDetail:
                bonusAmount = addBonusPosition(bonusPositions, disc, docType, bonusAmount)
            else:
                listBonusForCheck.append(disc)
    if len(bonusPositions):
        listBonusForCheck += bonusPositions
        bonusPosition(file, bonusPositions[0], docType, cashCode, bonusAmount, bonusDiscPerc, docNum, userCode)
    return listBonusForCheck


def workShiftClosing(file, shift, unloadShiftsFromKkm):
    if shift['timeEnd']:
        docNum = 1
        if shift['checkNum2']:
            docNum = shift['checkNum2'] + 1

        shiftNum = 0
        sumDrawer = 0.0
        sumGain = 0.0
        if unloadShiftsFromKkm:
            if len(shift['kkms']) > 0:
                shiftNum = shift['kkms'][0].get('shiftnumkkm', 0)

            for kkm in shift['kkms']:
                sumDrawer += float(kkm.get('sumCashEnd', 0.0))
                sumGain += float(kkm.get('sumGain', 0.0))
        else:
            shiftNum = shift['shift']
            sumDrawer = shift['sumDrawer']
            sumGain = shift['sumGain']

        insertData(file, shift['timeEnd'], 61, shift['cashCode'], docNum, shift['userCode'], shiftNum, 0, sumDrawer, 0, sumGain)

def documentClosing(file, check):
    typeTrans = 55 if check['closed'] == 1 else 56
    insertData(file, check['timeEnd'], typeTrans, check['cashCode'], check['docNum'], check['userCode'], 0, 0, 0, 0, check['docSum'])

def reports(file, chk, shift, securityLogReports, sumDrawer, sumGain):
    check = copy.copy(chk)
    if not check and shift['timeEnd']:
        check = {}
        check['timeBeg'] = shift['timeEnd']
        check['docNum'] = shift['checkNum2'] + 1 if shift['checkNum2'] else 1
    check['docNum'] -= 1
    while securityLogReports.current and securityLogReports.current['optime'] < check['timeBeg']:
        if securityLogReports.current['opcode'] == 8:
            xReport(file, check, shift, securityLogReports.current, sumDrawer, sumGain)
        elif securityLogReports.current['opcode'] == 11:
            extendedReport(file, check, shift, securityLogReports.current)
        securityLogReports.next()

def xReport(file, check, shift, op, sumDrawer, sumGain):
    insertData(file, op['optime'], 60, shift['cashCode'], check['docNum'], op['cashiercard'], 0, 0, sumDrawer, 0, sumGain)

def extendedReport(file, check, shift, op):
    insertData(file, op['optime'], 62, shift['cashCode'], check['docNum'], op['cashiercard'], 3, 0, 0, 0, 0)

def documentPayment(file, check):

    if check['closed'] != 1:
        return

    defPayment = (currentConverter.defaultCardType, 0, 0, datetime.datetime.max)
    payments = {}

    for pos in check['moneyPositions']:
        if pos['opCode'] not in [70, 72, 74, 76]:
            continue

        if pos['valutOperation'] == 3: # сертификат
            isRepaid = 1 if float(pos['balance']) <= 0.001 else 0
            insertData(file, pos['posTime'], 73, check["cashCode"], check["docNum"], check["userCode"], pos["cardnum"], '', pos['sumB'], isRepaid, pos['sumB'])
            continue

        valCode = pos['valCode']
        tab9, tab10, tab12, postime = defPayment
        if payments.has_key(valCode):
            tab9, tab10, tab12, postime = payments[valCode]

        if currentConverter.unloadCardType and pos['docnum'] != '':
            tab9 = pos['docnum']

        if pos['opCode'] in [72, 76]:       # Сдача
            tab10 += pos['sumB']

        if pos['opCode'] == 70:             # Сумма оплаты
            tab12 += pos['sumB']
        elif pos['opCode'] == 74:
            tab12 -= pos['sumB']

        postime = min(postime, pos['posTime'])

        payments[valCode] = (tab9, tab10, tab12, postime)

    if len(payments):
        for valCode in payments:
            payment = payments[valCode]
            tab8 = 0
            if currentConverter.unloadPaymentByCard and valCode != 1:
                tab8 = str(valCode).rjust(24, '0')
                insertData(file, payment[3], 95, check["cashCode"], check["docNum"], check["userCode"],tab8, payment[0], payment[1], valCode, payment[2])
            insertData(file, payment[3], 40, check["cashCode"], check["docNum"], check["userCode"], tab8, payment[0], payment[1], valCode, payment[2])
    elif len(check['moneyPositions']) == 0:
        insertData(file, check['timeEnd'], 40, check["cashCode"], check["docNum"], check["userCode"], 0, defPayment[0], defPayment[1], 1, defPayment[2])

def moneyOperations(file, check):
    userCode = check['userCode']
    if currentConverter.useActorCodeForMoneyOperations:
        userCode = check['moneyPositions'][0]['acode']
    typeTrans = 51
    if check['docType'] == 3:
        typeTrans = 50

    insertData(file, check['timeEnd'], typeTrans, check['cashCode'], check['docNum'], userCode, 0, 0, 0, 0, check['docSum'])

def stornoPositionsRemove(file, invent, cashCode, docNum):
    if not currentConverter.unloadPaymentCommission and invent['opCode'] in [1004, 1006]:
        return
    posSum = abs(invent['posSum'])
    code = invent['inventCode']
    typeTrans = 12

    cquant = abs(invent.get('accountingQuant', 1.000))
    modSum = abs(invent['modSum'])
    price = 0.00
    if abs(float(cquant) - 1.000) <= 0.0001:
        price = abs(invent['finalPrice'])
    else:
        price = float(modSum) / float(cquant)

    if invent['opCode'] in [50, 52, 1000, 1004]:
        posSum = -abs(invent['posSum'])
        cquant = -abs(invent['accountingQuant'])

    if invent['opCode'] in [52, 56, 60]:
        code = ''
        typeTrans = 2

    insertData(file, invent['timeStorno'], typeTrans, cashCode, docNum, invent["userCode"], code, invent['deptCode'], "%.2f" % (price), cquant, posSum)

def positionsAdd(file, invent, cashCode, docNum):
    if not currentConverter.unloadPaymentCommission  and invent['opCode'] in [1004, 1006]:
        return
    typeTransaction = None
    code = invent['inventCode']
    cquant = abs(invent.get('accountingQuant', 1.000))
    posSum = abs(invent['modSum'])

    price = 0.00
    if abs(float(cquant) - 1.000) <= 0.0001:
        price = abs(invent['finalPrice'])
    else:
        price = float(posSum) / float(cquant)

    if not invent['opCode'] in [50, 52, 1000, 1004]:
        cquant = -abs(invent['accountingQuant'])
        posSum = -abs(invent['modSum'])

    if invent['opCode'] in [52, 56, 60]:
        code = ''

    if invent['opCode'] in [50, 1000, 1004]:
        typeTransaction = 11
    elif invent['opCode'] == 52:
        typeTransaction = 1
    elif invent['opCode'] == 54:
        typeTransaction = 14
    elif invent['opCode'] == 56:
        typeTransaction = 4
    elif invent['opCode'] == 58:
        typeTransaction = 13
    elif invent['opCode'] == 60:
        typeTransaction = 3
    elif invent['opCode'] == 63:
        typeTransaction = 72
    elif invent['opCode'] == 64:
        typeTransaction = 74

    if invent['opCode'] in [63, 64]: # продажа/возврат сертификата
        code = invent['barCode']
        insertData(file, invent['posTime'], typeTransaction, cashCode, docNum, invent["userCode"], code, '', "%.2f" % (price), 0, "%.2f" % (price))
        return

    insertData(file, invent['posTime'], typeTransaction, cashCode, docNum, invent["userCode"], code, invent['deptCode'], "%.2f" % (price), cquant, posSum)
    if abs(float(abs(invent.get('accountingQuant', 1.000))) - 1.000) > 0.0001:
        insertData(file, invent['posTime'], 24, cashCode, docNum, invent["userCode"], invent['barCode'], invent['deptCode'], 0.00, cquant, 0.00)


def addBonusPosition(bonusPositions, disc, docType, amount):
    if disc['discType'] != 'bonus':
        return amount
    if docType not in [1, 2, 25]:
        return amount
    amount = amount + disc['discSum']
    bonusPositions.append(disc)
    return amount

def bonusPosition(file, disc, docType, cashCode, amount, bonusDiscPerc, docNum, userCode):
    typeTrans = 162 if docType == 1 else 164
    sum1 = amount if docType == 1 else -amount
    insertData(file, disc['posTime'], typeTrans, cashCode, docNum, userCode, disc['cardNumber'], 0, 0, bonusDiscPerc, sum1)

# Добавляет начисления баллов на позицию
def bonusEarnForPosition(file, bonusPositions, docType, docNum, cashCode, userCode):
    if docType not in [1, 2, 25]:
        return []

    resultBonusPos = None
    for bonusPosition in bonusPositions:
        if bonusPosition['ispositionbonus']:
            if resultBonusPos:
                resultBonusPos['amount'] += bonusPosition['amount']
            else:
                resultBonusPos = bonusPosition

    if resultBonusPos:
        if docType in [2, 25]:
            resultBonusPos['amount'] = -resultBonusPos['amount']
        typeTrans = 161 if docType == 1 else 163
        insertData(file, resultBonusPos['posTime'], typeTrans, cashCode, docNum, userCode, resultBonusPos['cardNumber'], 0, 0, 0, resultBonusPos['amount'])

    return bonusPositions

# Метод добавляет информацию о начислении баллов на чек. Если есть начисления на чек,
# все эти начисления группируются по полю 'bonusnumber' и добавляются в виде 161 или 163
# транзакции. Если нет начислений на чек, но есть начисления на позиции или списания баллов,
# также добавляется транзакция, но с нулевой суммой.
# 9 поле - признак бонусного начисления на чек
# 10 поле - номер бонусной акции
def bonusEarnForCheck(file, bonusPositions, docType, docNum, cashCode, userCode, listBonusDiscForCheck):
    if docType not in [1, 2, 25]:
        return

    amountFunc = lambda x: x
    if docType in [2, 25]:
        amountFunc = lambda x: -x

    actions = {}
    # Собираем все акции на чек из позиционных
    for bonusPosition in bonusPositions:
        if not bonusPosition['ispositionbonus']:
            bonusnumber = bonusPosition['bonusnumber']
            if actions.has_key(bonusnumber):
                actions[bonusnumber]['amount'] += bonusPosition['amount']
            else:
                actions[bonusnumber] = bonusPosition

    typeTrans = 161 if docType == 1 else 163

    if len(actions):
        for bonusnumber in actions:
            action = actions[bonusnumber]
            insertData(file, action['posTime'], typeTrans, cashCode, docNum, userCode, action['cardNumber'], 1,  action['campaignid'], 0, amountFunc(action['amount']))
    elif len(bonusPositions) or len(listBonusDiscForCheck):
        posTime = bonusPositions[0]['posTime'] if len(bonusPositions) else listBonusDiscForCheck[0]['posTime']
        cardNumber = bonusPositions[0]['cardNumber'] if len(bonusPositions) else listBonusDiscForCheck[0]['cardNumber']
        insertData(file, posTime, typeTrans, cashCode, docNum, userCode, cardNumber, 1, 0, 0, 0)

def positionsRevaluation(file, invent, cashCode, docNum):
    code = invent['inventCode']
    if invent['opCode'] in [52, 56, 60]:
        code = ''

    if invent['price'] != invent['finalPrice']:
        insertData(file, invent['posTime'], 30, cashCode, docNum, invent["userCode"], code, invent['deptCode'], invent['price'], invent['priceIndex'], invent['finalPrice'])

def insertData(file, transDateTime, transType, cashCode, docNum, userCode, tab8, tab9, tab10, tab11, tab12):
    if tab10 == None:
        tab10 = 0
    if tab12 == None:
        tab12 = 0

    if tab8 == None:
        tab8 = 0

    if tab9 == None:
        tab9 = 0

    tab11 = round(tab11, 3)
    global id
    id += 1
    transNum = str(id)
    if transType in [300, 302, 308]:
        transNum = '2' + transNum
    transDate = transDateTime.strftime('%d.%m.%Y')
    if currentConverter.unloadShortDateFormat:
        transDate = transDateTime.strftime('%d.%m.%y')
    transTime = transDateTime.strftime('%H:%M:%S')

    strTrans = '%s$dl%s$dl%s$dl%s$dl%s$dl%s$dl%s$dl%s$dl%s$dl%s$dl%s$dl%s$dl\r\n' % (transNum, transDate, transTime, transType, cashCode, docNum, userCode, tab8, tab9, tab10, tab11, tab12)
    strTransWithDelimiter = string.replace (strTrans,'$dl',currentConverter.delimiter)
    file.write(unicode(strTransWithDelimiter, 'utf-8'))

def sendSales(cash, node, workDirectory):
    log = logger.Logger().getLoggerUnload(node.id) 
    rewriteDestFile = filesmodify.checkFilesInRemoteDirectory(rewrite=node.currentConverter.rewriteDestFile, workDirectory = workDirectory, remoteDirectory = str(os.path.dirname(node.currentConverter.path)))
    return filesmodify.safeMoveDirectory(log, workDirectory, os.path.dirname(node.currentConverter.path), backupDir = cash.backup_dir, rewrite=rewriteDestFile, maxCountFiles=cash.maxCountFiles, actualCountDays=cash.actualCountDays)

def sendOnlineSales(cash, node, workDirectory):
    return False, []
