# -*- coding: UTF-8 -*-
'''
Created on 25.04.2012

@author: kov
'''

import os, sys
from common import filesmodify
from common import logger
from common import macros
from common import config
import datetime
import shutil

unloadDocumetTypes = [1,2,3,4,25]

def getValutCodes():
    dictValuts = {}
    if config.read(os.path.normpath(sys.path[0] + "/../config/cash/exchangers.ini")):
        valuts = config.options('unload.converter.smag.valut')
        for val in valuts:
            dictValuts[val] = config.getInt('unload.converter.smag.valut', val, 0)
    return dictValuts

def run(salesFunc, securityLogsFunc, shiftsFunc, cash, node, workDirectory, currentSales):

    log = logger.Logger().getLoggerUnload(node.id)
    log.info(u"Старт конвертирования")
    dictValuts = getValutCodes()
    workDirectory = macros.applyMacros(workDirectory, log, shopcode=node.id, cashcode=cash.code)

    if node.currentConverter.appendSaleFiles and not currentSales:
        if not filesmodify.simpleCopyFiles(node.currentConverter.path, workDirectory, log)[0]:
            log.info(u"Не удалось скопировать файлы продаж из целевой папки в рабочую!")
            return

    plugin = __import__("unload.converter.smag.%s" % node.currentConverter.format, fromlist='q')
    header = plugin.DataHeader(workDirectory, node.currentConverter.charset, currentSales)

    curMoneys = {}
    currests = {}
    shifts = {}

    for shift in shiftsFunc():
        timeEnd = None
        if shift['timeEnd']:
            timeEnd = shift['timeEnd']
        else:
            timeEnd = datetime.datetime.now()

        if shift['shopcode']:
            shifts[shift['shift']] = {'SHOPINDEX' : shift['shopcode'], 'DATE' : timeEnd}
        else:
            shifts[shift['shift']] = {'SHOPINDEX' : 0, 'DATE' : timeEnd}

    # CHECK
    for check in salesFunc():
        earnSumCheck = 0
        lastPosNum = 0
        if not check['docType'] in unloadDocumetTypes and check['docType'] != 13:
            continue
        docNum = int(check['docNum'])
        cashCode = int(check['cashCode'])
        shiftNumber = int(check['shift'])
        shopIndex = int(shifts[shiftNumber]['SHOPINDEX'])

        if not shiftNumber in currests:
            currests[shiftNumber] = {}
            curMoneys[shiftNumber] = {}

        if not cashCode in currests[shiftNumber]:
            currests[shiftNumber][cashCode] = {'SHOPINDEX' : shopIndex, 'DATE' : shifts[shiftNumber]['DATE'], 'SALE' : 0, 'RETURN' : 0,}
            curMoneys[shiftNumber][cashCode] = {'SHOPINDEX' : shopIndex, 'PERIODIN' : 0, 'PERIODOUT' : 0, 'BEFORE' : 0, 'AFTER' : 0}


        if check['docType'] == 13:
            curMoneys[shiftNumber][cashCode]['BEFORE'] = check['docSum']

        valCode = None
        moneyPos = 0
        payment = 0
        moneyForValut = {}
        for moneyPosition in check['moneyPositions']:
            if not moneyPos:
                valCode = moneyPosition['valCode']

            # Если оплата производилась наличными, то
            payment = moneyPosition['valCode']
            if moneyPosition['valCode'] == 1:
                moneyPos += 1
                if moneyPosition['opCode'] == 80:
                    curMoneys[shiftNumber][cashCode]['PERIODIN'] += moneyPosition['sumB']
                elif moneyPosition['opCode'] == 82:
                    curMoneys[shiftNumber][cashCode]['PERIODOUT'] += moneyPosition['sumB']

                if moneyPosition['opCode'] in [80, 70, 76]:
                    curMoneys[shiftNumber][cashCode]['AFTER'] += moneyPosition['sumB']
                else:
                    curMoneys[shiftNumber][cashCode]['AFTER'] -= moneyPosition['sumB']
                payment = 0

            if dictValuts.has_key(str(valCode)):
                payment = dictValuts.get(str(valCode))
            if not payment in moneyForValut:
                moneyForValut[payment] = 0
            if moneyPosition['opCode'] in [70, 74]:
                moneyForValut[payment] += moneyPosition['sumB']
            else:
                moneyForValut[payment] -= moneyPosition['sumB']

            if int(moneyPosition['opCode']) == 70:
                currests[shiftNumber][cashCode]['SALE'] += moneyPosition['sumB']
            elif int(moneyPosition['opCode']) == 72:
                currests[shiftNumber][cashCode]['SALE'] -= moneyPosition['sumB']

            if int(moneyPosition['opCode']) == 74:
                currests[shiftNumber][cashCode]['RETURN'] += moneyPosition['sumB']
            elif int(moneyPosition['opCode']) == 76:
                currests[shiftNumber][cashCode]['RETURN'] -= moneyPosition['sumB']
        for key, value in moneyForValut.items():
            if check['docType'] in [1,2,25]:
                header.setCashPay(shopIndex, cashCode, shiftNumber, docNum, key, value)
        discountCards = {}
        for position in check['inventPositions']:

            datePosition = datetime.datetime.strptime(str(position['posTime']), '%Y-%m-%d %H:%M:%S')
            replace = 0
            operation = 0
            lastPosNum = int(position['posNum'])
            inventCode = position['inventCode']
            if int(position['opCode']) in [50, 52, 1000]:
                replace = 1
                operation = 1
            elif int(position['opCode']) is 60:
                inventCode = 0
            if valCode != 1:
                operation += 4
            earnSumPosition = 0
            for bonusPosition in position['bonusPositions']:
                if bonusPosition['goodsitemid'] == position['dopdata']:
                    earnSumPosition += bonusPosition['amount']
                    earnSumCheck += bonusPosition['amount']
            # CASHSAIL
            header.setCashSail(
                                 shopIndex,
                                 cashCode,
                                 shiftNumber,
                                 docNum,
                                 int(position['posNum']),
                                 datePosition,
                                 datePosition.strftime("%H%M"),
                                 inventCode,
                                 position['quant'],
                                 position['price'],
                                 position['posSum'] - earnSumPosition,
                                 int(position['userCode']),
                                 position['deptCode'],
                                 replace,
                                 operation,
                                 0 if valCode == 1 else valCode
                              )

            for discountPosition in position['discountPositions']:
                # если скидка бонус - выгружаем ее только как оплату
                if discountPosition['discType'] == 'bonus':
                    header.setCashPay(shopIndex, cashCode, shiftNumber, docNum, node.currentConverter.spendPaymentCode, discountPosition['discSum'])
                    if int(position['opCode']) == 50:
                        currests[shiftNumber][cashCode]['SALE'] += discountPosition['discSum']
                    if int(position['opCode']) == 58:
                        currests[shiftNumber][cashCode]['RETURN'] += discountPosition['discSum']
                    continue
                discountIndex = 2
                if discountPosition['discMode'] == 'fixed':
                    if discountPosition['isPositionDiscount'] == 1:
                        discountIndex = 1
                    else:
                        discountIndex = 3
                elif discountPosition['discMode'] == 'byCard':
                    discountIndex = 4

                discSum = 0
                if discountPosition['discSum']:
                    discSum = discountPosition['discSum']

                posBaseSum = 0
                if position['baseSum']:
                    posBaseSum = position['baseSum']
                if posBaseSum == 0:
                    posBaseSum = 1


                if discountPosition['discMode'] == 'byCard':
                    if discountPosition['cardNumber']:
                        if not discountPosition['cardNumber'] in discountCards:
                            discountCards[discountPosition['cardNumber']] = 0

                        discountCards[discountPosition['cardNumber']] += discountPosition['discSum']

                # CASHDISC
                header.setCashDisc(shopIndex, cashCode, shiftNumber, docNum, lastPosNum, discountIndex,
                                     float(discSum / posBaseSum) * 100,
                                     discSum
                                     )

            for bonusPosition in position['bonusPositions']:
                discountIndex = 4
                discSum = bonusPosition['amount']
                posBaseSum = 0
                if position['baseSum']:
                    posBaseSum = position['baseSum']
                if posBaseSum == 0:
                    posBaseSum = 1
                # CASHDISC
                header.setCashDisc(shopIndex, cashCode, shiftNumber, docNum, lastPosNum, discountIndex,
                                     float(discSum / posBaseSum) * 100,
                                     discSum
                                     )
        if earnSumCheck > 0:
            dateCheck = datetime.datetime.strptime(str(check['timeEnd']), '%Y-%m-%d %H:%M:%S')
            header.setCashSail(
                                    shopIndex,
                                    cashCode,
                                    shiftNumber,
                                    docNum,
                                    lastPosNum + 1,
                                    dateCheck,
                                    dateCheck.strftime("%H%M"),
                                    node.currentConverter.earnPositionCode,
                                    1,
                                    earnSumCheck,
                                    earnSumCheck,
                                    check['userCode'],
                                    node.currentConverter.earnDeptCode,
                                    1 if check['docType'] == 1 else 0,
                                    1,
                                    0 if valCode == 1 else valCode
                                    )
            header.setCashDcrd(shopIndex, cashCode, shiftNumber, docNum, node.currentConverter.earnCardNumber, earnSumCheck)
        # CASHDCRD
        for cardNumber in discountCards:
            header.setCashDcrd(shopIndex, cashCode, shiftNumber, docNum, cardNumber, discountCards[cardNumber])

        log.info(u"смена №%s cashcode №%s document №%s " % (shiftNumber, str(cash.code), str(check['docNum'])))

    # CURRESTS
    header.setCurrests(currests)

    # CURMONEY
    header.setCurMoney(curMoneys)
    header.close()

    statusFileName = "ready"
    statusDir = workDirectory
    if currentSales:
        statusFileName += "Online"
        statusDir = os.path.dirname(workDirectory)

    converterSalesReadyFile = open('%s/%s' % (statusDir, statusFileName), 'w')
    converterSalesReadyFile.close()
    salesReadyFile = open('%s/%s' % (os.path.dirname(os.path.dirname(statusDir)), statusFileName), 'w')
    salesReadyFile.close()

    log.info(u"Конвертирование завершено")

def sendSales(cash, node, workDirectory):
    log = logger.Logger().getLoggerUnload(node.id)
    return filesmodify.safeMoveDirectory(log, workDirectory, node.currentConverter.path, backupDir = cash.backup_dir, rewrite=node.currentConverter.rewriteDestFile, maxCountFiles=cash.maxCountFiles, actualCountDays=cash.actualCountDays)

def sendOnlineSales(cash, node, workDirectory):
    log = logger.Logger().getLoggerUnload(node.id)
    return filesmodify.simpleCopyFiles(workDirectory, node.currentConverter.currentSalesPath, log), []

def deleteCurrentSales(node, workDirectory):
    log = logger.Logger().getLoggerUnload(node.id)
    return filesmodify.removeFilesInDirectory(log, node.currentConverter.currentSalesPath) and \
           filesmodify.removeFilesInDirectory(log, workDirectory)
