# -*- coding: UTF-8 -*-

from sqlUnloader import SQLUnloader, CaseInsensitiveDict

class DocumentUnloader(SQLUnloader):

    discountTypes = {1:'percent', 2:'summ', 3:'index', 4:'price', 5:'bonus'}
    discountModes = {1:'automatic', 2:'byCard', 3:'fixed', 4:'byPaymentCard', 5:'byDiscKit', 6:'byChange', 7:'byPayment', 8:'certificate', 9:'byCoupon'}
    quantMode = {1:'default', 2:'manual', 3:'fromBarCode', 4:'fromScales', 5:'external', 6:'softCheck'}
    bcodeMode = {1:'manual', 2:'scanner', 3:'hotkey', 4:'selected', 5:'quickCode', 6:'complect', 7:'external', 8:'mcr', 9:'softCheck'}

    def __init__(self, connection, log):
        self.cursor = connection.cursor()
        self.tdata = connection.cursor()
        self.tsdata = connection.cursor()
        self.mdata = connection.cursor()
        self.cdata = connection.cursor()
        self.discdata = connection.cursor()
        self.bonusdata = connection.cursor()
        self.carddata = connection.cursor()
        self.deptdata = connection.cursor()
        self.clientdata = connection.cursor()
        self.paymentdata = connection.cursor()
        self.aspectdata = connection.cursor()
        self.buttondata = connection.cursor()
        self.log = log
        
    def produceMDataObject(self, dict_lower):
        '''
        moneyitem
        '''

        docnum = self.getValue(dict_lower, 'moneyitem.docnum')
        docnum = "" if docnum is None else docnum

        return {
            'dopdata': self.getValue(dict_lower, 'moneyitem.moneyitemid'),
            'docNum': self.getValue(dict_lower, 'moneyitem.documentid'),
            'acode': self.getValue(dict_lower, 'moneyitem.acode'),
            'userCode': self.getValue(dict_lower, 'moneyitem.scode'),
            'posTime': self.getValue(dict_lower, 'moneyitem.mtime'),
            'opCode': self.getValue(dict_lower, 'moneyitem.opcode'),
            'valCode': self.getValue(dict_lower, 'moneyitem.valcode'),
            'nrate': self.getValue(dict_lower, 'moneyitem.nrate'),
            'bond': self.getValue(dict_lower, 'moneyitem.bond'),
            'bond_quant': self.getValue(dict_lower, 'moneyitem.bond_quant'),
            'vsum': self.getValue(dict_lower, 'moneyitem.vsum'),
            'sumB': self.getValue(dict_lower, 'moneyitem.sumb'),
            'sumn': self.getValue(dict_lower, 'moneyitem.sumn'),
            'sume': self.getValue(dict_lower, 'moneyitem.sume'),
            'docnum': docnum,
            'c_link': self.getValue(dict_lower, 'moneyitem.c_link'),
            'frnum': self.getValue(dict_lower, 'moneyitem.frnum'),
            'cardnum': self.getValue(dict_lower, 'moneyitem.cardnum'),
            'cashcode': self.getValue(dict_lower, 'moneyitem.cashcode'),
            'valutOperation': self.getValue(dict_lower, 'moneyitem.valutoperation'),
            'valName': self.getValue(dict_lower, 'moneyitem.valname'),
            'balance': self.getValue(dict_lower, 'moneyitem.balance'),
            'operationId': self.getValue(dict_lower, 'moneyitem.operationid'),
            }

    def produceTDataObject(self, dict_lower):
        '''
        goodsitem
        '''
        return {
            'dopdata': self.getValue(dict_lower, 'goodsitem.goodsitemid'),
            'docNum': self.getValue(dict_lower, 'goodsitem.documentid'),
            'deptCode': self.getValue(dict_lower, 'goodsitem.deptcode'),
            'userCode': self.getValue(dict_lower, 'goodsitem.scode'),
 #           'cashcode': dict_lower[4],
 #           'shift': dict_lower[5],
            'posTime': self.getValue(dict_lower, 'goodsitem.ttime'),
            'opCode': self.getValue(dict_lower, 'goodsitem.opcode'),
            'status': self.getValue(dict_lower, 'goodsitem.status'),
            'barCode': self.getValue(dict_lower, 'goodsitem.bcode'),
            'name': self.getValue(dict_lower, 'goodsitem.name'),
            'articul': self.getValue(dict_lower, 'goodsitem.articul'),
            'measureCode': self.getValue(dict_lower, 'goodsitem.measure'),
            'bcodeMode': self.bcodeMode.get(self.getValue(dict_lower, 'goodsitem.bcode_mode'), 'unknown'),#decrypt
            'bcode_main': self.getValue(dict_lower, 'goodsitem.bcode_main'),#decrypt
            'quant': self.getValue(dict_lower, 'goodsitem.bquant'),
            'quantMode': self.quantMode.get(self.getValue(dict_lower, 'goodsitem.bquant_mode'), 'unknown'),#decrypt
            'ost_modif': self.getValue(dict_lower, 'goodsitem.ost_modif'),
            'accountingQuant': self.getValue(dict_lower, 'goodsitem.cquant'),
            'priceType': self.getValue(dict_lower, 'goodsitem.pricetype'),
            'pricevcode': self.getValue(dict_lower, 'goodsitem.pricevcode'),
            'price': self.getValue(dict_lower, 'goodsitem.price'),
            'minPrice': self.getValue(dict_lower, 'goodsitem.minprice'),
            'finalPrice': self.getValue(dict_lower, 'goodsitem.pricei'),
            'priceMode': self.getValue(dict_lower, 'goodsitem.price_mode'),#decrypt
            'priceIndex': self.getValue(dict_lower, 'goodsitem.tindex'),
            'disc_perc': self.getValue(dict_lower, 'goodsitem.disc_perc'),
            'disc_abs': self.getValue(dict_lower, 'goodsitem.disc_abs'),
            'modSum': self.getValue(dict_lower, 'goodsitem.sumi'),
            'posSum': self.getValue(dict_lower, 'goodsitem.sumb'),
            'baseSum': self.getValue(dict_lower, 'goodsitem.sumn'),
            'sume': self.getValue(dict_lower, 'goodsitem.sume'),
            'vatcode1': self.getValue(dict_lower, 'goodsitem.vatcode1'),
            'vatrate1': self.getValue(dict_lower, 'goodsitem.vatrate1'),
            'vatsum1': self.getValue(dict_lower, 'goodsitem.vatsum1'),
            'vatcode2': self.getValue(dict_lower, 'goodsitem.vatcode2'),
            'vatrate2': self.getValue(dict_lower, 'goodsitem.vatrate2'),
            'vatsum2': self.getValue(dict_lower, 'goodsitem.vatsum2'),
            'vatcode3': self.getValue(dict_lower, 'goodsitem.vatcode3'),
            'vatrate3': self.getValue(dict_lower, 'goodsitem.vatrate3'),
            'vatsum3': self.getValue(dict_lower, 'goodsitem.vatsum3'),
            'vatcode4': self.getValue(dict_lower, 'goodsitem.vatcode4'),
            'vatrate4': self.getValue(dict_lower, 'goodsitem.vatrate4'),
            'vatsum4': self.getValue(dict_lower, 'goodsitem.vatsum4'),
            'vatcode5': self.getValue(dict_lower, 'goodsitem.vatcode5'),
            'vatrate5': self.getValue(dict_lower, 'goodsitem.vatrate5'),
            'vatsum5': self.getValue(dict_lower, 'goodsitem.vatsum5'),
            'docnum': self.getValue(dict_lower, 'goodsitem.docnum'),
            'c_link': self.getValue(dict_lower, 'goodsitem.c_link'),
            'inventCode': self.getValue(dict_lower, 'goodsitem.code'),
            'posNum': self.getValue(dict_lower, 'goodsitem.posnum'),
            'frnum': self.getValue(dict_lower, 'goodsitem.frnum'),
            'extendetoptions': self.getValue(dict_lower, 'goodsitem.extendetoptions'),
            'opid': self.getValue(dict_lower, 'goodsitem.opid'),
            'buttonid': self.getValue(dict_lower, 'goodsitem.buttonid'),
            'paymentitemid': self.getValue(dict_lower, 'goodsitem.paymentitemid'),
            'departmentid': self.getValue(dict_lower, 'goodsitem.departmentid'),
            'taramode': self.getValue(dict_lower, 'goodsitem.taramode'),
            'taracapacity': self.getValue(dict_lower, 'goodsitem.taracapacity'),
            'cashcode': self.getValue(dict_lower, 'goodsitem.cashcode'),
            'extdocid': self.getValue(dict_lower, 'goodsitem.extdocid'),
            'excisemark': self.getValue(dict_lower, 'goodsitem.excisemark'),
            'inn': self.getValue(dict_lower, 'goodsitem.inn'),
            'kpp': self.getValue(dict_lower, 'goodsitem.kpp'),
            'alcoholPercent': self.getValue(dict_lower, 'goodsitem.alcoholpercent'),
            'consultantid': self.getValue(dict_lower, 'goodsitem.consultantid'),
            'additionalexcisemark': self.getValue(dict_lower, 'goodsitem.additionalexcisemark'),
            'alcocode': self.getValue(dict_lower, 'goodsitem.alcocode'),
            }

    def produceTSDataObject(self, dict_lower):
        '''
        stornogoodsitem
        '''
        return {
            'dopdata': self.getValue(dict_lower, 'stornogoodsitem.stornogoodsitemid'),
            'docNum': self.getValue(dict_lower, 'stornogoodsitem.documentid'),
            'deptCode': self.getValue(dict_lower, 'stornogoodsitem.deptcode'),
            'userCode': self.getValue(dict_lower, 'stornogoodsitem.scode'),
 #           'cashcode': dict_lower[4],
 #           'shift': dict_lower[5],
            'posTime': self.getValue(dict_lower, 'stornogoodsitem.tstime'),
            'opCode': self.getValue(dict_lower, 'stornogoodsitem.opcode'),
            'status': self.getValue(dict_lower, 'stornogoodsitem.status'),
            'barCode': self.getValue(dict_lower, 'stornogoodsitem.bcode'),
            'name': self.getValue(dict_lower, 'stornogoodsitem.name'),
            'articul': self.getValue(dict_lower, 'stornogoodsitem.articul'),
            'measureCode': self.getValue(dict_lower, 'stornogoodsitem.measure'),
            'bcodeMode': self.bcodeMode.get(self.getValue(dict_lower, 'stornogoodsitem.bcode_mode'), 'unknown'),#decrypt
            'bcode_main': self.getValue(dict_lower, 'stornogoodsitem.bcode_main'),#decrypt
            'quant': self.getValue(dict_lower, 'stornogoodsitem.bquant'),
            'quantMode': self.quantMode.get(self.getValue(dict_lower, 'stornogoodsitem.bquant_mode'), 'unknown'),#decrypt
            'ost_modif': self.getValue(dict_lower, 'stornogoodsitem.ost_modif'),
            'accountingQuant': self.getValue(dict_lower, 'stornogoodsitem.cquant'),
            'priceType': self.getValue(dict_lower, 'stornogoodsitem.pricetype'),
            'pricevcode': self.getValue(dict_lower, 'stornogoodsitem.pricevcode'),
            'price': self.getValue(dict_lower, 'stornogoodsitem.price'),
            'minPrice': self.getValue(dict_lower, 'stornogoodsitem.minprice'),
            'finalPrice': self.getValue(dict_lower, 'stornogoodsitem.pricei'),
            'priceMode': self.getValue(dict_lower, 'stornogoodsitem.price_mode'),#decrypt
            'priceIndex': self.getValue(dict_lower, 'stornogoodsitem.tsindex'),
            'disc_perc': self.getValue(dict_lower, 'stornogoodsitem.disc_perc'),
            'disc_abs': self.getValue(dict_lower, 'stornogoodsitem.disc_abs'),
            'modSum': self.getValue(dict_lower, 'stornogoodsitem.sumi'),
            'posSum': self.getValue(dict_lower, 'stornogoodsitem.sumb'),
            'baseSum': self.getValue(dict_lower, 'stornogoodsitem.sumn'),
            'sume': self.getValue(dict_lower, 'stornogoodsitem.sume'),
            'vatcode1': self.getValue(dict_lower, 'stornogoodsitem.vatcode1'),
            'vatrate1': self.getValue(dict_lower, 'stornogoodsitem.vatrate1'),
            'vatsum1': self.getValue(dict_lower, 'stornogoodsitem.vatsum1'),
            'vatcode2': self.getValue(dict_lower, 'stornogoodsitem.vatcode2'),
            'vatrate2': self.getValue(dict_lower, 'stornogoodsitem.vatrate2'),
            'vatsum2': self.getValue(dict_lower, 'stornogoodsitem.vatsum2'),
            'vatcode3': self.getValue(dict_lower, 'stornogoodsitem.vatcode3'),
            'vatrate3': self.getValue(dict_lower, 'stornogoodsitem.vatrate3'),
            'vatsum3': self.getValue(dict_lower, 'stornogoodsitem.vatsum3'),
            'vatcode4': self.getValue(dict_lower, 'stornogoodsitem.vatcode4'),
            'vatrate4': self.getValue(dict_lower, 'stornogoodsitem.vatrate4'),
            'vatsum4': self.getValue(dict_lower, 'stornogoodsitem.vatsum4'),
            'vatcode5': self.getValue(dict_lower, 'stornogoodsitem.vatcode5'),
            'vatrate5': self.getValue(dict_lower, 'stornogoodsitem.vatrate5'),
            'vatsum5': self.getValue(dict_lower, 'stornogoodsitem.vatsum5'),
            'docnum': self.getValue(dict_lower, 'stornogoodsitem.docnum'),
            'c_link': self.getValue(dict_lower, 'stornogoodsitem.c_link'),
            'inventCode': self.getValue(dict_lower, 'stornogoodsitem.code'),
            'posNum': self.getValue(dict_lower, 'stornogoodsitem.posnum'),
            'timeStorno': self.getValue(dict_lower, 'stornogoodsitem.timestorno'),
            'frnum': self.getValue(dict_lower, 'stornogoodsitem.frnum'),
            'extendetoptions': self.getValue(dict_lower, 'stornogoodsitem.extendetoptions'),
            'opid': self.getValue(dict_lower, 'stornogoodsitem.opid'),
            'buttonid': self.getValue(dict_lower, 'stornogoodsitem.buttonid'),
            'paymentitemid': self.getValue(dict_lower, 'stornogoodsitem.paymentitemid'),
            'departmentid': self.getValue(dict_lower, 'stornogoodsitem.departmentid'),
            'taramode': self.getValue(dict_lower, 'stornogoodsitem.taramode'),
            'taracapacity': self.getValue(dict_lower, 'stornogoodsitem.taracapacity'),
            'cashcode': self.getValue(dict_lower, 'stornogoodsitem.cashcode'),
            'extdocid': self.getValue(dict_lower, 'stornogoodsitem.extdocid'),
            'excisemark': self.getValue(dict_lower, 'stornogoodsitem.excisemark'),
            'inn': self.getValue(dict_lower, 'stornogoodsitem.inn'),
            'kpp': self.getValue(dict_lower, 'stornogoodsitem.kpp'),
            'alcoholPercent': self.getValue(dict_lower, 'stornogoodsitem.alcoholpercent'),
            'consultantid': self.getValue(dict_lower, 'stornogoodsitem.consultantid'),
            'additionalexcisemark': self.getValue(dict_lower, 'stornogoodsitem.additionalexcisemark'),
            }
        
    def produceCDataObject(self, dict_lower):
        '''
        document
        '''
        return {
            'dopdata': self.getValue(dict_lower, 'document.documentid'),
            'docNum': self.getValue(dict_lower, 'document.checknum'),
            'docType': self.getValue(dict_lower, 'document.doctype'),
            'deptCode': self.getValue(dict_lower, 'document.dept'),
            'userCode': self.getValue(dict_lower, 'document.scode'),
            'cashCode': self.getValue(dict_lower, 'document.cashcode'),
            'shift': self.getValue(dict_lower, 'workshift.shiftnum'),
            'timeBeg': self.getValue(dict_lower, 'document.time_beg'),
            'timeEnd': self.getValue(dict_lower, 'document.time_end'),
            'vbrate': self.getValue(dict_lower, 'document.vbrate'),
            'verate': self.getValue(dict_lower, 'document.verate'),
            'baseSum': self.getValue(dict_lower, 'document.sum1'),
            'modSum': self.getValue(dict_lower, 'document.sum2'),
            'sum2m': self.getValue(dict_lower, 'document.sum2m'),
            'summode': self.getValue(dict_lower, 'document.summode'),
            'sumtype': self.getValue(dict_lower, 'document.sumtype'),
            'disc_perc': self.getValue(dict_lower, 'document.disc_perc'),
            'disc_abs': self.getValue(dict_lower, 'document.disc_abs'),
            'docSum': self.getValue(dict_lower, 'document.sumb'),
            'sumn': self.getValue(dict_lower, 'document.sumn'),
            'sume': self.getValue(dict_lower, 'document.sume'),
            'vatsum': self.getValue(dict_lower, 'document.vatsum'),
            'docnum': self.getValue(dict_lower, 'document.docnum'),
            'c_link': self.getValue(dict_lower, 'document.c_link'),
            'closed': self.getValue(dict_lower, 'document.closed'),
            'shiftType': self.getValue(dict_lower, 'workshift.shifttype'),
            'opid': self.getValue(dict_lower, 'document.opid'),
            'dateincrement': self.getValue(dict_lower, 'document.dateincrement'),
            'buttonid': self.getValue(dict_lower, 'document.buttonid'),
            'linkeddocumentid': self.getValue(dict_lower, 'document.linkeddocumentid'),
            'departmentid': self.getValue(dict_lower, 'document.departmentid'),
            'clientitemid': self.getValue(dict_lower, 'document.clientitemid'),
            'identifier': self.getValue(dict_lower, 'document.identifier'),
            'actorCode': self.getValue(dict_lower, 'document.actorcode'),
            'moneyouttype': self.getValue(dict_lower, 'document.moneyouttype'),
            'urlEgais': self.getValue(dict_lower, 'document.url_egais'),
            'digitalSignatureEgais': self.getValue(dict_lower, 'document.digital_signature_egais'),
            'noPdfUrlEgais': self.getValue(dict_lower, 'document.nopdfUrlEgais'),
            'noPdfDigitalSignatureEgais': self.getValue(dict_lower, 'document.nopdfDigitalSignatureEgais'),
    }

    def produceDiscDataObject(self, dict_lower):
        return {
                'discitemid': self.getValue(dict_lower, 'discitem.discitemid'),
                'goodsitemid': self.getValue(dict_lower, 'discitem.goodsitemid'),                
                'positionnum': self.getValue(dict_lower, 'discitem.positionnum'),
                'userCode': self.getValue(dict_lower, 'discitem.scode'),
                'posTime': self.getValue(dict_lower, 'discitem.dtime'),
                'isPositionDiscount': self.getValue(dict_lower, 'discitem.ispositiondiscount') == 1,
                'discNumber': self.getValue(dict_lower, 'discitem.discnumber'),
                'discMode':self.discountModes.get(self.getValue(dict_lower, 'discitem.discmode'), 'unknown'),
                'discType':self.discountTypes.get(self.getValue(dict_lower, 'discitem.disctype'), 'unknown'),
                'discSize': self.getValue(dict_lower, 'discitem.discsize'),
                'discSum': self.getValue(dict_lower, 'discitem.discsum'),
                'discName': self.getValue(dict_lower, 'discitem.discname'),
                'discCode': self.getValue(dict_lower, 'discitem.disccode'),
                'discScheme': self.getValue(dict_lower, 'discitem.discscheme'),
                'checkSum': self.getValue(dict_lower, 'discitem.checksum'),
                'accumulateSum': self.getValue(dict_lower, 'discitem.accumulatesum'),
                'cardNumber': self.getValue(dict_lower, 'discitem.cardnumber'),
                'cardName': self.getValue(dict_lower, 'discitem.cardname'),
                'cardType': self.getValue(dict_lower, 'discitem.cardtype'),
                'carditemid': self.getValue(dict_lower, 'discitem.carditemid'),
                'cashcode': self.getValue(dict_lower, 'discitem.cashcode'),
                }

    def produceBonusDataObject(self, dict_lower):
        return {
                'bonusitemid': self.getValue(dict_lower, 'bonusitem.bonusitemid'),
                'documentid': self.getValue(dict_lower, 'bonusitem.documentid'),
                'goodsitemid': self.getValue(dict_lower, 'bonusitem.goodsitemid'),
                'posnum': self.getValue(dict_lower, 'bonusitem.posnum'),
                'opCode': self.getValue(dict_lower, 'bonusitem.opcode'),
                'cardNumber': self.getValue(dict_lower, 'bonusitem.cardnumber'),
                'amount': self.getValue(dict_lower, 'bonusitem.amount'),
                'posTime': self.getValue(dict_lower, 'bonusitem.btime'),
                'valCode': self.getValue(dict_lower, 'bonusitem.valcode'),
                'initialSum': self.getValue(dict_lower, 'bonusitem.initialsum'),
                'scode': self.getValue(dict_lower, 'bonusitem.scode'),
                'cashcode': self.getValue(dict_lower, 'bonusitem.cashcode'),
                'campaignid': self.getValue(dict_lower, 'bonusitem.campaignid'),
                'ispositionbonus': self.getValue(dict_lower, 'bonusitem.ispositionbonus'),
                'bonusnumber': self.getValue(dict_lower, 'bonusitem.bonusnumber'),
                }

    def produceCardDataObject(self, dict_lower):
        return {
                'carditemid': self.getValue(dict_lower, 'carditem.carditemid'),
                'documentid': self.getValue(dict_lower, 'carditem.documentid'),
                'idCard': self.getValue(dict_lower, 'carditem.idcard'),
                'idCardGroup': self.getValue(dict_lower, 'carditem.idcardgroup'),
                'idClient': self.getValue(dict_lower, 'carditem.idclient'),
                'number': self.getValue(dict_lower, 'carditem.number'),
                'validityDateBeg': self.getValue(dict_lower, 'carditem.validitydatebeg'),
                'validityDateEnd': self.getValue(dict_lower, 'carditem.validitydateend'),
                'cardSum': self.getValue(dict_lower, 'carditem.cardSum'),
                'nameCardGroup': self.getValue(dict_lower, 'carditem.namecardgroup'),
                'textCardGroup': self.getValue(dict_lower, 'carditem.textcardgroup'),
                'cardMode': self.getValue(dict_lower, 'carditem.cardmode'),
                'prefixBeg': self.getValue(dict_lower, 'carditem.prefixbeg'),
                'prefixEnd': self.getValue(dict_lower, 'carditem.prefixend'),
                'lenBeg': self.getValue(dict_lower, 'carditem.lenbeg'),
                'lenEnd': self.getValue(dict_lower, 'carditem.lenend'),
                'regexPattern': self.getValue(dict_lower, 'carditem.regexpattern'),
                'inputMask': self.getValue(dict_lower, 'carditem.inputmask'),
                'regexRule': self.getValue(dict_lower, 'carditem.regexrule'),
                'valutCode': self.getValue(dict_lower, 'carditem.valutcode'),
                'discountType': self.getValue(dict_lower, 'carditem.discounttype'),
                'discountValue': self.getValue(dict_lower, 'carditem.discountvalue'),
                'nameClient': self.getValue(dict_lower, 'carditem.nameclient'),
                'textClient': self.getValue(dict_lower, 'carditem.textclient'),
                'sex': self.getValue(dict_lower, 'carditem.sex'),
                'birthday': self.getValue(dict_lower, 'carditem.birthday'),
                'specialDate1': self.getValue(dict_lower, 'carditem.specialdate1'),
                'specialDate2': self.getValue(dict_lower, 'carditem.specialdate2'),
                'specialDate3': self.getValue(dict_lower, 'carditem.specialdate3'),
                'specialDate1Name': self.getValue(dict_lower, 'carditem.specialdate1name'),
                'specialDate2Name': self.getValue(dict_lower, 'carditem.specialdate2name'),
                'specialDate3Name': self.getValue(dict_lower, 'carditem.specialdate3name'),
                'zipcode': self.getValue(dict_lower, 'carditem.zipcode'),
                'address': self.getValue(dict_lower, 'carditem.address'),
                'email': self.getValue(dict_lower, 'carditem.email'),
                'webpage': self.getValue(dict_lower, 'carditem.webpage'),
                'phoneNumber': self.getValue(dict_lower, 'carditem.phonenumber'),
                'inn': self.getValue(dict_lower, 'carditem.inn'),
                'document': self.getValue(dict_lower, 'carditem.document'),
                'okpo': self.getValue(dict_lower, 'carditem.okpo'),
                'okpd': self.getValue(dict_lower, 'carditem.okpd'),
                'occupation': self.getValue(dict_lower, 'carditem.occupation'),
                'childrenCount': self.getValue(dict_lower, 'carditem.childrencount'),
                'extendedOptions': self.getValue(dict_lower, 'carditem.extendedoptions'),
                'posTime': self.getValue(dict_lower, 'carditem.itemtime'),
                'itemorder': self.getValue(dict_lower, 'carditem.itemorder'),
                'inputSource': self.getValue(dict_lower, 'carditem.inputsource'),
                'bonusBalance': self.getValue(dict_lower, 'carditem.bonusbalance'),
                'positionnumber': self.getValue(dict_lower, 'carditem.positionnumber'),
                'clientitemid': self.getValue(dict_lower, 'carditem.clientitemid'),
                'cashcode': self.getValue(dict_lower, 'carditem.cashcode'),
                'cardstatus': self.getValue(dict_lower, 'carditem.cardstatus'),
                }

    def produceClientDataObject(self, dict_lower):
        return{
                 'clientitemid': self.getValue(dict_lower, 'client.clientitemid'),
                 'idclient': self.getValue(dict_lower, 'client.idclient'),
                 'name': self.getValue(dict_lower, 'client.name'),
                 'text': self.getValue(dict_lower, 'client.text'),
                 'sex': self.getValue(dict_lower, 'client.sex'),
                 'birthday': self.getValue(dict_lower, 'client.birthday'),
                 'specialdate1': self.getValue(dict_lower, 'client.specialdate1'),
                 'specialdate2': self.getValue(dict_lower, 'client.specialdate2'),
                 'specialdate3': self.getValue(dict_lower, 'client.specialdate3'),
                 'specialdate1name': self.getValue(dict_lower, 'client.specialdate1name'),
                 'specialdate2name': self.getValue(dict_lower, 'client.specialdate2name'),
                 'specialdate3name': self.getValue(dict_lower, 'client.specialdate3name'),
                 'zipcode': self.getValue(dict_lower, 'client.zipcode'),
                 'address': self.getValue(dict_lower, 'client.address'),
                 'email': self.getValue(dict_lower, 'client.email'),
                 'webpage': self.getValue(dict_lower, 'client.webpage'),
                 'phonenumber': self.getValue(dict_lower, 'client.phonenumber'),
                 'inn': self.getValue(dict_lower, 'client.inn'),
                 'document': self.getValue(dict_lower, 'client.document'),
                 'okpo': self.getValue(dict_lower, 'client.okpo'),
                 'okpd': self.getValue(dict_lower, 'client.okpd'),
                 'occupation': self.getValue(dict_lower, 'client.occupation'),
                 'childrencount': self.getValue(dict_lower, 'client.childrencount'),
                 'extendedoptions': self.getValue(dict_lower, 'client.extendedoptions'),
                 'cashcode': self.getValue(dict_lower, 'client.cashcode'),
               }
  
    def produceDeptDataObject(self,dict_lower):
        return{
               'departmentid': self.getValue(dict_lower, 'department.departmentid'),
               'departmentcode': self.getValue(dict_lower, 'department.departmentcode'),
               'departmentname': self.getValue(dict_lower, 'department.departmentname'),
               'clientitemid': self.getValue(dict_lower, 'department.clientitemid'),
               'documentprefix': self.getValue(dict_lower, 'department.documentprefix'),
               'documentnumber': self.getValue(dict_lower, 'department.documentnumber'),
               'cashcode': self.getValue(dict_lower, 'department.cashcode'),
               }

    def producePaymentDataObject(self,dict_lower):
        return{
               'paymentitemid': self.getValue(dict_lower, 'paymentitem.paymentitemid'),
               'paymentid': self.getValue(dict_lower, 'paymentitem.paymentid'),
               'creationtime': self.getValue(dict_lower, 'paymentitem.creationtime'),
               'accountnumber': self.getValue(dict_lower, 'paymentitem.accountnumber'),
               'amount': self.getValue(dict_lower, 'paymentitem.amount'),
               'commissionagent': self.getValue(dict_lower, 'paymentitem.commissionagent'),
               'commissionsubagent': self.getValue(dict_lower, 'paymentitem.commissionsubagent'),
               'terminalcode': self.getValue(dict_lower, 'paymentitem.terminalcode'),
               'terminaladdress': self.getValue(dict_lower, 'paymentitem.terminaladdress'),
               'providercode': self.getValue(dict_lower, 'paymentitem.providercode'),
               'providername': self.getValue(dict_lower, 'paymentitem.providername'),
               'providerinn': self.getValue(dict_lower, 'paymentitem.providerinn'),
               'providerphone': self.getValue(dict_lower, 'paymentitem.providerphone'),
               'provideraddress': self.getValue(dict_lower, 'paymentitem.provideraddress'),
               'categorycode': self.getValue(dict_lower, 'paymentitem.categorycode'),
               'categoryname': self.getValue(dict_lower, 'paymentitem.categoryname'),
               'agentcode': self.getValue(dict_lower, 'paymentitem.agentcode'),
               'agentname': self.getValue(dict_lower, 'paymentitem.agentname'),
               'agentinn': self.getValue(dict_lower, 'paymentitem.agentinn'),
               'agentphone': self.getValue(dict_lower, 'paymentitem.agentphone'),
               'agentaddress': self.getValue(dict_lower, 'paymentitem.agentaddress'),
               'subagentcode': self.getValue(dict_lower, 'paymentitem.subagentcode'),
               'subagentname': self.getValue(dict_lower, 'paymentitem.subagentname'),
               'subagentinn': self.getValue(dict_lower, 'paymentitem.subagentinn'),
               'subagentphone': self.getValue(dict_lower, 'paymentitem.subagentphone'),
               'subagentaddress': self.getValue(dict_lower, 'paymentitem.subagentaddress'),
               'cardnumber': self.getValue(dict_lower, 'paymentitem.cardnumber'),
               'cashcode': self.getValue(dict_lower, 'paymentitem.cashcode'),
               }

    def produceAspectDataObject(self, dict_lower):
        return{
               'aspectitemid': self.getValue(dict_lower, 'aspectitem.aspectitemid'),
               'goodsitemid': self.getValue(dict_lower, 'aspectitem.goodsitemid'),
               'aspectcode': self.getValue(dict_lower, 'aspectitem.aspectcode'),
               'aspectvaluecode': self.getValue(dict_lower, 'aspectitem.aspectvaluecode'),
               'cashcode': self.getValue(dict_lower, 'aspectitem.cashcode'),
               }

    def produceButtonDataObject(self,dict_lower):
        return{
               'buttonid': self.getValue(dict_lower, 'button.buttonid'),
               'code': self.getValue(dict_lower, 'button.code'),
               'name': self.getValue(dict_lower, 'button.name'),
               'tag': self.getValue(dict_lower, 'button.tag'),
               'rateobject': self.getValue(dict_lower, 'button.rateobject'),
               'ratetype': self.getValue(dict_lower, 'button.ratetype'),
               'ratevalue': self.getValue(dict_lower, 'button.ratevalue'),
               'maxratevalue': self.getValue(dict_lower, 'button.maxratevalue'),
               'minratevalue': self.getValue(dict_lower, 'button.minratevalue'),
               'inputvalue': self.getValue(dict_lower, 'button.inputvalue'),
               'cashcode': self.getValue(dict_lower, 'button.cashcode'),
               }

    def unloadSaleObjects(self):
        i = -1
        for i, row in enumerate(self.cdata):
            dict_lower = CaseInsensitiveDict(row)
            cdata = self.produceCDataObject(dict_lower)
            documentid = str(self.getValue(dict_lower, 'document.documentid'))
            cdata["inventPositions"] = [ p for p in self.unloadTData(documentid) ]
            cdata["stornoPositions"] = [ p for p in self.unloadTSData(documentid) ]
            cdata["moneyPositions"] = [ p for p in self.unloadMData(documentid) ]
            cdata['bonusPositions'] = [pos for pos in self.unloadBonusDataCheck(documentid)]
            cdata['cardPositions'] = [pos for pos in self.unloadCardDataCheck(documentid)]
            cdata['deptPositions'] = [pos for pos in self.unloadDeptData(documentid)]
            cdata['clientPositions'] = [pos for pos in self.unloadClientData(documentid)]
            cdata['paymentPositions'] = [pos for pos in self.unloadPaymentData(documentid)]
            cdata['buttonPositions'] = [pos for pos in self.unloadButtonData(documentid)]
            yield cdata
        self.log.info("total: %d" % (i+1))

    # Выгрузка всех документов для указанных смен
    def unloadDocuments(self, shifts):
        if len(shifts) > 2:
            self.log.info('unload documents for shifts %s' % shifts)
            self.cdata.execute("select * from document, workshift where \
                                document.workshiftid IN %s and workshift.workshiftid = document.workshiftid and closed <> 0" % shifts)
            for o in self.unloadSaleObjects():
                yield o

    # Получение измененных документов для указанных смен
    def getChangedDocuments(self, shifts):
        if len(shifts) > 2:
            self.log.info('get changed documents')
            query = "select documentid from document where document.workshiftid IN %s and document.changed = 1 and closed <> 0" % shifts
            self.cursor.execute(query)
            return self.cursor.fetchall()
        return ()

    # Выгрузка измененных документов
    def unloadChangedDocuments(self, documentIds):
        if len(documentIds) > 2:
            self.log.info('unload changed documents')
            self.cdata.execute("select * from document, workshift where \
                                document.documentid IN %s and workshift.workshiftid = document.workshiftid and document.changed = 1 and closed <> 0" % documentIds)
            for o in self.unloadSaleObjects():
                yield o

    # Выгрузка документов для указанных смен по диапазону времени
    def unloadDocumentsByDate(self, shifts, fromDate, toDate):
        if len(shifts) > 2:
            self.log.info('unload documents from %s to %s' % (str(fromDate), str(toDate)))
            self.cdata.execute("select * from document, workshift where document.workshiftid IN %s and workshift.workshiftid = document.workshiftid \
                                and document.TIME_END > '%s' and document.TIME_END <= '%s' and closed <> 0 order by document.TIME_END" % (shifts, fromDate, toDate))
            for o in self.unloadSaleObjects():
                yield o


    def unloadTData(self, documentid):
        self.tdata.execute('select * from goodsitem where documentid=%s', documentid)
        for row in self.tdata:
            dict_lower = CaseInsensitiveDict(row)
            tdata = self.produceTDataObject(dict_lower)
            tdata["taxes"] = []
            for i in xrange(5):
                tdata["taxes"].append({"taxCode": self.getValue(dict_lower, 'goodsitem.vatcode%s' % (i+1)), 
                                       "taxRate": self.getValue(dict_lower, 'goodsitem.vatrate%s' % (i+1)), 
                                       "taxSum": self.getValue(dict_lower, 'goodsitem.vatsum%s' % (i+1))})
            goodsitemid = self.getValue(dict_lower, 'goodsitem.goodsitemid')
            tdata['discountPositions'] = [pos for pos in self.unloadDiscData(goodsitemid)]
            tdata['bonusPositions'] = [pos for pos in self.unloadBonusData(goodsitemid)]
            tdata['aspectPosition'] = [pos for pos in self.unloadAspectData(goodsitemid)]
            yield tdata


    def unloadDiscData(self, goodsitemid):
        self.discdata.execute('select * from discitem where goodsitemid=%s', goodsitemid)
        for row in self.discdata:
            dict_lower = CaseInsensitiveDict(row)
            discdata = self.produceDiscDataObject(dict_lower)
            yield discdata


    def unloadBonusData(self, goodsitemid):
        self.bonusdata.execute('select * from bonusitem where goodsitemid = %s order by posnum', goodsitemid)
        for row in self.bonusdata:
            dict_lower = CaseInsensitiveDict(row)
            bonusObject = self.produceBonusDataObject(dict_lower)
            yield bonusObject


    def unloadAspectData(self, goodsitemid):
        self.aspectdata.execute('SELECT * FROM aspectitem WHERE goodsitemid = %s', goodsitemid)
        for row in self.aspectdata:
            dict_lower = CaseInsensitiveDict(row)
            aspectObject = self.produceAspectDataObject(dict_lower)
            yield aspectObject


    def unloadTSData(self, documentid):
        self.tsdata.execute('select * from stornogoodsitem where documentid=%s', documentid)
        for row in self.tsdata:
            dict_lower = CaseInsensitiveDict(row)
            tsdata = self.produceTSDataObject(dict_lower)
            tsdata["taxes"] = []
            for i in xrange(5):
                tsdata["taxes"].append({"taxCode": self.getValue(dict_lower, 'stornogoodsitem.vatcode%s' % (i+1)), 
                                        "taxRate": self.getValue(dict_lower, 'stornogoodsitem.vatrate%s' % (i+1)), 
                                        "taxSum": self.getValue(dict_lower, 'stornogoodsitem.vatsum%s' % (i+1))})
            yield tsdata


    def unloadMData(self, documentid):
        self.mdata.execute('select * from moneyitem where documentid=%s', documentid)
        for row in self.mdata:
            dict_lower = CaseInsensitiveDict(row)
            mdata = self.produceMDataObject(dict_lower)
            yield mdata


    def unloadBonusDataCheck(self, documentid):
        self.bonusdata.execute('select * from bonusitem where documentid=%s and goodsitemid is null order by posnum', documentid)
        for row in self.bonusdata:
            dict_lower = CaseInsensitiveDict(row)
            bonusObject = self.produceBonusDataObject(dict_lower)
            yield bonusObject


    def unloadCardDataCheck(self, documentid):
        self.carddata.execute('select * from carditem where documentid = %s', documentid)
        for row in self.carddata:
            dict_lower = CaseInsensitiveDict(row)
            yield self.produceCardDataObject(dict_lower)


    def unloadDeptData(self,documentid):
        self.deptdata.execute('SELECT * FROM department JOIN (SELECT DISTINCT departmentid FROM goodsitem WHERE documentid = %s \
                                UNION ALL SELECT DISTINCT departmentid FROM stornogoodsitem WHERE documentid = %s \
                                UNION ALL SELECT departmentid FROM document WHERE documentid = %s) t ON department.departmentid = t.departmentid', (documentid,documentid,documentid))
        for row in self.deptdata:
            dict_lower = CaseInsensitiveDict(row)
            deptObject = self.produceDeptDataObject(dict_lower)            
            yield deptObject


    def unloadClientData(self,documentid):
        self.clientdata.execute('SELECT * FROM `client` join (SELECT DISTINCT clientitemid FROM carditem WHERE documentid = %s \
                        UNION ALL SELECT DISTINCT d2.clientitemid FROM document d1 join department d2 on d1.departmentid=d2.departmentid WHERE d1.documentid=%s \
                        UNION ALL SELECT clientitemid FROM document WHERE documentid = %s \
                        UNION ALL SELECT DISTINCT d3.clientitemid FROM goodsitem g join department d3 on g.departmentid=d3.departmentid WHERE g.documentid=%s \
                        UNION ALL SELECT DISTINCT d3.clientitemid FROM stornogoodsitem s join department d3 on s.departmentid = d3.departmentid WHERE s.documentid = %s \
                        ) t on client.clientitemid = t.clientitemid', (documentid,documentid,documentid,documentid,documentid))
        for row in self.clientdata:
            dict_lower = CaseInsensitiveDict(row)
            clientObejct = self.produceClientDataObject(dict_lower)
            yield clientObejct


    def unloadPaymentData(self, documentid):
        self.paymentdata.execute('SELECT * FROM paymentitem JOIN (SELECT DISTINCT paymentitemid FROM goodsitem WHERE documentid = %s UNION ALL SELECT DISTINCT  paymentitemid \
                                FROM stornogoodsitem WHERE documentid = %s) t ON paymentitem.paymentitemid = t.paymentitemid', (documentid, documentid))
        for row in self.paymentdata:
            dict_lower = CaseInsensitiveDict(row)
            paymentObject = self.producePaymentDataObject(dict_lower)
            yield paymentObject


    def unloadButtonData(self, documentid):
        self.buttondata.execute('SELECT * FROM button JOIN (SELECT DISTINCT buttonid FROM goodsitem WHERE documentid = %s UNION ALL SELECT DISTINCT buttonid FROM stornogoodsitem \
                                WHERE documentid = %s UNION ALL SELECT buttonid FROM document WHERE documentid = %s) t USING(buttonid)', (documentid,documentid,documentid))
        for row in self.buttondata:
            dict_lower = CaseInsensitiveDict(row)
            buttonObject = self.produceButtonDataObject(dict_lower)
            yield buttonObject
