# -*- coding: UTF-8 -*-

import sys
import os
import registry
from unload.node.node import Node
from unload.node.nodeparams import DbReader, Cash
from unload.node.nodeparams import AifConverter, SmagConverter, ShtrihConverter, OracleConverter, AtolAstorConverter, AtolConverter, CrystalConverter, NS2000Converter, PilotConverter
from common import macros, logger, config

registryPath = '/linuxcash/cash/data/cash.reg'

log = logger.Logger().getLoggerUnload("main")

shopCode = None
cashCode = None

def readStandartParams(section):

    salePath = config.getString(section, 'salePath', '/tmp/sales/')

    dirName = os.path.dirname(salePath)
    fileName = os.path.basename(salePath)
    dirName = macros.applyMacros(dirName, log, shopcode = shopCode, cashcode = cashCode)
    flagPath = macros.applyMacros(config.getString(section, 'flagPath', '/tmp/sales/sale.req'), log, shopcode = shopCode, cashcode = cashCode)
    rewriteDestFile = config.getBool(section, 'rewriteDestFile', False)
    return os.path.join(dirName, fileName), flagPath, rewriteDestFile

def getConverter(converterName):
    converter = None
    section = 'unload.converter.%s' % converterName
    if converterName == 'aif':
        salePath, flagPath, rewriteDestFile = readStandartParams(section)
        dataType = config.getString(section, 'dataType', 'sales')
        withClosedShifts = config.getBool(section, 'withClosedShifts', True)
        onlineSalePath = macros.applyMacros(config.getString(section, 'onlineSalePath', '/tmp/salesOnline/'), 
                log, shopcode = shopCode, cashcode = cashCode)
        converter = AifConverter(salePath, flag = flagPath, rewriteDestFile = rewriteDestFile, 
                dataType = dataType, withClosedShifts= withClosedShifts, onlineSalePath=onlineSalePath)
    elif converterName == 'shtrih':
        salePath, flagPath, rewriteDestFile = readStandartParams(section)
        useActorCodeForMoneyOperations = config.getBool(section, 'useActorCodeForMoneyOperations', True)
        discountsDetail = config.getBool(section, 'discountsDetail', True)
        unloadShortDateFormat = config.getBool(section, 'unloadShortDateFormat', False)
        unloadCardType = config.getBool(section, 'unloadCardType', True)
        defaultCardType = config.getString(section, 'defaultCardType', '')
        unloadPaymentCommission = config.getBool(section, 'unloadPaymentCommission', False)
        charset = config.getString(section, 'charset', 'cp1251')
        delimiter = config.getString(section,'delimiter', ';')
        unloadPaymentByCard = config.getBool(section,'unloadPaymentByCard', False)
        useTransaction300ForDocuments = config.getBool(section,'useTransaction300ForDocuments', True)
        unloadShiftsFromKkm = config.getBool(section,'unloadShiftsFromKkm', False)
        converter = ShtrihConverter(salePath, discountsDetail = discountsDetail, flag = flagPath, rewriteDestFile = rewriteDestFile,
                                            useActorCodeForMoneyOperations = useActorCodeForMoneyOperations, unloadShortDateFormat = unloadShortDateFormat,
                                            unloadCardType = unloadCardType, defaultCardType = defaultCardType, charset = charset, delimiter = delimiter,
                                            unloadPaymentByCard = unloadPaymentByCard,unloadPaymentCommission = unloadPaymentCommission,
                                            useTransaction300ForDocuments = useTransaction300ForDocuments, unloadShiftsFromKkm=unloadShiftsFromKkm)
    elif converterName == 'atol-astor':
        salePath, flagPath, rewriteDestFile = readStandartParams(section)
        discountsDetail = config.getBool(section, 'discountsDetail', True)
        charset = config.getString(section, 'charset', 'cp1251')
        onlineSalePath = macros.applyMacros(config.getString(section, 'onlineSalePath', '/tmp/salesOnline/'), log, shopcode = shopCode, cashcode = cashCode)
        converter = AtolAstorConverter(salePath, discountsDetail = discountsDetail, flag = flagPath, rewriteDestFile = rewriteDestFile,
                                        charset = charset, onlineSalePath = onlineSalePath)
    elif converterName == 'atol':
        salePath, flagPath, rewriteDestFile = readStandartParams(section)
        discountsDetail = config.getBool(section, 'discountsDetail', True)
        charset = config.getString(section, 'charset', 'cp1251')
        onlineSalePath = macros.applyMacros(config.getString(section, 'onlineSalePath', '/tmp/salesOnline/'), log, shopcode = shopCode, cashcode = cashCode)
        converter = AtolConverter(salePath, discountsDetail = discountsDetail, flag = flagPath, rewriteDestFile = rewriteDestFile,
                                        charset = charset, onlineSalePath = onlineSalePath)
    elif converterName == 'smag':
        salePath, flagPath, rewriteDestFile = readStandartParams(section)
        fileType = config.getString(section, 'fileType', 'dat')
        earnPositionCode = config.getString(section, 'earnPositionCode', '999999')
        earnCardNumber = config.getString(section, 'earnCardNumber', '99999999')
        spendPaymentCode = config.getInt(section, 'spendPaymentCode', 10)
        earnDeptCode = config.getInt(section, 'earnDeptCode', 0)
        currentSalesPath = macros.applyMacros(config.getString(section, 'currentSalesPath', '/tmp/currentSales'), log, shopcode = shopCode, cashcode = cashCode)
        charset = config.getString(section, 'charset', 'cp866')
        appendSaleFiles = config.getBool(section, 'appendSaleFiles', False)
        converter = SmagConverter(salePath, format = fileType, flag = flagPath, rewriteDestFile = rewriteDestFile, 
                                        currentSalesPath = currentSalesPath, charset = charset, earnPositionCode = earnPositionCode, earnDeptCode = earnDeptCode,
                                        earnCardNumber = earnCardNumber, spendPaymentCode = spendPaymentCode, appendSaleFiles = appendSaleFiles)
    elif converterName == 'oracle':
        salePath, flagPath, rewriteDestFile = readStandartParams(section)
        converter = OracleConverter(salePath, flag = flagPath, rewriteDestFile = rewriteDestFile)
    elif converterName == 'crystal':
        salePath, flagPath, rewriteDestFile = readStandartParams(section)
        formdiscounts = config.getBool(section, 'formdiscounts', True)
        converter = CrystalConverter(salePath, flag = flagPath, rewriteDestFile = rewriteDestFile, formdiscounts = formdiscounts)
    elif converterName == 'ns2000':
        salePath, flagPath, rewriteDestFile = readStandartParams(section)
        charset = config.getString(section, 'charset', 'cp1251')
        converter = NS2000Converter(salePath, flag = flagPath, rewriteDestFile = rewriteDestFile,charset = charset)
    elif converterName == 'pilot':
        salePath, flagPath, rewriteDestFile = readStandartParams(section)
        charset = config.getString(section, 'charset', 'cp1251')
        converter = PilotConverter(salePath, flag = flagPath, rewriteDestFile = rewriteDestFile,charset = charset)
    return converter

def getNodes():
    global cashCode
    global shopCode
    reg = registry.Registry(registryPath)
    cashCode = str(reg.cashcode())
    shopCode = str(reg.shopcode())

    if not shopCode or shopCode == '':
        log.error("Cannot get shopCode from %s." % registryPath)
        sys.exit(1)
    
    if not config.read(os.path.normpath(sys.path[0] + "/../config/cash/exchangers.ini")):
        log.error("Configuration file 'exchangers.ini' not found.")
        sys.exit(1)

    convertersInConf = config.getString('UNLOAD', 'converter', 'aif').split(',')
    converters = []
    backupDir = config.getString('UNLOAD', 'backupDir', '/linuxcash/cash/data/backup/sales')
    maxCountFiles = config.getInt('UNLOAD', 'maxCountBackupFiles', 30)
    actualCountDays = config.getInt('UNLOAD', 'actualCountDaysForBackupFiles', 30)

    for converterName in convertersInConf:
        converterName = converterName.strip().lower()
        converter = getConverter(converterName)
        if converter:
            converters.append(converter)

    nodes = [
            Node(
                id = shopCode,
                cashes = [
                            Cash(cashCode, converters, backup_dir = backupDir, maxCountFiles=maxCountFiles, actualCountDays=actualCountDays),
                            ],
                reader = DbReader( 
                                    host = 'localhost',
                                    port = 3306, 
                                    user = 'root', 
                                    passwd = '', 
                                    db = 'documents',
                                    ),
                ),
            ]
    return nodes
