import os, sys
import urllib2
import json
from unload.node.node import Node
from unload.node.nodeparams import DbReader, AifConverter, ShtrihConverter, SmagConverter, Cash, AtolAstorConverter, AtolConverter, OracleConverter, CrystalConverter
from common import macros, logger, config
log = logger.Logger().getLoggerUnload("main")

def getNodes():
    proxy_support = urllib2.ProxyHandler({})
    opener = urllib2.build_opener(proxy_support)
    urllib2.install_opener(opener)
    response = urllib2.urlopen('http://localhost:8080/cashinfo/rest/shops')
    html = response.read()
    shops = json.loads(html)
    nodes = []
    
    maxCountFiles=30
    actualCountDays=30
    if not config.read(sys.path[0] + "/../config/%s/exchangers.ini" % (logger.Logger().location)):
        log.warning("Configuration file 'exchangers.ini' not found. Configure backup properties by default...")
    else:
        maxCountFiles = config.getInt('UNLOAD', 'maxCountBackupFiles', maxCountFiles)
        actualCountDays = config.getInt('UNLOAD', 'actualCountDaysForBackupFiles', actualCountDays)
    
    for shop in shops:
        shopCode = str(shop.get('code'))
        cashes = []
        cashesRest = json.loads(urllib2.urlopen('http://localhost:8080/cashinfo/rest/cashes/%s' % shopCode).read())
        for cash in cashesRest:
            cashCode = str(cash.get('code'))
            
            converters = []
            saleExchangeTypes = cash.get('saleExchangeTypes')
            for saleExchangeType in saleExchangeTypes:
                saleExchangeTypeName = saleExchangeType.get('name')
                converter = None
                
                saleFile = macros.applyMacros(str(saleExchangeType.get('saleDir')), log, shopcode = shopCode, cashcode = cashCode)
                flagPath = saleExchangeType.get('flagFile')
                if flagPath:
                    flagPath = macros.applyMacros(str(flagPath), log, shopcode = shopCode, cashcode = cashCode)
                else:
                    flagPath = os.path.join(os.path.dirname(saleFile), 'pos%s.flr' % shopCode)
                backupDir = saleExchangeType.get('backupDir')
                if backupDir:
                    backupDir = macros.applyMacros(str(backupDir), log, shopcode = shopCode, cashcode = cashCode)
                else:
                    backupDir = '/opt/exchangers/backup/sales/%s/%s' % (shopCode, cashCode)

                if (saleExchangeTypeName == 'DEFAULT'):
                    continue
                elif (saleExchangeTypeName == 'ARTIXINTERCHANGE'):
                    dataType = saleExchangeType['dataType']
                    converter = AifConverter(saleFile, dataType=dataType)
                elif (saleExchangeTypeName == 'SHTRIHM'):
                    converter = ShtrihConverter(saleFile, discountsDetail=False, flag = flagPath)
                elif (saleExchangeTypeName == 'ATOL-ASTOR'):
                    converter = AtolAstorConverter(saleFile, discountsDetail=True)
                elif (saleExchangeTypeName == 'ATOL'):
                    converter = AtolConverter(saleFile, discountsDetail=True)
                elif (saleExchangeTypeName == 'SUPERMAG'):
                    converter = SmagConverter(saleFile, format='dat')
                elif (saleExchangeTypeName == 'CRYSTAL'):
                    converter = CrystalConverter(saleFile, flag = flagPath)
                elif (saleExchangeTypeName == 'ORACLE'):
                    converter = OracleConverter(saleFile, flag = flagPath)
                converters.append(converter)
            cashes.append(Cash(cashCode, converters, backup_dir=backupDir, maxCountFiles=maxCountFiles, actualCountDays=actualCountDays, cashId=cashCode))

        nodes.append(Node(id = shopCode,
                          cashes = cashes,
                          reader = DbReader(
                                            host = 'localhost',
                                            port = 3306,
                                            user = 'netroot',
                                            passwd = 'netroot',
                                            db = 'documents%s' % shopCode,
                                            ),
                          )
                     )
    return nodes
