import os, sys
import httplib2
import json
from unload.node.node import Node
from unload.node.nodeparams import DbReader, AifConverter, ShtrihConverter, SmagConverter, Cash, AtolAstorConverter, AtolConverter, OracleConverter, CrystalConverter, NS2000Converter, PilotConverter
from common import macros, logger, config
log = logger.Logger().getLoggerUnload("main")

jsonHeader = {'Content-Type':'application/json', "Accept": "application/json"}
http = httplib2.Http()

def str2bool(v):
    return str(v).lower() in ("yes", "true", "t", "1")

def getParamValue(exchange, key, defaultValue=None):
    l = filter(lambda param: param.get('key', None) == key, exchange.get('params', []))
    value = l[0].get('value', defaultValue) if len(l) > 0 else defaultValue
    return value

def getConverter(converterName, saleExchangeConfig = {}, **kwargs):
    converter = None
    
    salePath = str(getParamValue(saleExchangeConfig, 'salePath', '../sales/%(shopcode)s/%(cashcode)s'))
    saleFile = macros.applyMacrosWithoutTime(salePath, log, shopcode = kwargs.get('shopCode', None), cashcode = kwargs.get('cashCode', None), 
                                  shopid = kwargs.get('shopId', None), cashid = kwargs.get('cashId', None), clusterid = kwargs.get('clusterId', None))
    
    onlineSalePath = getParamValue(saleExchangeConfig, 'onlineSalePath')
    onlineSaleWithMacro = None
    if onlineSalePath:
        onlineSaleWithMacro = macros.applyMacrosWithoutTime(str(onlineSalePath), log, shopcode = kwargs.get('shopCode', None), cashcode = kwargs.get('cashCode', None), 
                                shopid = kwargs.get('shopId', None), cashid = kwargs.get('cashId', None), clusterid = kwargs.get('clusterId', None))

    rewriteDestFile = str2bool(getParamValue(saleExchangeConfig, 'rewriteDestFile', False))
    
    if converterName in ['aif', 'ARTIXINTERCHANGE']:
        dataType = getParamValue(saleExchangeConfig, 'dataType', 'sales')
        converter = AifConverter(saleFile, dataType=dataType, rewriteDestFile=rewriteDestFile,
                onlineSalePath = onlineSaleWithMacro)
    elif converterName in ['shtrih', 'SHTRIHM']:
        flagPath = str(getParamValue(saleExchangeConfig, 'saleFlag', os.path.join(os.path.dirname(saleFile), 'pos%(shopcode)s.flr')))
        flagFile = macros.applyMacrosWithoutTime(str(flagPath), log, shopcode = kwargs.get('shopCode', None), cashcode = kwargs.get('cashCode', None), 
                                shopid = kwargs.get('shopId', None), cashid = kwargs.get('cashId', None), clusterid = kwargs.get('clusterId', None))
        discountsDetail = str2bool(getParamValue(saleExchangeConfig, 'discountsDetail', False))
        useTransaction300ForDocuments =str2bool(getParamValue(saleExchangeConfig, 'useTransaction300ForDocuments', True))
        unloadShiftsFromKkm = str2bool(getParamValue(saleExchangeConfig, 'unloadShiftsFromKkm', True))
        unloadPaymentByCard = str2bool(getParamValue(saleExchangeConfig, 'unloadPaymentByCard', False))
        unloadCardType = str2bool(getParamValue(saleExchangeConfig, 'unloadCardType', True))
        defaultCardType = str(getParamValue(saleExchangeConfig, 'defaultCardType', ''))
        converter = ShtrihConverter(saleFile, discountsDetail=discountsDetail, flag = flagFile, rewriteDestFile=rewriteDestFile,
                                                        unloadPaymentByCard = unloadPaymentByCard, unloadCardType = unloadCardType,
                                                        defaultCardType = defaultCardType, useTransaction300ForDocuments = useTransaction300ForDocuments,
                                                        unloadShiftsFromKkm=unloadShiftsFromKkm)
    elif converterName in ['atol-astor', 'ATOL-ASTOR']:
        discountsDetail = str2bool(getParamValue(saleExchangeConfig, 'discountsDetail', False))
        converter = AtolAstorConverter(saleFile, discountsDetail=discountsDetail, rewriteDestFile=rewriteDestFile, 
                                onlineSalePath = onlineSaleWithMacro)
    elif converterName in ['atol', 'ATOL']:
        discountsDetail = str2bool(getParamValue(saleExchangeConfig, 'discountsDetail', False))
        converter = AtolConverter(saleFile, discountsDetail=discountsDetail, rewriteDestFile=rewriteDestFile, 
                                onlineSalePath = onlineSaleWithMacro)
    elif converterName in ['smag', 'SUPERMAG']:
        format = getParamValue(saleExchangeConfig, 'format', 'dat')
        converter = SmagConverter(saleFile, format=format, rewriteDestFile=rewriteDestFile)
    elif converterName in ['oracle', 'ORACLE']:
        converter = OracleConverter(saleFile, rewriteDestFile = rewriteDestFile)
    elif converterName in ['crystal', 'CRYSTAL']:
        formdiscounts = str2bool(getParamValue(saleExchangeConfig, 'formdiscounts', True))
        converter = CrystalConverter(saleFile, rewriteDestFile=rewriteDestFile, formdiscounts=formdiscounts)
    elif converterName in ['ns2000', 'NS2000']:
        converter = NS2000Converter(saleFile, rewriteDestFile = rewriteDestFile)
    elif converterName in ['pilot', 'PILOT']:
        converter = PilotConverter(saleFile, rewriteDestFile = rewriteDestFile)
    return converter


def getNodes():
    nodes = []
    response, content = http.request('http://localhost:8080/CSrest/rest/shops', 'GET', headers=jsonHeader)
    if response.status == 200:
        shops = json.loads(content)
        maxCountFiles=30
        actualCountDays=30
        if not config.read(sys.path[0] + "/../config/%s/exchangers.ini" % (logger.Logger().location)):
            log.warning("Configuration file 'exchangers.ini' not found. Configure backup properties by default...")
        else:
            maxCountFiles = config.getInt('UNLOAD', 'maxCountBackupFiles', maxCountFiles)
            actualCountDays = config.getInt('UNLOAD', 'actualCountDaysForBackupFiles', actualCountDays)
            
        for shop in shops:
            shopCode = str(shop.get('shopCode'))
            shopId = str(shop.get('id'))
            clusterId = str(shop.get('clusterId', ''))
            
            cashes = []
            for cashId in shop.get('cashes', []):
                response, content = http.request('http://localhost:8080/CSrest/rest/cashes/%s' % cashId, 'GET', headers=jsonHeader)
                if response.status == 200:
                    cash = json.loads(content)
                    converters = []
                    cashCode = str(cash.get('cashCode'))
                    saleExchangeConfigs = cash.get('exchange', {}).get('saleConfigs', {})
                    for saleExchangeConfig in saleExchangeConfigs:
                        saleExchangeTypeName = saleExchangeConfig.get('type', 'DEFAULT')
                        if (saleExchangeTypeName == 'DEFAULT'):
                            continue
                        converter = getConverter(saleExchangeTypeName, saleExchangeConfig, shopCode = shopCode, cashCode = cashCode, shopId = shopId, 
                                                      cashId = cashId, clusterId = clusterId)
                        backupPath = str(getParamValue(saleExchangeConfig, 'backupDir', '../backup/sales/%(shopcode)s/%(cashcode)s'))
                        backupDir = macros.applyMacrosWithoutTime(str(backupPath), log, shopcode = shopCode, cashcode = cashCode, shopid = shopId, 
                                                      cashid = cashId, clusterid = clusterId)
                        
                        converters.append(converter)
                    if len(converters) > 0:
                        cashes.append(Cash(cashCode, converters, backup_dir=backupDir, maxCountFiles=maxCountFiles, actualCountDays=actualCountDays, cashId=cashId))
            
            dbSuffix = shopId
            response, content = http.request('http://localhost:8080/CSrest/rest/system/mode', 'GET', headers=jsonHeader)
            if response.status == 200:
                serverMode = json.loads(content)
                if str(serverMode.get('hierarhyMode')) == 'CENTRAL' and str(shop.get('parentObject', {}).get('parentType')) == 'MIDDLE_SERVER':
                    dbSuffix = str(shop.get('parentObject', {}).get('parentCode'))
            nodes.append(Node(id = shopId,
                                  cashes = cashes,
                                  reader = DbReader(
                                                    host = 'localhost',
                                                    port = 3306,
                                                    user = 'netroot',
                                                    passwd = 'netroot',
                                                    db = '%sdocuments%s' % (clusterId, dbSuffix),
                                                    ),
                                  )
                             )
    return nodes
