# -*- coding: UTF-8 -*-
'''
Created on 03.05.2012

@author: kov
'''
import re
from common.parameters import DatabaseParameters

class DbReader(DatabaseParameters):
    def __init__(self, host, port, user, passwd, db):
        DatabaseParameters.__init__(self, host, port, user, passwd, db)

class SmagConverter(object):
    def __init__(self, path, format, flag = None, rewriteDestFile = False, currentSalesPath = None, charset='cp1251',
                 earnPositionCode = '999999', earnDeptCode = 0,
                 earnCardNumber = '99999999', spendPaymentCode = 10,
                 appendSaleFiles = False):
        self.format = format            # dat | dbf
        self.name = 'smag'              # name of converter
        self.path = path                # directory
        self.flag = flag                # file-flag for unload
        self.rewriteDestFile = rewriteDestFile
        self.currentSalesPath = currentSalesPath
        self.charset = charset
        self.earnPositionCode = earnPositionCode
        self.earnDeptCode = earnDeptCode
        self.earnCardNumber = earnCardNumber
        self.spendPaymentCode = spendPaymentCode
        self.appendSaleFiles = appendSaleFiles

class AifConverter(object):
    def __init__(self, path, flag = None, rewriteDestFile = False, dataType = None, withClosedShifts=True,
                 onlineSalePath = "/tmp/salesOnline/"):
        self.name = 'aif'               # name of converter
        self.path = path                # filename
        self.flag = flag                # file-flag for unload
        self.rewriteDestFile = rewriteDestFile
        self.dataType = 'sales'
        if dataType in ['sales','crights','all']:
            self.dataType = dataType        # unload data type: sales, crights, all
        self.withClosedShifts = withClosedShifts
        self.onlineSalePath = onlineSalePath

class ShtrihConverter(object):
    def __init__(self, path, discountsDetail = False, flag = None, rewriteDestFile = False, 
                 useActorCodeForMoneyOperations = False, unloadShortDateFormat = False, 
                 unloadCardType = True, defaultCardType = '', charset='cp1251', delimiter=';',
                 unloadPaymentByCard = False, unloadPaymentCommission = False,
                 useTransaction300ForDocuments = True, unloadShiftsFromKkm=False):
        self.name = 'shtrih'
        self.path = path                                    # filename
        self.discountsDetail = discountsDetail              # detail the discount on the check or not
        self.flag = flag                                    # file-flag for unload
        self.rewriteDestFile = rewriteDestFile
        self.useActorCodeForMoneyOperations = useActorCodeForMoneyOperations
        self.unloadShortDateFormat = unloadShortDateFormat
        self.unloadCardType = unloadCardType
        self.defaultCardType = defaultCardType if defaultCardType else ''
        self.charset = charset
        if delimiter:
            p = re.compile('(\[)([0-9]+)(\])')
            m = p.match(delimiter)
            if m:
                delimiter = chr (int (m.group (2)))
        else:
            delimiter = ';'
        self.delimiter = delimiter
        self.unloadPaymentByCard = unloadPaymentByCard
        self.unloadPaymentCommission = unloadPaymentCommission
        self.useTransaction300ForDocuments = useTransaction300ForDocuments
        self.unloadShiftsFromKkm = unloadShiftsFromKkm

class AtolAstorConverter(object):
    def __init__(self, path, discountsDetail = False, flag = None, rewriteDestFile = False, 
                 clearUnloadDirectory = False, charset='cp1251', onlineSalePath = "/tmp/salesOnline/"):
        self.name = 'atol-astor'
        self.path = path                                    # filename
        self.discountsDetail = discountsDetail              # detail the discount on the check or not
        self.flag = flag                                    # file-flag for unload
        self.rewriteDestFile = rewriteDestFile
        self.charset = charset
        self.onlineSalePath = onlineSalePath

class AtolConverter(object):
    def __init__(self, path, discountsDetail = False, flag = None, rewriteDestFile = False, 
                 clearUnloadDirectory = False, charset='cp1251', onlineSalePath = "/tmp/salesOnline/"):
        self.name = 'atol'
        self.path = path                                    # filename
        self.discountsDetail = discountsDetail              # detail the discount on the check or not
        self.flag = flag                                    # file-flag for unload
        self.rewriteDestFile = rewriteDestFile
        self.charset = charset
        self.onlineSalePath = onlineSalePath

class OracleConverter(object):
    def __init__(self, path, flag = None, rewriteDestFile = False):
        self.name = 'oracle'
        self.path = path                # filename
        self.flag = flag                # file-flag for unload
        self.rewriteDestFile = rewriteDestFile

class CrystalConverter(object):
    def __init__(self, path, flag = None, rewriteDestFile = False, formdiscounts = True):
        self.name = 'crystal'
        self.path = path
        self.flag = flag
        self.rewriteDestFile = rewriteDestFile
        self.formdiscounts = formdiscounts
        
class NS2000Converter(object):
    def __init__(self, path, flag = None, rewriteDestFile = False, charset='cp866'):
        self.name = 'ns2000'
        self.path = path                # filename
        self.flag = flag                # file-flag for unload
        self.rewriteDestFile = rewriteDestFile
        self.charset = charset

class PilotConverter(object):
    def __init__(self, path, flag = None, rewriteDestFile = False, charset='cp866'):
        self.name = 'pilot'
        self.path = path                # filename
        self.flag = flag                # file-flag for unload
        self.rewriteDestFile = rewriteDestFile
        self.charset = charset
        
class Cash(object):
    def __init__(self, code, converters = [], backup_dir = None, maxCountFiles=30, actualCountDays=30, cashId=None):        
        self.code = code            # code of cash 
        self.converters = converters
        self.backup_dir = backup_dir
        self.maxCountFiles = maxCountFiles
        self.actualCountDays = actualCountDays
        self.cashId = cashId if cashId else code
