# -*- coding: UTF-8 -*-
'''
Created on 05.05.2012

@author: kov
'''

import sys, os
sys.path[0] = sys.path[0] + '/../../..'
from string import Template
from datetime import datetime, date
import time

from common import logger, config, serviceLock, macros
logger.Logger('cs')
from unload.service.reader import Reader
import optparse
import atexit
import tempfile
import signal
import traceback

log = logger.Logger().getLoggerUnload("main")

lockFile = os.path.normpath('%s/locks/lockArtixStartReaderCS' % tempfile.gettempdir())
srvLock = serviceLock.ServiceLock(lockFile, log)
atexit.register(srvLock.unlock)
signal.signal(signal.SIGTERM, srvLock.unlock)


r = Reader()

def unload(node, workDirectory, request = None, converterName = None, shifts = None, dates = None, granularity = 'document', closedShifts=0, 
           changedShifts=False, cashCode=None, cashUnloadStateId=None, statusHost=None):
    log = logger.Logger().getLoggerUnload(node.id)
    unloadNowFile = workDirectory+'/%s/unload/unloadNow' % node.id

    srvLockUnload = serviceLock.ServiceLock(unloadNowFile, log)
    atexit.register(srvLockUnload.unlock)
    try:
        signal.signal(signal.SIGTERM, srvLockUnload.unlock)
    except:
        pass

    if srvLockUnload.lock(False):
        try:
            if request and converterName:
                log.info('unload by request %s' % request)
                shifts = None
                dates = None
                try:
                    plugin = __import__('unload.converter.%s.%s' % (converterName, converterName), fromlist='q')
                    shifts, dates = plugin.parseFlag(request)
                except Exception, e:
                    log.error(e)
                    log.error(traceback.format_exc())
                log.info('remove file-flag %s' % request)
                os.remove(request)
                if not shifts and not dates:
                    r.read(node, 2, shifts, dates, "document", False, workDirectory, converterName = converterName, cashCode=cashCode, cashUnloadStateId=cashUnloadStateId, statusHost=statusHost)
                else:
                    r.read(node, 2, shifts, dates, "shift", False, workDirectory, converterName = converterName, cashCode=cashCode, cashUnloadStateId=cashUnloadStateId, statusHost=statusHost)
            else:
                r.read(node, closedShifts, shifts, dates, granularity, changedShifts, workDirectory, converterName = converterName, cashCode=cashCode, cashUnloadStateId=cashUnloadStateId, statusHost=statusHost)
            if os.path.exists(workDirectory+'/%s/unload/ready' % node.id):
                r.send(node, workDirectory)
        except Exception, e:
            log.error(str(e))
        srvLockUnload.unlock()
        return True
    else:
        log.info('Can not start unload. Unloading is already running...')
        return False

def applyConverterAndPathOptions(node, cash, converterName, dataPath, nodeGetter):
    if converterName:
        if not any(x.name == converterName for x in cash.converters):
            # Если конвертора из параметра нет в ноде, добавим его сами
            converter = nodeGetter.getConverter(converterName)
            if converter:
                cash.converters = [converter]
            else:
                log.error(u"Не поддерживаемый тип конвертера %s" % converterName)
                return False

        for converter in cash.converters:
            if converter.name == converterName and dataPath:
                converter.path = macros.applyMacros(dataPath, log, shopid = node.id, cashcode = cash.code, cashid = cash.cashId)
    return True

def mainRun(cashCode, shopCode, shifts, dates, converterName=None, dataPath=None, cashUnloadStateId=None, statusHost=None, granularity="document", closedShifts=0, changedShifts=False):
    result = False
    configFilePath = sys.path[0] + "/../config/cs/exchangers.ini"
    if config.read(configFilePath):

        nodes = []
        try:
            nodeGetter = __import__('unload.node.%s.nodeGetter' % config.getString('UNLOAD', 'nodeGetter', 'cs30'), fromlist='q')
            nodes = nodeGetter.getNodes()
        except Exception, e:
            log.error(u'Ошибка получения нод. %s' % e.message)
        
        replace = {}
        replace['date'] = date.today()
        replace['time'] = datetime.now().strftime('%H-%M-%S')
        replace['timestamp'] = int(time.time())
        
        workDirectory = config.getString('main', 'workDirectory', '/opt/exchangers/work')
        # generate sales
        for node in nodes:
            if not shopCode or shopCode == node.id:
                for i in range(len(node.cashes)):
                    
                    if not applyConverterAndPathOptions(node, node.cashes[i], converterName, dataPath, nodeGetter):
                        return
                    
                    for j in range(len(node.cashes[i].converters)):
                        node.cashes[i].converters[j].path = Template(node.cashes[i].converters[j].path).safe_substitute(replace)
                    if cashCode and cashCode == node.cashes[i].code:
                        result = unload(node, workDirectory, shifts=shifts, dates=dates, granularity=granularity, closedShifts=closedShifts, 
                               changedShifts=changedShifts, cashCode=cashCode, cashUnloadStateId=cashUnloadStateId, statusHost=statusHost)
                        if not result:
                            return result
                        break
                if not cashCode:
                    result = unload(node, workDirectory, shifts=shifts, dates=dates, granularity=granularity, closedShifts=closedShifts, 
                           changedShifts=changedShifts, cashUnloadStateId=cashUnloadStateId, statusHost=statusHost)
                    if not result:
                        return result
    else:
        log.error("Configuration file '%s' not found. The system 'exchangers' is not running." % configFilePath)
        result = False
    return result

def main(argv):
    if not srvLock.lock(False):
        log.error("Can not run service. This service is runing.")
        os._exit(1) # выход без выполнения atexit

    parser = optparse.OptionParser()
    parser.add_option("-K", "--cashCode", dest="cashCode", help="Unload for this cash")
    parser.add_option("-S", "--shopCode", dest="shopCode", help="Unload for this shop")
    parser.add_option("-s", "--shifts", dest="shifts", help="@-separated shift range")
    parser.add_option("-d", "--dates", dest="dates", help="@-separated dates range")
    parser.add_option("-g", "--granularity", dest="granularity", default="document", help="granularity of data: shift or document [default: %default]")
    parser.add_option("-c", "--closedShifts", dest="closedShifts", default=0, type='int', 
                      help="Unload only closed, opened or closed and opened shifts. Use: 0-closed, 1-opened, 2-closedAndOpened  [default: %default]")
    parser.add_option("-C", "--changedShifts", action="store_true", dest="changedShifts", default=False, help="Unload all documents for changed shifts")
    parser.add_option("-v", "--converter", dest="converterName", help=u"Тип конвертора: aif, atol-astor, crystal, ns2000, oracle, shtrih, smag")
    parser.add_option("-p", "--path", dest="dataPath", help=u"Путь куда выгружаются файлы продаж")
    parser.add_option("-u", "--cashUnloadStateId", dest="cashUnloadStateId", help=u"Идентификатор статуса выгрузки продаж")
    parser.add_option("-r", "--statusHost", dest="statusHost", help=u"Хост и порт REST сервера разделенный :")
    (options, args) = parser.parse_args(argv)

    mainRun(options.cashCode, options.shopCode, options.shifts, options.dates, options.converterName, options.dataPath, options.cashUnloadStateId, 
         options.statusHost, options.granularity, options.closedShifts, options.changedShifts)
    srvLock.unlock()

if __name__ == '__main__':
    main(sys.argv[1:])
