# -*- coding: UTF-8 -*-

import sys, os
import re

from common import logger
from upload.converter.shtrihPlugins.sthrihformat import ShtrihFormatter, getClearCommands
#import binascii
import xml.etree.ElementTree as ET

delimiter = None
cashierName = None
errorObjectsCount = 0
hotKeyByBarcode = False
isMultiLineCommand = False  # Разбита ли команда на несколько строк
lineCommands = ''  # строки одной команды
inventCodeInCommands = ''  # код товара для текущей xml команды
fewXmlCommands = ''  # несколько xml команд для одного товара 
tagInCommands = ''

def addInventSprMode(line, mode):
    return '0' + line + delimiter + mode


def getCommand(line):
    '''
    Если одна команда разбита на несколько строк, то эти строки собираются в один блок
    '''
    global lineCommands
    global isMultiLineCommand
    command = line.strip()
    if line.startswith('<') and '</' not in line:
        isMultiLineCommand = True
    if isMultiLineCommand:
        if '</' in command:
            command = lineCommands + command
            isMultiLineCommand = False
            lineCommands = ''
            return command
        else:
            lineCommands += command
            return ''
    else:
        return command


def getFuncKeyAndCommandBody(command):
    if command[0] == '<':
        return command.split()[0][6:], command
    else:
        return command[0], command[1:]


def getCommandList(command):
    global fewXmlCommands
    global inventCodeInCommands
    global tagInCommands
    result = []
    currentCode = ''
    currentTag = ''

    if command[0] != '<':
        result.append(command)
        if fewXmlCommands:
            result.append(fewXmlCommands)
            fewXmlCommands = ''
            inventCodeInCommands = ''
            tagInCommands = ''
        return result

    try:
        xmlItem = ET.fromstring(command)
        if 'id' in xmlItem.attrib:
            currentCode = xmlItem.attrib['id']
        currentTag = xmlItem.tag
    except:
        result.append(fewXmlCommands)
        result.append(command)
        return result

    if not fewXmlCommands:
        inventCodeInCommands = currentCode
        fewXmlCommands += command
        tagInCommands = currentTag
    else:
        if inventCodeInCommands == currentCode and tagInCommands == currentTag:
            fewXmlCommands = fewXmlCommands + command
        else:
            result.append(fewXmlCommands)
            fewXmlCommands = command
            inventCodeInCommands = currentCode
            tagInCommands = currentTag
    return result


def getAifObject(line):
    command = line

    if command[0].isdigit():
        command = addInventSprMode(command, sprMode)
    elif command[0].isalpha():
        if len(command.split(delimiter)) == 17:
            command = addInventSprMode(command, sprMode)
    elif command[0] == '<':
        command = '<rows>' + command + '</rows>'

    func_key, func_value = getFuncKeyAndCommandBody(command)
    formatter = ShtrihFormatter(delimiter, cashierName, hotKeyByBarcode)
    if func_key in formatter.formatDescription:
        if func_key == '#' and not command.split(delimiter)[1]:
            log.warn(u'Некорректные данные в строке %s ' % command)
            return None
        func = formatter.formatDescription[func_key]
        try:
            return func(func_value)
        except Exception, e:
            log.error(u'Ошибка в строке %s. %s' % (command, str(e)))
            errorList.append(u'Ошибка в строке %s. %s' % (command, str(e)))
    elif not command.startswith('$$$'):
        log.warn(u'Строка %s не обработана' % command)
    return None


def parseLine(line):
    command = getCommand(line)
    if not command:
        return
    commands = getCommandList(command)
    if not commands:
        return
    aifList = []
    for item in commands:
        if not item:
            continue
        aif = getAifObject(item)
        if aif:
            aifList.extend(aif) if isinstance(aif, list) else aifList.append(aif)
    return aifList


def parseKeysFromSection(openFile):
    result = []
    line = openFile.readline()
    while not line.startswith('##@@&&') and line:
        if line.strip().startswith('$$$'):
            result += getKeyFromLine(line)
        line = openFile.readline()
    return result

def getKeyFromLine(line):
    line = line.strip()
    global sprMode
    result = []
    flagsRegularExpression = re.compile("(\{[^\}]*\})")
    if line.startswith('$$$'):
        if line.startswith('$$$CLR'):
            sprFlags = flagsRegularExpression.findall(line)
            for flag in sprFlags:
                if flag in getClearCommands():
                    result.append(getClearCommands()[flag])
            if not getClearCommands()['{NO_TOV}'] in result:
                result.append(getClearCommands()['{NO_TOV}'])
            else:
                result.remove(getClearCommands()['{NO_TOV}'])
        elif line.startswith('$$$ADD'):
            sprMode = "ADD"

    return result    

# filename - имя открытого файла, каретка которого установлена в начало секции, которую надо пропустить.
# Возвращается ссылка на тот же файл, но каретка находится на EOF либо на начале следующей секции
# (на ВТОРОЙ строке заголовка). В последнем случае возвращается еще и первая строчка заголовка.   
def skipSection(filename):
    line = filename.readline()
    while not line.startswith('##@@&&') and line :
        line = filename.readline()
    if not line:
        return filename, None
    else:
        return filename, line

# plugin - плагин, который выполняет загрузку, listObjs - списко объектов, которые нужно загрузить,
# size - порог срабатывания загрузки
def loadObjs(inserter, listObjs, size):
    if len(listObjs) >= size:
        inserter.add_package(listObjs)
        listObjs[0:] = [] # объекты загружены - очищаем список

# workDirectory - рабочая директория, куда копируются файлы со справочниками
# data - массив путей до файлов со справочниками
# node - информация об узле для загрузки справочников 
def general_method(workDirectory, data, node):
    global log
    global errorObjectsCount
    log = logger.Logger().getLoggerUpload(node.id)
    global delimiter
    delimiter = node.currentConverter.delimiter
    global cashierName
    cashierName = node.currentConverter.cashierName
    global hotKeyByBarcode
    hotKeyByBarcode = node.currentConverter.hotKeyByBarcode
    listOfKeys = []
    listOfHeadKeys = []
    global sprMode
    global errorList
    errorList = []
    plugin = __import__('upload.inserter.%s' % (node.inserter.name), fromlist='Inserter')
    cl = getattr(plugin, 'Inserter')
    inserter = cl(node)
    result = inserter.start()
    if result:
        for currentFile in data:
            openFile = open(currentFile, 'rb+') # 'rb' для корректной работы функции file.tell() в Windows
            line = openFile.readline()
            if line.strip().startswith('##@@&&'):
                listOfHeadKeys = getKeyFromLine(line[6:])
                loadObjs(inserter, listOfKeys, node.pack_size)
            else:
                log.error(u'Неверное начало файла (%s): %s. Ожидалось: ##@@&&' %(currentFile, line.strip()))
                return False, errorList
            flag = True
            while flag: # Цикл по секциям. На старте цикла строка ##@@&& уже прочитана.
                line = openFile.readline()
                sprMode = "RPL" # По-умолчанию включен режим "Замены"
                if line.strip() == '#':
                    listOfKeys += listOfHeadKeys
                    listOfHeadKeys = []
                    position = openFile.tell()
                    res = parseKeysFromSection(openFile) # Находим ключи в секции
                    listOfKeys += res
                    loadObjs(inserter, listOfKeys, node.pack_size)
                    openFile.seek(position)
                    line = openFile.readline()
                    if line.strip() == '@@@ENABLECHECK':
                        line = openFile.readline()                    
                    while not line.startswith('##@@&&') and line:
                        aif_obj = parseLine(line.decode('cp1251'))
                        if aif_obj:
                            listOfKeys.extend(aif_obj if isinstance(aif_obj, list) else [aif_obj])
                        loadObjs(inserter, listOfKeys, node.pack_size)
                        line = openFile.readline()
                        if line.strip() == '@@@EOF':
                            line = openFile.readline()
                    if fewXmlCommands:
                        aif_object = getAifObject(fewXmlCommands)
                        if aif_object:
                            listOfKeys.extend([aif_object])
                            loadObjs(inserter, listOfKeys, node.pack_size)
                    if not line:        # Файл весь прочитан?
                        flag = False    # тогда "Выход"
                    else:
                        listOfHeadKeys = getKeyFromLine(line[6:])
                    tmpPos = openFile.tell()
                    openFile.seek(position - 3)
                    openFile.write('@')
                    openFile.seek(tmpPos)
                elif line.strip() == '@':   # Секция уже загружена?
                    openFile, line = skipSection(openFile)
                    if not line:            # Секция была последней в файле?
                        flag = False        # тогда "Выход"
                    else:
                        listOfHeadKeys = getKeyFromLine(line[6:])
                else:
                    log.error(u'Неверная вторая строка: %s. Ожидалось \'#\' или \'@\'' % (line))
                    return False, errorList
            loadObjs(inserter, listOfKeys, 1)
            openFile.close()
        result = inserter.stop()
        errorObjectsCount = inserter.invalidObjectsCount()
    return result, errorList

def invalidObjectsCount():
    return errorObjectsCount
