# -*- coding: UTF-8 -*-

from upload.inserter.queue import Inserter
from common import aifSoapClasses as soapCl
from common import logger, rest_state
from collections import Iterable
import datetime
import traceback

objectsCount = 0
inserter = None
clearCmd = False

def convertToMarshalableType(value):
    if isinstance(value, datetime.datetime) or isinstance(value, datetime.date) or isinstance(value, datetime.time):
        value = str(value)
    return value

def getCommand(clazz, obj, method, returnClass = False):
    dictParams = {}
    if callable(clazz):
        clazz = clazz()
    func = getattr(clazz, method)
    if callable(func):
        for paramName in func.func_code.co_varnames:
            if paramName != 'self' and paramName in dir(obj):
                paramValue = getattr(obj, paramName)
                if isinstance(paramValue, soapCl.BaseAif):
                    paramValue = converBaseAifToDict(paramValue)
                elif isinstance(paramValue, Iterable) and len(paramValue) > 0 and isinstance(paramValue[0], soapCl.BaseAif):
                    paramValue = [converBaseAifToDict(x) for x in paramValue]
                dictParams[paramName] = convertToMarshalableType(paramValue)
        if returnClass:
            func(**dictParams)
            return clazz
        else:
            return func(**dictParams)

def converBaseAifToDict(baseAifObject):
    dictObj = {}
    if isinstance(baseAifObject, soapCl.BaseAif):
        params = [x for x in dir(baseAifObject) if x not in dir(soapCl.BaseAif)]
        for param in params:
            value = getattr(baseAifObject, param)
            if value != None:
                dictObj[param] = converBaseAifToDict(value) if isinstance(value, soapCl.BaseAif) else convertToMarshalableType(value)
    return dictObj

def start(node):
    global objectsCount
    global inserter
    global clearCmd
    objectsCount = 0
    clearCmd = False
    result = soapCl.DictConvertResult()
    result.result = False
#    result.info = 'Ошибка на стадии блокировки БД'
    result.info = 'Error at the stage starting'
    inserter = Inserter(node)
    result.result = inserter.start()
    if not result.result:
        inserter.stop()
#            result.info = 'Ошибка открытия транзакции для записи объектов справочника в БД'
            #result.info = 'Error start for writing dictionary objects to the DB'
    else:
        result.info = 'OK'
    return result

def convert(objects, shopCode, aifCommandClass, node, curMethod, clear = False):
    global objectsCount
    global clearCmd
    log = logger.Logger().getLoggerUpload(node.id)
    hotKeyByBarcode = node.currentConverter.hotKeyByBarcode
    try:
        result = soapCl.DictConvertResult()
        result.result = False
    #    result.info = 'Ошибка на стадии начала конвертирования'
        result.info = 'Error at the stage start converting'
        listObjects = []
        clearCmd = clear
        if clear:
            listObjects.append(getCommand(aifCommandClass, None, 'clearCmd'))
            objectsCount -= 1
        for obj in objects:
            cmd = getCommand(aifCommandClass, obj, curMethod)
            if hotKeyByBarcode:
                if 'addHotKey' == cmd['command'] and not 'bybarcode' in cmd['hotkey']:
                    cmd['hotkey']['bybarcode'] = hotKeyByBarcode
            listObjects.append(cmd)
            log.debug('Add command: %s' % cmd)
            listObjectsSize = len(listObjects)
            if listObjectsSize >= node.pack_size:
                objectsCount += listObjectsSize
                inserter.add_package(listObjects)
                del(listObjects[:])
        listObjectsSize = len(listObjects)
        if listObjectsSize > 0:
            objectsCount += listObjectsSize
            inserter.add_package(listObjects)
            del(listObjects[:])
        result.result = True
    except Exception, e:
#         db.stop(node)
        messEn = 'Can not convert SOAP data to AIF or send dictionary to queue. Error: %s' % e.message
        messRu = u'Ошибка конвертирования или отправки справочника в очередь. Error: %s' % e.message
        log.error('%s %s' % (messEn, traceback.format_exc()))
        node.currentConverter.dict_state_id = rest_state.sendDictState(node, messEn, messRu, rest_state.STATUS_ERROR, node.currentConverter.dict_state_id)
        result.result = False
        result.info = messEn
        return result
    if result.result:
        result.info = u'OK'
    else:
        node.currentConverter.dict_state_id = rest_state.sendDictState(node, u'Can not send dictionary to queue', u'Ошибка отправки справочника в очередь', rest_state.STATUS_ERROR, node.currentConverter.dict_state_id)
        if not result.info:
            result.info = 'Can not send dictionary to queue'
    return result

def stop(node):
    result = soapCl.DictConvertResult()
    result.result = inserter.stop()
    if objectsCount == 0:
        if clearCmd:
            result.info = 'Sent only dictionary clear command.'
            node.currentConverter.dict_state_id = rest_state.sendDictState(node, result.info, u'Отправлена только команда очистки справочника.', rest_state.STATUS_READY_WITH_ERROR, node.currentConverter.dict_state_id)
        else:
            result.info = 'No entries have been added to queue.'
            node.currentConverter.dict_state_id = rest_state.sendDictState(node, result.info, u'Ни одной записи не добавлено в очередь.', rest_state.STATUS_ERROR, node.currentConverter.dict_state_id)
        result.result = True
        return result
    result.info = u'OK. Objects count = %s' % objectsCount
    if result.result:
        node.currentConverter.dict_state_id = rest_state.sendDictState(node, u'OK. Objects count = %s' % objectsCount, u'ОК. Количество объектов = %s' % objectsCount, rest_state.STATUS_READY, node.currentConverter.dict_state_id)
#       result.info = 'Ошибка закрытия транзакции записи объектов справочника в БД.'
    else:
        result.info = 'Error closing the transaction writting dictionary objects.'
        node.currentConverter.dict_state_id = rest_state.sendDictState(node, result.info, u'Ошибка записи справочника в очередь.', rest_state.STATUS_ERROR, node.currentConverter.dict_state_id)
    return result

def rollback(node, sessionIdle=False):
    result = soapCl.DictConvertResult()
    result.result = inserter.rollback()
    result.info = u'OK'
    enPrefix = u''
    ruPrefix = u''
    if sessionIdle:
        enPrefix = u'Exceeded session idle timeout. '
        ruPrefix = u'Превышено время ожидания сессии. '
    if result.result:
        node.currentConverter.dict_state_id = rest_state.sendDictState(node, u'%sCancel dictionary loading' % enPrefix, u'%sЗагрузка справочника отменена' % ruPrefix, rest_state.STATUS_ERROR, node.currentConverter.dict_state_id)
    else:
        node.currentConverter.dict_state_id = rest_state.sendDictState(node, u'%sError canceling dictionary loading' % enPrefix, u'%sОшибка отмены загрузки справочника' % ruPrefix, rest_state.STATUS_ERROR, node.currentConverter.dict_state_id)
        result.info = 'Error rollback the transaction writting dictionary objects.'
    return result
