# -*- coding: utf-8 -*-

from bottle import route, run, response, request, HTTPResponse, install
from httplib import INTERNAL_SERVER_ERROR, UNAUTHORIZED
from functools import wraps
import traceback
import logging
import json
import time
import os
from common import logger, parameters, rest_state, filesmodify
from upload.node.node import QueueInserterConfig

restLogName = 'dict_REST_debug'
backuplog = logging.getLogger('dict.rest.backup')
backuplogHandler = logging.handlers.WatchedFileHandler('%s/%s.log' % (logger.Logger().getLogDirectory(), restLogName))
backuplogHandler.setFormatter(logging.Formatter('%(asctime)s %(levelname)-8s %(message)s %(module)5s(%(lineno)s)'))
backuplog.addHandler(backuplogHandler)
backuplog.setLevel(logging.DEBUG)

log = logger.Logger().getLoggerUpload("rest")

restServiceHost = '0.0.0.0'
restServicePort = 28080

timeoutNodeReread = 300
lastTimeReadNodes = time.time() - timeoutNodeReread
curNodes = []
primitiveAuth = True
restLogin = 'login'
restPasswd = 'login'
curConfig = None
isCashRun = False  # Рест сервис запущен на кассе
shopCodeIsShopId = True # Является ли код магазина, который передают в запросе идентификатором магазина.
maxCountBackupRequests = 0
actualCountDaysForBackupRequests = 3

def filterKeys(d):
    lowerD = {}
    for k in d.keys():
        if d[k] != None and d[k] != []:
            lowerD[k.lower()] = d[k]
    return lowerD

class RestConverter(object):
    name = 'rest'
    dict_state_id = None
    def __init__(self, hotKeyByBarcode = False):
        self.hotKeyByBarcode = hotKeyByBarcode

class EmptyNode(object):
    def __init__(self, shopCode):
        self.id = shopCode
        self.pack_size = curConfig.getInt('QUEUE', 'aifObjectsCountInMessage', 200)
        self.inserter = QueueInserterConfig(queue = parameters.QueueParameters(host = curConfig.getString('QUEUE', 'host', 'localhost'), 
                                                                               port = curConfig.getInt('QUEUE', 'port', 5672),
                                                                               username=curConfig.getString('QUEUE', 'user', 'guest'), 
                                                                               password=curConfig.getString('QUEUE', 'password', 'guest'), 
                                                                               queue='artix.loaddict.cash' if isCashRun else None))
        self.currentConverter = RestConverter()
        self.version = None

def readNodes():
    global curNodes
    global lastTimeReadNodes
    nodes = None
    if time.time() - lastTimeReadNodes > timeoutNodeReread:
        lastTimeReadNodes = time.time()
        try:
            log.info(u'Чтение нод')
            nodeGetter = __import__('upload.node.%s.nodeGetter' % (curConfig.getString('UPLOAD', 'nodeGetter', 'cs20')), fromlist='q')
            nodes = nodeGetter.getNodes()
            log.info(u'ноды = %s' % nodes)
        except Exception, e:
            log.warning(u'Ошибка считывания нод. %s' % str(e))
    if nodes:
        curNodes = nodes
    return curNodes

def checkShopCode(shopCode):
    if not shopCode or shopCode.strip() == '':
        return False, shopCode
    nodes = readNodes()
    if nodes:
        if not shopCodeIsShopId:
            ids = [n.id for n in nodes if str(n.shopCode) == shopCode]
            if len(ids) == 1:
                shopCode = ids[0]
            else:
                log.warning(u'По коду магазина %s найдено несколько идентификаторов либо ни одного. Невозможно однозначно определить целевой магазин.' % shopCode)
                return False, shopCode
        elif not str(shopCode) in [str(node.id) for node in nodes]:
            return False, shopCode
    elif not shopCodeIsShopId:
        log.warning(u'Невозможно найти по коду магазина его идентификатор, т.к. ноды не считаны.')
        return False, shopCode
    return True, shopCode

def getNode(shopCode, dictionaryId = None, statusServer = 'localhost:8080'):
    node = EmptyNode(shopCode)
    node.statusServer = statusServer
    if readNodes():
        for n in readNodes():
            if str(n.id) == str(node.id):
                node.inserter = n.inserter
                for converter in n.converters:
                    if converter.name == 'aif':
                        node.currentConverter.hotKeyByBarcode = converter.hotKeyByBarcode
                break
    if not dictionaryId:
        dictionaryId = 'rest' + str(time.time())
    try:
        node.currentConverter.dict_state_id = rest_state.sendDictState(node, 'Start convert dict', u'Начало конвертирования справочника.', rest_state.STATUS_DURING, dictionaryId)
        if not node.currentConverter.dict_state_id:
            node.currentConverter.dict_state_id = dictionaryId
    except Exception,e:
        log.warning(u'Ошибка отправки статуса на КС через REST. %s' % e)
        log.warning(traceback.format_exc())
    return node

def load(dictObjects, shopCode = 'All', dictionaryId = None, statusServer = 'localhost:8080'):
    res, shopCode = checkShopCode(shopCode)
    if not res:
        return None, {u'success': False, 'info': 'Shop with code = %s does not found.' % shopCode, 'info_ru': u'Магазин с кодом = %s не найден.' % shopCode}
    node = getNode(shopCode, dictionaryId, statusServer)
    plugin = __import__('upload.inserter.%s' % (node.inserter.name), fromlist='Inserter')
    cl = getattr(plugin, 'Inserter')
    inserter = cl(node)
    result = False
    try:
        if node.currentConverter.hotKeyByBarcode:
            for item in dictObjects:
                if 'addHotKey' == item['command'] and not 'bybarcode' in item['hotkey']:
                    item['hotkey']['bybarcode'] = node.currentConverter.hotKeyByBarcode
        result = inserter.start()
        if result:
            n = len(dictObjects)
            k = n / node.pack_size
            if n % node.pack_size > 0:
                k += 1
            for i in range(k):
                inserter.add_package(dictObjects[i*node.pack_size:(i*node.pack_size)+node.pack_size])
            result = inserter.stop()
            if result:
                return node, {u'success': result, 'info': u'OK', u'info_ru': 'OK'}
            else:
                return node, {u'success': result, 'info': u'Can not commit to queue.', 'info_ru': u'Не возможно положить в очередь объекты'}
        else:
            return node, {u'success': result, 'info': u'Can not start for load to queue.', 'info_ru': u'Не возможно начать загрузку в очередь.'}
    except Exception, e:
        log.error(u'Ошибка: ' + unicode(e))
        log.error(traceback.format_exc())
        return node, {u'success': result, 'info': u'Error: ' + unicode(e), 'info_ru': u'Ошибка: ' + unicode(e)}
    return node, {u'success': result, 'info': u'Unknown error.', 'info_ru': u'Неизвестная ошибка.'}

def require_uid(fn):
    def check_uid(**kwargs):
        if primitiveAuth:
            if request.auth and request.auth == (restLogin, restPasswd):
                return fn(**kwargs)
            else:
                raise HTTPResponse("Unauthorized", status=UNAUTHORIZED)
        return fn(**kwargs)
    return check_uid

@route('/aifexchange/loaddicts', method='POST')
@require_uid
def restStatus():
    response.content_type = 'application/json'
    body = None
    result = {u'success': False, 'info': u'unknown status', 'info_ru': u'неизвестный статус'}
    dictionaryId = request.query.get('dictionaryid', None)
    node = None
    try:
        if (request.content_length > 0 or request.chunked):
            body = json.loads(request.body.read(), encoding='utf8', object_hook = filterKeys)
        if not isinstance(body, list):
            raise Exception('Body is not array.')
        if len(body) == 0:
            body = None
            raise Exception('Array is empty.')
        node, result = load(body, shopCode = request.query.get('shopcode', 'All'), dictionaryId = dictionaryId, statusServer = request.query.get('addressstatusserver', 'localhost:8080'))

    except Exception, e:
        errorStr = u''
        errorStr_ru = u''
        if isinstance(body, list):
            errorStr = u'Error loading dicts. %s' % str(e)
            errorStr_ru = u'Ошибка загрузки справочника. %s' % str(e)
        else:
            errorStr = u'Incorrect request. %s' % str(e)
            errorStr_ru = u'Неправильный запрос. %s' % str(e)
        log.error(errorStr)
        log.error(traceback.format_exc())
        response.status = INTERNAL_SERVER_ERROR
        result = {u'success': False, u'info': errorStr, u'info_ru': errorStr_ru}
    if node:
        rest_state.sendDictState(node, result['info'], result['info_ru'], rest_state.STATUS_READY if result['success'] else rest_state.STATUS_ERROR, dictionaryId)
    
    return json.dumps(result, ensure_ascii=False)

def loggingRequest(fn):
    @wraps(fn)
    def wrapper(*args, **kwargs):
        backuplog.info('remote_addr = %s ' % request.remote_addr)
        backuplog.info('url = %s ' % request.url)
        backuplog.info('method = %s ' % (request.method))
        backuplog.info('headers = %s ' % (request.headers.items()))
        backuplog.info('content_length = %s ' % request.content_length)
        backuplog.info('chunked = %s ' % request.chunked)
        if (request.content_length > 0 or request.chunked):
            backuplog.info('body = %s ' % request.body.read())
        backupRequest(request.query.get('shopcode', 'All'))
        log.info('begin of request handling...')
        try:
            actual_response = fn(*args, **kwargs)
        except Exception, e:
            log.error(traceback.format_exc())
            raise(e)
        log.info('end of request handling')
        log.info(actual_response)
        return actual_response
    return wrapper

def backupRequest(shopCode):
    logDir = logger.Logger().getLogDirectory()
    logFiles = filesmodify.getFilesByMask(logDir, ['%s.*' % restLogName])
    for f in logFiles:
        filesmodify.archive(log, os.path.dirname(f), os.path.join(logDir, 'REST_BACKUP', shopCode), maxCountFiles=maxCountBackupRequests, actualCountDays=actualCountDaysForBackupRequests, removeSRC = True, fileName = os.path.basename(f), loginfo=False)

def start(config, isCash = False):
    global timeoutNodeReread
    global primitiveAuth
    global restLogin
    global restPasswd
    global restServicePort
    global lastTimeReadNodes
    global curConfig
    global isCashRun
    global shopCodeIsShopId
    global maxCountBackupRequests
    global actualCountDaysForBackupRequests
    isCashRun = isCash
    curConfig = config
    timeoutNodeReread = config.getInt('UPLOAD', 'timeoutNodeReread', 300)
    lastTimeReadNodes = time.time() - timeoutNodeReread
    primitiveAuth = config.getBool('REST_UPLOAD', 'authorize', True)
    restLogin = config.getString('REST_UPLOAD', 'login', 'admin')
    restPasswd = config.getString('REST_UPLOAD', 'passwd', 'admin')
    restServicePort = config.getInt('REST_UPLOAD', 'port', 28080)
    shopCodeIsShopId = config.getBool('REST_UPLOAD', 'shopCodeIsShopId', True)
    maxCountBackupRequests = config.getInt('REST_UPLOAD', 'maxCountBackupRequests', 0)
    actualCountDaysForBackupRequests = config.getInt('REST_UPLOAD', 'actualCountDaysForBackupRequests', 3)

    try:
        install(loggingRequest)
        run(host=restServiceHost, port=restServicePort, quiet=True)
    except Exception, e:
        log.error(u'Ошибка: %s' % e.message)
        log.error(traceback.format_exc())

#if __name__ == '__main__':
#    start()