# -*- coding: utf-8 -*-
from common import logger, filesmodify
log = logger.Logger().getLoggerUpload("soap")
from common import aifSoapClasses as soapCl
from upload.converter import soap
from common import aif_objects as aif
from common import rest_state, parameters
from upload.node.node import QueueInserterConfig, DBInserterConfig

import os
import time
from datetime import datetime
import inspect
import threading

from spyne.decorator import srpc
from spyne.model.complex import Array
from spyne.model.primitive import String, Boolean
from spyne.service import ServiceBase
from spyne.const.xml_ns import DEFAULT_NS

from spyne.application import Application
from spyne.protocol.soap import Soap11
from spyne.server.wsgi import WsgiApplication
#from spyne.util.wsgi_wrapper import run_twisted
from wsgiref.simple_server import WSGIRequestHandler, ServerHandler
import traceback, sys
from base64 import b64encode

import logging
fileLogName = 'dict_SOAP_debug'
spynelog = logging.getLogger('spyne.protocol.xml')
spynelogHandler = logging.handlers.WatchedFileHandler('%s/%s.log' % (logger.Logger().getLogDirectory(), fileLogName))
spynelogHandler.setFormatter(logging.Formatter('%(asctime)s %(levelname)-8s %(message)s %(module)5s(%(lineno)s)'))
spynelog.addHandler(spynelogHandler)
spynelog.setLevel(logging.DEBUG)

timeoutNodeReread = 300
primitiveAuth = False
soapLogin = 'login'
soapPasswd = 'passwd'
errorInCalling = 'Error at the stage of calling the load'
tns = 'http://aif.soap.exchange.artix.su'
curConfig = None
curNodes = []
lastTimeReadNodes = time.time()
lastSessionTimeActive = 0

# Нода, для которой в данный момент открыта сессия. Если приходят запросы для других магазинов пока не закрыта сессия для текущего, то не выполняем их.
sessionNode = None
sessionDictId = None

sessionIdleTimeout = 300
isCashRun = False  # Соап сервис запущен на кассе
shopCodeIsShopId = False # Является ли код магазина, который передают в запросе идентификатором магазина.
lastShopCode = None # Код магазина, который последний раз обрабатывался
maxCountBackupRequests = 0
actualCountDaysForBackupRequests = 3
useSession = False

def rereadNodes():
    nodes = None
    try:
        nodeGetter = __import__('upload.node.%s.nodeGetter' % (curConfig.getString('UPLOAD', 'nodeGetter', 'cs23')), fromlist='q')
        nodes = nodeGetter.getNodes()
    except Exception, e:
        log.warning(u'Ошибка считывания нод. %s' % str(e))
    return nodes

def checkShopCode(shopCode):
    global curNodes
    if not shopCode or shopCode.strip() == '':
        return False, shopCode
    if time.time() - lastTimeReadNodes > timeoutNodeReread:
        nodes = rereadNodes()
        if nodes:
            curNodes = nodes
    if curNodes:
        if not shopCodeIsShopId:
            ids = [n.id for n in curNodes if str(n.shopCode) == shopCode]
            if len(ids) == 1:
                shopCode = ids[0]
            else:
                log.warning(u'По коду магазина %s найдено несколько идентификаторов либо ни одного. Невозможно однозначно определить целевой магазин.' % shopCode)
                return False, shopCode
        elif not str(shopCode) in [str(node.id) for node in curNodes]:
            return False, shopCode
    elif not shopCodeIsShopId:
        log.warning(u'Невозможно найти по коду магазина его идентификатор, т.к. ноды не считаны.')
        return False, shopCode
    return True, shopCode

def getPrestartResult(shopCode):
    global lastShopCode
    dictConvertResult = soapCl.DictConvertResult()
    dictConvertResult.result = False
    dictConvertResult.info = errorInCalling
    res, shopCode = checkShopCode(shopCode)
    lastShopCode = shopCode
    if not res:
        dictConvertResult.info = 'Shop with code = %s does not exist.' % shopCode
        lastShopCode = None
    return dictConvertResult, shopCode

class SoapConverter(object):
    name = 'soap'
    dict_state_id = None
    def __init__(self, hotKeyByBarcode = False):
        self.hotKeyByBarcode = hotKeyByBarcode

class EmptyNode(object):
    def __init__(self, shopCode, realShopCode):
        self.id = shopCode
        self.shopCode=realShopCode
        self.pack_size = curConfig.getInt('QUEUE', 'aifObjectsCountInMessage', 200)
#        self.inserter = DBInserterConfig()
        self.inserter = QueueInserterConfig(queue = parameters.QueueParameters(host = curConfig.getString('QUEUE', 'host', 'localhost'), 
                                                                               port = curConfig.getInt('QUEUE', 'port', 5672), 
                                                                               username=curConfig.getString('QUEUE', 'user', 'guest'), 
                                                                               password=curConfig.getString('QUEUE', 'password', 'guest'),
                                                                               queue='artix.loaddict.cash' if isCashRun else None))
        self.currentConverter = SoapConverter()
        self.version = None

def getNode(shopCode, realShopCode, dictionaryId = None):
    global curNodes
    node = EmptyNode(shopCode, realShopCode)
    if not dictionaryId:
        dictionaryId = 'soap' + str(time.time())
    try:
        node.currentConverter.dict_state_id = rest_state.sendDictState(node, 'Start convert dict', u'Начало конвертирования справочника.', rest_state.STATUS_DURING, dictionaryId)
        for n in curNodes:
            if str(n.id) == str(node.id):
                for converter in n.converters:
                    if converter.name == 'aif':
                        node.currentConverter.hotKeyByBarcode = converter.hotKeyByBarcode
                break
    except Exception,e:
        log.warning('Con not write state to CS by REST. %s' % e)
    return node

def getMethodName(callerMethod):
    if callerMethod.startswith('add'):
        return 'addCmd'
    elif callerMethod.startswith('delete'):
        return 'deleteCmd'
    elif callerMethod.startswith('update'):
        return 'updateCmd'
    return

def startSession(shopCode, dictionaryId):
    global sessionNode
    global lastSessionTimeActive
    global sessionDictId
    realShopCode=shopCode
    result, shopCode = getPrestartResult(shopCode)
    if result.info != errorInCalling:
        log.info(result.info)
        return result
    if not useSession:
        result.result = False
        result.info = "Service is not configured for session mode."
        return result
    if sessionNode:
        result.info = 'Already opened session for shop %s' % sessionNode.id
        log.warning(result.info)
        return result
    node = getNode(shopCode, realShopCode, dictionaryId)
    log.info('Start session %s for dictionaryId %s' % (node.currentConverter.dict_state_id, dictionaryId))
    result = soap.start(node)
    if result.result:
        sessionNode = node
        lastSessionTimeActive = time.time()
        sessionDictId = dictionaryId
    return result

def callCommand(aifClazz, soapObjects, shopCode = 'All', dictionaryId = None, clear=False):
    global sessionNode
    global lastSessionTimeActive
    realShopCode=shopCode
    result, shopCode = getPrestartResult(shopCode)
    if result.info != errorInCalling:
        log.info(result.info)
        return result
    node = sessionNode
    # если пришел запрос не в сессии
    if not node:
        if useSession:
            result.result = False
            result.info = "Can not add package with dictionaryId=%s. Session is not opened" % dictionaryId
            log.warning(result.info)
            return result
        node = getNode(shopCode, realShopCode, dictionaryId)
        result = soap.start(node)
        if not result.result:
            return result
        log.info('Loading dictionary id=%s' % (dictionaryId))
    else:
        if shopCode == node.id:
            # проверим относится ли эта посылка к текущей сессии
            if sessionDictId != dictionaryId:
                result.result = False
                result.info = "Current session opened for dictionaryId=%s, but this package with dictionaryId=%s" % (sessionDictId, dictionaryId)
                log.warning(result.info)
                return result
            log.info('Add package in session %s for dictionaryId %s' % (node.currentConverter.dict_state_id, dictionaryId))
            result.result = True
            lastSessionTimeActive = time.time()
        else:
            result.info = 'Already opened session for shop %s' % sessionNode.id
            log.warning(result.info)
            return result
    result = soap.convert(soapObjects, shopCode, aifClazz, node, getMethodName(inspect.stack()[1][3]), clear) if result.result else result.result
    if not sessionNode:
        if result.result:
            result = soap.stop(node)
        else:
            soap.stop(node)
    return result

def stopSession(shopCode, dictionaryId):
    global sessionNode
    global lastSessionTimeActive
    result, shopCode = getPrestartResult(shopCode)
    if result.info != errorInCalling:
        log.info(result.info)
        return result
    if not useSession:
        result.result = False
        result.info = "Service is not configured for session mode."
        return result
    node = sessionNode
    if not node or (shopCode != node.id or sessionDictId != dictionaryId):
        result.info = 'Session for shopCode=%s and dictionaryId=%s is not open' % (shopCode, dictionaryId)
        log.warning(result.info)
        return result
    sessionNode = None
    lastSessionTimeActive = 0
    log.info('Stop session %s for dictionaryId %s' % (node.currentConverter.dict_state_id, dictionaryId))
    result = soap.stop(node)
    return result

def rollbackSession(shopCode, dictionaryId, sessionIdle=False):
    global sessionNode
    global lastSessionTimeActive
    result, shopCode = getPrestartResult(shopCode)
    if result.info != errorInCalling:
        log.info(result.info)
        return result
    if not useSession:
        result.result = False
        result.info = "Service is not configured for session mode."
        return result
    node = sessionNode
    if not node or (shopCode != node.id or sessionDictId != dictionaryId):
        result.info = 'Session for shopCode=%s and dictionaryId=%s is not open' % (shopCode, dictionaryId)
        log.warning(result.info)
        return result
    sessionNode = None
    lastSessionTimeActive = 0
    log.info('Rollback session %s for dictionaryId %s' % (node.currentConverter.dict_state_id, dictionaryId))
    result = soap.rollback(node, sessionIdle=sessionIdle)
    return result

class AifExchangeService(ServiceBase):
#    __service_name__ = 'aaa'
#    __tns__ = 'bbb'
    __namespace__ = DEFAULT_NS

    @srpc(String, String, _returns=soapCl.DictConvertResult)
    def startSession(shopCode, dictionaryId):
        return startSession(shopCode, dictionaryId)
    
    @srpc(String, String, _returns=soapCl.DictConvertResult)
    def stopSession(shopCode, dictionaryId):
        return stopSession(shopCode, dictionaryId)

    @srpc(String, String, _returns=soapCl.DictConvertResult)
    def rollbackSession(shopCode, dictionaryId):
        return rollbackSession(shopCode, dictionaryId)

    @srpc(Array(soapCl.Agerestrict), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addAgerestricts(agerestricts, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.Agerestrict, agerestricts, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.AgerestrictKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteAgerestricts(agerestrictKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.Agerestrict, agerestrictKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Barcode), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addBarcodes(barcodes, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.Barcode, barcodes, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.BarcodeKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteBarcodes(barcodeKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.Barcode, barcodeKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Button), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addButtons(buttons, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.Button, buttons, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.ButtonKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteButtons(buttonKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.Button, buttonKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Card), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addCards(cards, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.Card, cards, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.CardKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteCards(cardKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.Card, cardKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.CardGroup), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addCardGroups(cardGroups, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.CardGroup, cardGroups, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.CardGroupKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteCardGroups(cardGroupKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.CardGroup, cardGroupKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.CashUser), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addCashUsers(cashUsers, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.CashUser, cashUsers, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.CashUserKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteCashUsers(cashUserKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.CashUser, cashUserKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.Catalog), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addCatalogs(catalogs, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.Catalog, catalogs, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.CatalogKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteCatalogs(catalogKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.Catalog, catalogKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.ChangeAccountBalance), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addChangeAccountBalances(changeAccountBalances, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.ChangeAccountBalance, changeAccountBalances, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.ChangeAccountBalanceKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteChangeAccountBalances(changeAccountBalanceKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.ChangeAccountBalance, changeAccountBalanceKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.Client), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addClients(clients, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.Client, clients, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.ClientKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteClients(clientKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.Client, clientKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.Command), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addCommands(commands, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.Command, commands, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.CommandKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteCommands(commandKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.Command, commandKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.Dept), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addDepts(depts, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.Dept, depts, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.DeptKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteDepts(deptKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.Dept, deptKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.DiscountAction), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addDiscountActions(discountActions, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.DiscountAction, discountActions, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.DiscountActionKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteDiscountActions(discountActionKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.DiscountAction, discountActionKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.HotKey), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addHotKeys(hotKeys, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.HotKey, hotKeys, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.HotKeyKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteHotKeys(hotKeyKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.HotKey, hotKeyKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.Invent), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addInvents(invents, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.Invent, invents, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.InventKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteInvents(inventKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.Invent, inventKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.InventCatalog), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addInventCatalogs(inventCatalogs, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.InventCatalog, inventCatalogs, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.InventCatalog), String, String, _returns=soapCl.DictConvertResult)
    def deleteInventCatalogs(inventCatalogs, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.InventCatalog, inventCatalogs, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.InventGroup), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addInventGroups(inventGroups, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.InventGroup, inventGroups, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.InventGroupKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteInventGroups(inventGroupKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.InventGroup, inventGroupKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.KbKey), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addKbKeys(kbKeys, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.KbKey, kbKeys, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.KbKeyKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteKbKeys(kbKeyKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.KbKey, kbKeyKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.LDiscAccumulate), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addLDiscAccumulates(lDiscAccumulates, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.LDiscAccumulate, lDiscAccumulates, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.LDiscAccumulateKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteLDiscAccumulates(lDiscAccumulateKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.LDiscAccumulate, lDiscAccumulateKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.LDiscCard), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addLDiscCards(lDiscCards, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.LDiscCard, lDiscCards, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.LDiscCardKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteLDiscCards(lDiscCardKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.LDiscCard, lDiscCardKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.LDiscFixed), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addLDiscFixeds(lDiscFixeds, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.LDiscFixed, lDiscFixeds, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.LDiscFixedKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteLDiscFixeds(lDiscFixedKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.LDiscFixed, lDiscFixedKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.LDiscScheme), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addLDiscSchemes(lDiscSchemes, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.LDiscScheme, lDiscSchemes, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.LDiscSchemeKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteLDiscSchemes(lDiscSchemeKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.LDiscScheme, lDiscSchemeKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.LVat), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addLVats(lVats, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.LVat, lVats, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.LVatKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteLVats(lVatKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.LVat, lVatKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.MCashUser), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addMCashUsers(mCashUsers, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.MCashUser, mCashUsers, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.MCashUserKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteMCashUsers(mCashUserKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.MCashUser, mCashUserKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.Measure), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addMeasures(measures, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.Measure, measures, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.MeasureKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteMeasures(measureKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.Measure, measureKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.Price), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addPrices(prices, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.Price, prices, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.PriceKey), String, String, _returns=soapCl.DictConvertResult)
    def deletePrices(pricesKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.Price, pricesKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Shop), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addShops(shops, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.Shop, shops, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.ShopKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteShops(shopKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.Shop, shopKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.TaxGroup), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addTaxGroups(taxGroups, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.TaxGroup, taxGroups, shopCode, dictionaryId, clear)

#    @srpc(Array(soapCl.TaxGroupKey), String, String, _returns=soapCl.DictConvertResult)
#    def deleteTaxGroups(taxGroupKeys, shopCode = 'All', dictionaryId = None):
#        return callCommand(aif.TaxGroup, taxGroupKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Timerestrict), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addTimerestricts(timerestricts, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.Timerestrict, timerestricts, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.TimerestrictKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteTimerestricts(timerestrictKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.Timerestrict, timerestrictKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.Unit), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addUnits(units, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.Unit, units, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.UnitKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteUnits(unitKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.Unit, unitKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.Currency), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addCurrencies(currencies, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.Currency, currencies, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.CurrencyKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteCurrencies(currencyKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.Currency, currencyKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.AdditionalPrice), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addAdditionalPrices(additionalPrices, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.AdditionalPrice, additionalPrices, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.AdditionalPriceKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteAdditionalPrices(additionalPriceKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.AdditionalPrice, additionalPriceKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.DiscountsAuto), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addDiscountsAutos(discountsAutos, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.DiscountsAuto, discountsAutos, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.DiscountsAutoKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteDiscountsAutos(discountsAutoKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.DiscountsAuto, discountsAutoKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.HotKeyInvent), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addHotKeyInvents(hotKeyInvents, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.HotKeyInvent, hotKeyInvents, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.HotKeyInvent), String, String, _returns=soapCl.DictConvertResult)
    def deleteHotKeyInvents(hotKeyInvents, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.HotKeyInvent, hotKeyInvents, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.Kit), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addKits(kits, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.Kit, kits, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.KitKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteKits(kitKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.Kit, kitKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.KitDisc), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addKitDiscs(kitDiscs, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.KitDisc, kitDiscs, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.KitDiscKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteKitDiscs(kitDiscKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.KitDisc, kitDiscKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.KitInvent), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addKitInvents(kitInvents, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.KitInvent, kitInvents, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.KitInventKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteKitInvents(kitInventKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.KitInvent, kitInventKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.TimePeriod), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addTimePeriods(timePeriods, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.TimePeriod, timePeriods, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.TimePeriodKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteTimePeriods(timePeriodKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.TimePeriod, timePeriodKeys, shopCode, dictionaryId)
    
    @srpc(Array(soapCl.Consultant), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addConsultants(consultants, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.Consultant, consultants, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.ConsultantKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteConsultants(consultantKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.Consultant, consultantKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.SoftCheck), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addSoftChecks(softchecks, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.SoftCheck, softchecks, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.SoftCheckKey), String, String, _returns=soapCl.DictConvertResult)
    def deleteSoftChecks(softcheckKeys, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.SoftCheck, softcheckKeys, shopCode, dictionaryId)

    @srpc(Array(soapCl.AlcoCode), String, Boolean, String, _returns=soapCl.DictConvertResult)
    def addAlcoCodes(alcocodes, shopCode = 'All', clear=False, dictionaryId = None):
        return callCommand(aif.AlcoCode, alcocodes, shopCode, dictionaryId, clear)

    @srpc(Array(soapCl.AlcoCode), String, String, _returns=soapCl.DictConvertResult)
    def deleteAlcoCodes(alcocodes, shopCode = 'All', dictionaryId = None):
        return callCommand(aif.AlcoCode, alcocodes, shopCode, dictionaryId)

#    @srpc(Array(soapCl.InventOptions), String, String, _returns=soapCl.DictConvertResult)
#    def updateInventOptions(inventOptions, shopCode = 'All', dictionaryId = None):
#        result = getPrestartResult(shopCode)
#        if result.info != errorInCalling:
#            return result
#        node = getNode(shopCode, dictionaryId)
#        result = soap.convert(inventOptions, shopCode, aif.InventOptions, node)
#        return result

class ServerHandlerAif(ServerHandler):
    def error_output(self, environ, start_response):
        try:
            start_response(self.error_status,self.error_headers[:],sys.exc_info())
            exc_type, exc_value, exc_traceback = sys.exc_info()
            mess = traceback.format_exception_only(exc_type, exc_value)
        except Exception, e:
            print e
        result = []
        result.append('''<?xml version='1.0' encoding='UTF-8'?>''')
        result.append('''<senv:Envelope xmlns:tns="%s" xmlns:senv="http://schemas.xmlsoap.org/soap/envelope/"><senv:Body><tns:Response><tns:Result><tns:info>%s</tns:info><tns:result>false</tns:result></tns:Result></tns:Response></senv:Body></senv:Envelope>''' % (tns, mess))

        return result
        

class WSGIRequestHandlerAif(WSGIRequestHandler):
    
    timeout = 300 # connection timeout
    
    def handle(self):
        self.raw_requestline = self.rfile.readline()
        if not self.parse_request(): # An error code has been sent, just exit
            return
        if primitiveAuth:
            userAndPass = b64encode(b"%s:%s" % (soapLogin, soapPasswd)).decode("ascii")
            if self.headers.getheader('Authorization') != 'Basic %s' %  userAndPass:
                self.send_response(401, message='Authorization error.')
                return
        handler = ServerHandlerAif(
            self.rfile, self.wfile, self.get_stderr(), self.get_environ()
        )
        handler.request_handler = self      # backpointer for logging
        handler.run(self.server.get_app())
        backupRequest()

def backupRequest():
    global lastShopCode
    logDir = logger.Logger().getLogDirectory()
    logFiles = filesmodify.getFilesByMask(logDir, ['%s.*' % fileLogName])
    for f in logFiles:
        filesmodify.archive(log, os.path.dirname(f), os.path.join(logDir, 'SOAP_BACKUP', lastShopCode if lastShopCode else ''), maxCountFiles=maxCountBackupRequests, actualCountDays=actualCountDaysForBackupRequests, removeSRC = True, fileName = os.path.basename(f), loginfo=False)
    lastShopCode = None

def sessionThread():
    try:
        while True:
            if sessionNode and lastSessionTimeActive != 0 and (time.time() - lastSessionTimeActive) > sessionIdleTimeout:
                log.info('Session spoiled!!!!!!')
                shopId = sessionNode.id if shopCodeIsShopId else sessionNode.shopCode
                log.info('Call rollback for shop %s' % shopId)
                rollbackSession(shopId, sessionDictId, sessionIdle=True)
            time.sleep(2)
    except Exception, e:
        log.error(e)

def start(config, isCash = False):
    global timeoutNodeReread
    global curNodes
    global curConfig
    global lastTimeReadNodes
    global primitiveAuth
    global soapLogin
    global soapPasswd
    global sessionIdleTimeout
    global isCashRun
    global shopCodeIsShopId
    global maxCountBackupRequests
    global actualCountDaysForBackupRequests
    global useSession
    isCashRun = isCash
    curConfig = config
    timeoutNodeReread = config.getInt('UPLOAD', 'timeoutNodeReread', 300)
    lastTimeReadNodes = time.time()
    curNodes = rereadNodes()
    
    application = Application([AifExchangeService], tns, name = 'AifExchange', in_protocol=Soap11(), out_protocol=Soap11())
    maxContentLength = config.getInt('SOAP_UPLOAD', 'maxContentLength', (8*1024*1024)*10)
    primitiveAuth = config.getBool('SOAP_UPLOAD', 'authorize', False)
    soapLogin = config.getString('SOAP_UPLOAD', 'login', 'admin')
    soapPasswd = config.getString('SOAP_UPLOAD', 'passwd', 'admin')
    sessionIdleTimeout = config.getInt('SOAP_UPLOAD', 'sessionIdleTimeout', 300)
    shopCodeIsShopId = config.getBool('SOAP_UPLOAD', 'shopCodeIsShopId', False)
    maxCountBackupRequests = config.getInt('SOAP_UPLOAD', 'maxCountBackupRequests', 0)
    actualCountDaysForBackupRequests = config.getInt('SOAP_UPLOAD', 'actualCountDaysForBackupRequests', 3)
    useSession = config.getBool('SOAP_UPLOAD', 'useSession', False)
    
    blockLength=10*1024*1024 # 10 MB
    if blockLength > maxContentLength:
        blockLength = maxContentLength
#    twisted_apps = [
#        (WsgiApplication(application, chunked=True, max_content_length=maxContentLength, block_length=blockLength), 'AifExchange'),
#    ]
    wsgi_app = WsgiApplication(application, max_content_length=maxContentLength, block_length=blockLength)
    port = config.getInt('SOAP_UPLOAD', 'port', 18080)
    from wsgiref.simple_server import make_server
    server = make_server('0.0.0.0', port, wsgi_app, handler_class=WSGIRequestHandlerAif)
    log.info("Starting exchangeUploadSoapService")
    thread = threading.Thread(target=sessionThread)
    thread.daemon = True
    thread.start()
    server.serve_forever()
    
#    run_twisted(twisted_apps, port)
    log.info("Stoped exchangeUploadSoapService")

