
import urllib2
import json, os, re, sys
from upload.node.node import Node, AifConverterParameters, SmagConverterParameters, QueueInserterConfig, QueueParameters, SmagUkm4ConverterParameters, OracleConverterParameters, ShtrihConverterParameters, AtolConverterParameters, ConverterParameters, CrystalConverterParameters
from common import macros, logger, config
log = logger.Logger().getLoggerUpload("main")

def getNodes():
    queue_host = 'localhost' 
    queue_port = 5672
    queue_user = 'guest'
    queue_password = 'guest'
    aifObjectsCountInMessage = 200
    if not config.read(sys.path[0] + "/../config/%s/exchangers.ini" % (logger.Logger().location)):
        log.warning("Configuration file 'exchangers.ini' not found. Configure queue by default...")
    else:
        queue_host = config.getString('QUEUE', 'host', queue_host) 
        queue_port = config.getInt('QUEUE', 'port', queue_port)
        queue_user = config.getString('QUEUE', 'user', queue_user)
        queue_password = config.getString('QUEUE', 'password', queue_password)
        aifObjectsCountInMessage = config.getInt('QUEUE', 'aifObjectsCountInMessage', aifObjectsCountInMessage)
        
    response = urllib2.urlopen('http://localhost:8080/cashinfo/rest/shops')
    
    html = response.read()
    
    shops = json.loads(html)
    nodes = []
    for shop in shops:
        shopCode = str(shop.get('code'))
        flags = [ f.strip() for f in re.split('[,;]', macros.applyMacros(str(shop.get('dictFlag')), log, shopcode = shopCode)) ]
        data = [ d.strip() for d in re.split('[,;]', macros.applyMacros(str(shop.get('dictDir')), log, shopcode = shopCode)) ]
        dictExchangeType = shop.get('dictExchangeType')
        if not dictExchangeType:
            continue
        elif (dictExchangeType == 'DEFAULT'):
            converter = ConverterParameters(flags = [], data = [])
        elif (dictExchangeType == 'ARTIXINTERCHANGE'):
            converter = AifConverterParameters(flags = flags, data = data)
        elif (dictExchangeType == 'SHTRIHM'):
            delimiter = ';'
            if shop.has_key('delimiter'):
                delimiter = str(shop.get('delimiter'))
            converter = ShtrihConverterParameters(flags = flags, data = data, delimiter = delimiter)
        elif (dictExchangeType == 'ATOL'):
            converter = AtolConverterParameters(flags = flags, data = data)
        elif (dictExchangeType.startswith('SUPERMAG')):
            charset = '866'
            if shop.has_key('charset'):
                charset = str(shop.get('charset'))
            format = 'dat'
            if shop.has_key('format'):
                format = str(shop.get('format'))
            discount = 'modern'
            if shop.has_key('discount'):
                discount = str(shop.get('discount'))
            for i in range(len(data)):
                data[i] = '%s/*.%s' % (data[i], format)
            if (dictExchangeType == 'SUPERMAG'):
                converter = SmagConverterParameters(flags = flags, data = data, charset = charset, format = format)
            elif (dictExchangeType == 'SUPERMAG4'):
                converter = SmagUkm4ConverterParameters(flags = flags, data = data, charset = charset, format = format, discount = discount)
        elif (dictExchangeType == 'ORACLE'):
            converter = OracleConverterParameters(flags = flags, data = data)
        elif (dictExchangeType == 'CRYSTAL'):
            visualVerifySG = ''
            requireQuantityManualSG = ''
            measureMapper = ''
            if shop.has_key('visualverifysg'):
                visualVerifySG = str(shop.get('visualverifysg'))
            if shop.has_key('requirequantitymanualsg'):
                requireQuantityManualSG = str(shop.get('requirequantitymanualsg'))
            if shop.has_key('measuremap'):
                measureMapper = str(shop.get('measuremap'))
            converter = CrystalConverterParameters(flags = flags, data = data, 
                                                        visualVerifySG=visualVerifySG, 
                                                        requireQuantityManualSG=requireQuantityManualSG,
                                                        measureMapper = measureMapper)

        nodes.append(
                    Node(id = shopCode,
                        backup_dir = str(macros.applyMacros(str(shop.get('backupDir')), log, shopcode = shopCode)),
                        converters = [converter],
                        pack_size = aifObjectsCountInMessage,
                        inserter = QueueInserterConfig(queue = QueueParameters(host = queue_host, port = queue_port, username=queue_user, password=queue_password)),
                        badDictsDirectory = macros.applyMacros(config.getString('UPLOAD', 'badDictsDirectory', '/opt/exchangers/badDicts').strip(), log, shopcode = shopCode))
                    )
    return nodes
