# -*- coding: utf-8 -*-

import httplib2
import json, sys, re
from upload.node.node import Node, AifConverterParameters, SmagConverterParameters, QueueInserterConfig, QueueParameters, SmagUkm4ConverterParameters, OracleConverterParameters, ShtrihConverterParameters, AtolConverterParameters, CrystalConverterParameters, SimpleFileConverterParameters
from common import macros, logger, config
log = logger.Logger().getLoggerUpload("main")

jsonHeader = {'Content-Type':'application/json', "Accept": "application/json"}
http = httplib2.Http()


def str2bool(v):
    return str(v).lower() in ("yes", "true", "t", "1")


def getNodes():
    queue_host = 'localhost' 
    queue_port = 5672
    queue_user = 'guest'
    queue_password = 'guest'
    aifObjectsCountInMessage = 200
    if not config.read(sys.path[0] + "/../config/%s/exchangers.ini" % (logger.Logger().location)):
        log.warning("Configuration file 'exchangers.ini' not found. Configure queue by default...")
    else:
        queue_host = config.getString('QUEUE', 'host', queue_host) 
        queue_port = config.getInt('QUEUE', 'port', queue_port)
        queue_user = config.getString('QUEUE', 'user', queue_user)
        queue_password = config.getString('QUEUE', 'password', queue_password)
        aifObjectsCountInMessage = config.getInt('QUEUE', 'aifObjectsCountInMessage', aifObjectsCountInMessage)

    nodes = []
    
    response, content = http.request('http://localhost:8080/CSrest/rest/shops', 'GET', headers=jsonHeader)
    if response.status == 200:
        shops = json.loads(content)
        for shop in shops:
            shopCode = str(shop.get('shopCode'))
            shopId = str(shop.get('id'))
            clusterId = str(shop.get('clusterId', ''))
            flags = macros.applyMacros(str(shop.get('exchange', {}).get('params', {}).get('flagPaths', '')), log, shopcode = shopCode, shopid = shopId, clusterid = clusterId)
            flags = [ f.strip() for f in re.split('[,;]', flags) ]
            data = macros.applyMacros(str(shop.get('exchange', {}).get('params', {}).get('dictPaths', '')), log, shopcode = shopCode, shopid = shopId, clusterid = clusterId)
            data = [ d.strip() for d in re.split('[,;]', data) ]
            
            backupPath = str(shop.get('exchange', {}).get('params', {}).get('backupDir', '../backup/dicts/%(clusterid)s/%(shopcode)s'))
            backupDir = macros.applyMacros(backupPath, log, shopcode = shopCode, shopid = shopId, clusterid = clusterId)
            
            dictExchangeType = shop.get('exchange', {}).get('type', 'DEFAULT')
            if (dictExchangeType == 'DEFAULT'):
                continue
            elif (dictExchangeType == 'ARTIXINTERCHANGE'):
                hotKeyByBarcode = str2bool(shop.get('exchange', {}).get('params', {}).get('hotKeyByBarcode', False))
                converter = AifConverterParameters(flags=flags, data=data, hotKeyByBarcode=hotKeyByBarcode)
            elif (dictExchangeType == 'SHTRIHM'):
                delimiter = shop.get('exchange', {}).get('params', {}).get('delimiter', ';')
                converter = ShtrihConverterParameters(flags = flags, data = data, delimiter = delimiter)
            elif (dictExchangeType == 'ATOL'):
                converter = AtolConverterParameters(flags = flags, data = data)
            elif (dictExchangeType.startswith('SUPERMAG')):
                charset = str(shop.get('exchange', {}).get('params', {}).get('charset', '866'))
                format = str(shop.get('exchange', {}).get('params', {}).get('format', 'dat'))
                discount = str(shop.get('exchange', {}).get('params', {}).get('discount', 'modern'))
                for i in range(len(data)):
                    data[i] = '%s/*.%s' % (data[i], format)
                if (dictExchangeType == 'SUPERMAG'):
                    converter = SmagConverterParameters(flags = flags, data = data, charset = charset, format = format)
                elif (dictExchangeType == 'SUPERMAG4'):
                    defaultRoleCode = shop.get('exchange', {}).get('params', {}).get('defaultRoleCode', 3)
                    additionallyRoleCode = shop.get('exchange', {}).get('params', {}).get('additionallyRoleCode', 2)
                    usersWithAdditionallyRole = str(shop.get('exchange', {}).get('params', {}).get('usersWithAdditionallyRole', ''))
                    relationRolesBy = str(shop.get('exchange', {}).get('params', {}).get('relationRolesBy', 'name'))
                    substringInAdminName = shop.get('exchange', {}).get('params', {}).get('substringInAdminName', u'Администратор').encode("utf8")
                    usersWithAdditionallyRole = usersWithAdditionallyRole.split(',')
                    converter = SmagUkm4ConverterParameters(flags=flags,
                                                            data=data,
                                                            charset=charset,
                                                            format=format,
                                                            discount=discount,
                                                            defaultRoleCode=defaultRoleCode,
                                                            additionallyRoleCode=additionallyRoleCode,
                                                            usersWithAdditionallyRole=usersWithAdditionallyRole,
                                                            relationRolesBy=relationRolesBy,
                                                            substringInAdminName=substringInAdminName)
            elif (dictExchangeType == 'ORACLE'):
                converter = OracleConverterParameters(flags = flags, data = data)
            elif (dictExchangeType == 'CRYSTAL'):
                visualVerifySG = str(shop.get('exchange', {}).get('params', {}).get('visualverifysg', ''))
                requireQuantityManualSG = str(shop.get('exchange', {}).get('params', {}).get('requirequantitymanualsg', ''))
                measureMapper = shop.get('exchange', {}).get('params', {}).get('measuremap', '')
                converter = CrystalConverterParameters(flags = flags, data = data, visualVerifySG=visualVerifySG, 
                                                       requireQuantityManualSG=requireQuantityManualSG, measureMapper = measureMapper)
            elif (dictExchangeType == 'SIMPLE-FILE'):
                cashPath = shop.get('exchange', {}).get('params', {}).get('cashPath', '/tmp/upload_files/dataFile.xml')
                cashPath = macros.applyMacros(cashPath, log, shopcode = shopCode, shopid = shopId, clusterid = clusterId)
                converter = SimpleFileConverterParameters(flags = flags, data = data, cashPath = cashPath, cashCodes = shop.get("cashes", []))
            else:
                log.warning("Exchange type %s not suported. shopId = %s" % (dictExchangeType, shopId))
                continue
            queueName = 'artix.loaddict.%s' % (shopId)
            if clusterId != '':
                queueName = queueName + '.' + clusterId
            nodes.append(
                         Node(id = shopId,
                              backup_dir = backupDir,
                              converters = [converter],
                              pack_size = aifObjectsCountInMessage,
                              inserter = QueueInserterConfig(queue = QueueParameters(host = queue_host, port = queue_port, username=queue_user, password=queue_password, queue=queueName)),
                              badDictsDirectory = macros.applyMacros(config.getString('UPLOAD', 'badDictsDirectory', '/opt/exchangers/badDicts').strip(), log, shopcode = shopCode, shopid = shopId, clusterid = clusterId),
                              shopCode = shopCode)
                         )
    return nodes
