# -*- coding: utf-8 -*-

import httplib2
import json, sys, re
from upload.node.node import Node, AifConverterParameters, SmagConverterParameters, QueueInserterConfig, QueueParameters, SmagUkm4ConverterParameters, OracleConverterParameters, ShtrihConverterParameters, AtolConverterParameters, CrystalConverterParameters, SimpleFileConverterParameters
from common import macros, logger, config
log = logger.Logger().getLoggerUpload("main")

jsonHeader = {'Content-Type':'application/json', "Accept": "application/json"}
http = httplib2.Http()


def str2bool(v):
    return str(v).lower() in ("yes", "true", "t", "1")


def getNodes():
    queue_host = 'localhost' 
    queue_port = 5672
    queue_user = 'guest'
    queue_password = 'guest'
    aifObjectsCountInMessage = 200
    if not config.read(sys.path[0] + "/../config/%s/exchangers.ini" % (logger.Logger().location)):
        log.warning("Configuration file 'exchangers.ini' not found. Configure queue by default...")
    else:
        queue_host = config.getString('QUEUE', 'host', queue_host) 
        queue_port = config.getInt('QUEUE', 'port', queue_port)
        queue_user = config.getString('QUEUE', 'user', queue_user)
        queue_password = config.getString('QUEUE', 'password', queue_password)
        aifObjectsCountInMessage = config.getInt('QUEUE', 'aifObjectsCountInMessage', aifObjectsCountInMessage)

    nodes = []
    
    response, content = http.request('http://localhost:8080/CSrest/rest/shops', 'GET', headers=jsonHeader)
    if response.status == 200:
        shops = json.loads(content)
        for shop in shops:
            converters = []
            shopCode = str(shop.get('shopCode'))
            shopId = str(shop.get('id'))
            clusterId = str(shop.get('clusterId', ''))
            for exchange in shop.get('exchanges', []):
                if not exchange:
                    continue
                flags = macros.applyMacros(str(exchange.get('params', {}).get('flagPaths', '')), log, shopcode = shopCode, shopid = shopId, clusterid = clusterId)
                flags = [ f.strip() for f in re.split('[,;]', flags) ]
                data = macros.applyMacros(str(exchange.get('params', {}).get('dictPaths', '')), log, shopcode = shopCode, shopid = shopId, clusterid = clusterId)
                data = [ d.strip() for d in re.split('[,;]', data) ]

                backupPath = str(exchange.get('params', {}).get('backupDir', '../backup/dicts/%(clusterid)s/%(shopcode)s'))
                backupDir = macros.applyMacros(backupPath, log, shopcode = shopCode, shopid = shopId, clusterid = clusterId)
                
                exchangeType = exchange.get('type', 'DEFAULT')
                if (exchangeType == 'DEFAULT'):
                    continue
                elif (exchangeType == 'ARTIXINTERCHANGE'):
                    hotKeyByBarcode = str2bool(exchange.get('params', {}).get('hotKeyByBarcode', False))
                    converters.append(AifConverterParameters(flags=flags, data=data, backup_dir=backupDir, hotKeyByBarcode=hotKeyByBarcode))
                elif (exchangeType == 'SHTRIHM'):
                    delimiter = exchange.get('params', {}).get('delimiter', ';')
                    converters.append(ShtrihConverterParameters(flags = flags, data = data, backup_dir = backupDir, delimiter = delimiter))
                elif (exchangeType == 'ATOL'):
                    converters.append(AtolConverterParameters(flags = flags, data = data, backup_dir = backupDir))
                elif (exchangeType.startswith('SUPERMAG')):
                    charset = str(exchange.get('params', {}).get('charset', '866'))
                    format = str(exchange.get('params', {}).get('format', 'dat'))
                    discount = str(exchange.get('params', {}).get('discount', 'modern'))
                    catalogBindCode =  str(exchange.get('params', {}).get('catalogBindCode', ''))
                    groupBindCode =  str(exchange.get('params', {}).get('groupBindCode', ''))
                    for i in range(len(data)):
                        data[i] = '%s/*.%s' % (data[i], format)
                    if (exchangeType == 'SUPERMAG'):
                        converters.append(SmagConverterParameters(flags = flags,
                                                                  data = data,
                                                                  backup_dir = backupDir,
                                                                  charset = charset,
                                                                  format = format,
                                                                  catalogBindCode=catalogBindCode,
                                                                  groupBindCode=groupBindCode))
                    elif (exchangeType == 'SUPERMAG4'):
                        defaultRoleCode = exchange.get('params', {}).get('defaultRoleCode', 3)
                        additionallyRoleCode = exchange.get('params', {}).get('additionallyRoleCode', 2)
                        usersWithAdditionallyRole = str(exchange.get('params', {}).get('usersWithAdditionallyRole', ''))
                        relationRolesBy = str(exchange.get('params', {}).get('relationRolesBy', 'name'))
                        substringInAdminName = exchange.get('params', {}).get('substringInAdminName', u'Администратор').encode("utf8")
                        usersWithAdditionallyRole = usersWithAdditionallyRole.split(',')
                        converters.append(SmagUkm4ConverterParameters(flags=flags,
                                                                      data=data,
                                                                      backup_dir=backupDir,
                                                                      charset=charset,
                                                                      format=format,
                                                                      discount=discount,
                                                                      defaultRoleCode=defaultRoleCode,
                                                                      additionallyRoleCode=additionallyRoleCode,
                                                                      usersWithAdditionallyRole=usersWithAdditionallyRole,
                                                                      relationRolesBy=relationRolesBy,
                                                                      substringInAdminName=substringInAdminName,
                                                                      catalogBindCode=catalogBindCode,
                                                                      groupBindCode=groupBindCode))
                elif (exchangeType == 'ORACLE'):
                    converters.append(OracleConverterParameters(flags = flags, data = data, backup_dir = backupDir))
                elif (exchangeType == 'CRYSTAL'):
                    visualVerifySG = str(exchange.get('params', {}).get('visualverifysg', ''))
                    requireQuantityManualSG = str(exchange.get('params', {}).get('requirequantitymanualsg', ''))
                    measureMapper = exchange.get('params', {}).get('measuremap', '')
                    converters.append(CrystalConverterParameters(flags = flags, data = data, backup_dir = backupDir, visualVerifySG=visualVerifySG, 
                                                           requireQuantityManualSG=requireQuantityManualSG, measureMapper = measureMapper))
                elif (exchangeType == 'SIMPLE-FILE'):
                    cashPath = exchange.get('params', {}).get('cashPath', '/tmp/upload_files/dataFile.xml')
                    cashPath = macros.applyMacros(cashPath, log, shopcode = shopCode, shopid = shopId, clusterid = clusterId)
                    converters.append(SimpleFileConverterParameters(flags = flags, data = data, backup_dir = backupDir, cashPath = cashPath, cashCodes = shop.get("cashes", [])))
                else:
                    log.warning("Exchange type %s not suported. shopId = %s" % (dictExchangeType, shopId))
                    continue
            queueName = 'artix.loaddict.%s' % (shopId)
            if clusterId != '':
                queueName = queueName + '.' + clusterId
            nodes.append(
                         Node(id = shopId,
                              backup_dir = None,
                              converters = converters,
                              pack_size = aifObjectsCountInMessage,
                              inserter = QueueInserterConfig(queue = QueueParameters(host = queue_host, port = queue_port, username=queue_user, password=queue_password, queue=queueName)),
                              badDictsDirectory = macros.applyMacros(config.getString('UPLOAD', 'badDictsDirectory', '/opt/exchangers/badDicts').strip(), log, shopcode = shopCode, shopid = shopId, clusterid = clusterId),
                              shopCode = shopCode)
                         )
    return nodes
